/*
 * Copyright (C) 2011 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "DFGOSRExit.h"

#if ENABLE(DFG_JIT)

#include "AssemblyHelpers.h"
#include "DFGGraph.h"
#include "DFGMayExit.h"
#include "DFGSpeculativeJIT.h"
#include "JSCellInlines.h"
#include "Operations.h"

namespace JSC { namespace DFG {

OSRExit::OSRExit(ExitKind kind, JSValueSource jsValueSource, MethodOfGettingAValueProfile valueProfile, SpeculativeJIT* jit, unsigned streamIndex, unsigned recoveryIndex)
	: OSRExitBase(kind, jit->m_origin.forExit, jit->m_origin.semantic)
	, m_jsValueSource(jsValueSource)
	, m_valueProfile(valueProfile)
	, m_recoveryIndex(recoveryIndex)
	, m_streamIndex(streamIndex)
	, m_willArriveAtOSRExitFromGenericUnwind(false)
	, m_isExceptionHandler(false)
{
	bool canExit = jit->m_origin.exitOK;
	if (!canExit && jit->m_currentNode) {
		ExitMode exitMode = mayExit(jit->m_jit.graph(), jit->m_currentNode);
		canExit = exitMode == ExitMode::Exits || exitMode == ExitMode::ExitsForExceptions;
	}
	//DFG_ASSERT(jit->m_jit.graph(), jit->m_currentNode, canExit);
}

CodeLocationJump OSRExit::codeLocationForRepatch() const
{
    return CodeLocationJump(m_patchableJumpLocation);
}

} } // namespace JSC::DFG

#endif // ENABLE(DFG_JIT)
