/*
 * Copyright (C) 2010 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ThunkGenerators_h
#define ThunkGenerators_h

#include "CodeSpecializationKind.h"
#include "RegisterPreservationMode.h"
#include "ThunkGenerator.h"

#if ENABLE(JIT)
namespace JSC {

class CallLinkInfo;

MacroAssemblerCodeRef callToJavaScript(JSGlobalData*);
MacroAssemblerCodeRef throwNotCaught(JSGlobalData*);

MacroAssemblerCodeRef throwExceptionFromCallSlowPathGenerator(JSGlobalData*);

MacroAssemblerCodeRef linkCallThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef linkPolymorphicCallThunkGenerator(JSGlobalData*);

MacroAssemblerCodeRef virtualThunkFor(JSGlobalData*, CallLinkInfo&);

MacroAssemblerCodeRef nativeCallGenerator(JSGlobalData*);
MacroAssemblerCodeRef nativeConstructGenerator(JSGlobalData*);
MacroAssemblerCodeRef arityFixupGenerator(JSGlobalData*);
MacroAssemblerCodeRef stringGetByValGenerator(JSGlobalData*);

MacroAssemblerCodeRef charCodeAtThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef charAtThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef fromCharCodeThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef absThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef ceilThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef expThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef floorThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef logThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef roundThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef sqrtThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef imulThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef randomThunkGenerator(JSGlobalData*);
MacroAssemblerCodeRef truncThunkGenerator(JSGlobalData*);
}
#endif // ENABLE(JIT)

#endif // ThunkGenerator_h
