/*
 *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
 *  Copyright (C) 2008 Apple Inc. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifndef ErrorInstance_h
#define ErrorInstance_h

#include "Interpreter.h"
#include "JSObject.h"
#include "RuntimeType.h"
#include "SourceProvider.h"
#include <wtf/Vector.h>

namespace JSC {

class ErrorInstance : public JSNonFinalObject {
public:
	typedef JSNonFinalObject Base;

    enum SourceTextWhereErrorOccurred { FoundExactSource, FoundApproximateSource };
    typedef String (*SourceAppender) (const String& originalMessage, const String& sourceText, RuntimeType, SourceTextWhereErrorOccurred);

    DECLARE_INFO;

	static Structure* createStructure(JSGlobalData& globalData, JSGlobalObject* globalObject, JSValue prototype)
	{
		return Structure::create(globalData, globalObject, prototype, TypeInfo(ErrorInstanceType, StructureFlags), info());
	}

    static ErrorInstance* create(ExecState* exec, JSGlobalData& vm, Structure* structure, const String& message, SourceAppender appender = 0, RuntimeType type = TypeNothing, bool useCurrentFrame = true)
    {
		ErrorInstance* instance = new (NotNull, allocateCell<ErrorInstance>(vm.heap)) ErrorInstance(vm, structure);
		instance->m_sourceAppender = appender;
		instance->m_runtimeTypeForCause = type;
		instance->finishCreation(exec, vm, message, useCurrentFrame);
		return instance;
	}

	static ErrorInstance* create(ExecState* exec, Structure* structure, JSValue message, SourceAppender appender = 0, RuntimeType type = TypeNothing, bool useCurrentFrame = true)
	{
		return create(exec, exec->globalData(), structure, message.isUndefined() ? String() : message.toString(exec)->value(exec), appender, type, useCurrentFrame);
	}

    static void addErrorInfo(ExecState*, JSGlobalData&, JSObject*, bool = true);

    bool hasSourceAppender() const { return !!m_sourceAppender; }
	SourceAppender sourceAppender() const { return m_sourceAppender; }
	void setSourceAppender(SourceAppender appender) { m_sourceAppender = appender; }
	void clearSourceAppender() { m_sourceAppender = 0; }
	void setRuntimeTypeForCause(RuntimeType type) { m_runtimeTypeForCause = type; }
	RuntimeType runtimeTypeForCause() const { return m_runtimeTypeForCause; }
	void clearRuntimeTypeForCause() { m_runtimeTypeForCause = TypeNothing; }

protected:
	explicit ErrorInstance(JSGlobalData&, Structure*);

	void finishCreation(ExecState*, JSGlobalData& globalData, const String& message, bool useCurrentFrame = true);

    SourceAppender m_sourceAppender;
    RuntimeType m_runtimeTypeForCause;
};

} // namespace JSC

#endif // ErrorInstance_h
