
#ifndef JSStringInlines_h
#define JSStringInlines_h

#include "ExceptionHelpers.h"
#include "JSString.h"

namespace JSC {

inline bool JSString::equal(ExecState* exec, JSString* other) const
{
    if (isRope() || other->isRope())
        return equalSlowCase(exec, other);
    return WTF::equal(*m_value.impl(), *other->m_value.impl());
}

template <typename CharacterType>
inline JSString* repeatCharacter(VM& vm, ExecState* exec, CharacterType character, unsigned repeatCount)
{
    CharacterType* buffer = 0;
    RefPtr<StringImpl> impl = StringImpl::tryCreateUninitialized(repeatCount, buffer);
    if (UNLIKELY(!impl)) {
        throwOutOfMemoryError(vm, exec);
        return 0;
    }

    // std::fill_n(buffer, repeatCount, character);
    for(int i = 0; i < repeatCount; ++i)
    	buffer[i] = character;

    return jsString(vm, impl.release());
}

} // namespace JSC

#endif // #ifndef JSStringInlines_h
