/*
 * Copyright (C) 2010 Espial Group Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Espial Group Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MemoryMonitor_h
#define MemoryMonitor_h

#include <stdlib.h>

namespace JSC
{
class ExecState;
class JSGlobalObject;
}

using JSC::JSGlobalObject;

namespace WTF
{

extern bool gLowMemory;

void setLowMemory(bool low);

typedef bool (*jsMmapChecker)(JSC::ExecState*, JSGlobalObject* jsGlobalObject, unsigned allocatedSize);

class MmapMonitor
{
public:
	static MmapMonitor& instance();

	void mmapStat(void* p, unsigned size);
	void munmapStat(void* p, unsigned size);
	
	ALWAYS_INLINE unsigned allocatedSize() const { return m_allocatedBytes; }

	ALWAYS_INLINE unsigned capSize() const { return m_capBytes; }

	ALWAYS_INLINE unsigned peakSize() const { return m_peakBytes; }
	
	ALWAYS_INLINE bool canGrow(JSC::ExecState* exec, JSGlobalObject* jsGlobalObject, unsigned size)
	{
		return !m_capBytes ||
			(m_allocatedBytes + size) <= m_capBytes ||
			m_mmapChecker(exec, jsGlobalObject, m_allocatedBytes);
	}

	ALWAYS_INLINE bool canGrow(unsigned size)
	{
		return !m_capBytes || (m_allocatedBytes + size) <= m_capBytes;
	}

	ALWAYS_INLINE void setChecker(jsMmapChecker mmapChecker, unsigned capBytes)
	{
		m_mmapChecker = mmapChecker;
		m_capBytes = capBytes;
	}
	
	ALWAYS_INLINE void resetPeakSize() { m_peakBytes = 0; }

	static bool shouldDeferGC(size_t& bytesAllocatedLimit);
	
	static void didGC(double lastGCLength, size_t sizeAfterLastCollect, size_t bytesAllocatedLimit, int opaqueRootCount);
	static void didDeleteAllCodeBlocks(bool all, int codeBlocks, int deletedBytes, int targetDeleteBytes);

private:
	MmapMonitor()
	: m_allocatedBytes(0)
	, m_peakBytes(0)
	, m_capBytes(0)
	, m_mmapChecker(0)
	{
	}

	static MmapMonitor gInstance;

	unsigned m_allocatedBytes;
	unsigned m_peakBytes;
	unsigned m_capBytes;
	jsMmapChecker m_mmapChecker;
};

}

#endif //MemoryMonitor_h
