/*
 * Copyright (C) 2008, 2010 Apple Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 *
 */

#include "config.h"
#include "WTFThreadData.h"

#include <wtf/text/AtomicStringTable.h>

namespace WTF {

#if WTFTHREADDATA_MULTITHREADED
ThreadSpecific<WTFThreadData>* WTFThreadData::staticData;
#else
WTFThreadData* WTFThreadData::staticData;
#endif

// If TLS is available, we also store a copy
// of the per-thread object in a __thread variable
// since __thread variables are faster to read
// than pthread_getspecific().  We still need
// pthread_setspecific() because __thread
// variables provide no way to run cleanup
// code when a thread is destroyed.
#if WTFTHREADDATA_MULTITHREADED_USE_TLS
static __thread WTFThreadData* threadlocal_WTFThreadData;
#endif

WTFThreadData::WTFThreadData()
    : m_currentAtomicStringTable(0)
	, m_defaultAtomicStringTable(0)
    , m_atomicStringTableDestructor(0)
    , m_stackBounds(StackBounds::currentThreadStackBounds())
#if ENABLE(STACK_STATS)
    , m_stackStats()
#endif
{
	AtomicStringTable::create(*this);
	m_currentAtomicStringTable = m_defaultAtomicStringTable;

#if WTFTHREADDATA_MULTITHREADED_USE_TLS
	threadlocal_WTFThreadData = this;
#endif
}

WTFThreadData::~WTFThreadData()
{
#if WTFTHREADDATA_MULTITHREADED_USE_TLS
	threadlocal_WTFThreadData = 0;
#endif

    if (m_atomicStringTableDestructor)
        m_atomicStringTableDestructor(m_defaultAtomicStringTable);
}

#if WTFTHREADDATA_MULTITHREADED_USE_TLS
WTFThreadData* tlsWTFThreadData()
{
	return threadlocal_WTFThreadData;
}
#endif

} // namespace WTF

