/*
 * Copyright (C) 2016 Metrological Group B.V.
 * Copyright (C) 2016 Igalia S.L.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "MediaKeyMessageEvent.h"

#if ENABLE(ENCRYPTED_MEDIA)

#include "EventNames.h"

namespace WebCore {

MediaKeyMessageEvent::MediaKeyMessageEvent()
: m_messageType(MediaKeyMessageType::LicenseRequest)
{
}

MediaKeyMessageEventInit::MediaKeyMessageEventInit()
: messageType(MediaKeyMessageType::LicenseRequest)
{
}

MediaKeyMessageEventInit::MediaKeyMessageEventInit(bool bubbles, bool cancelable, bool composed, Optional<MediaKeyMessageType> messageType, RefPtr<JSC::ArrayBuffer> message)
: EventInit(bubbles, cancelable, composed)
, messageType(messageType.valueOr(MediaKeyMessageType::LicenseRequest))
, message(message)
{
}

MediaKeyMessageEvent::MediaKeyMessageEvent(const AtomicString& type, const MediaKeyMessageEventInit& initializer)
    : Event(type, initializer)
    , m_messageType(initializer.messageType)
    , m_message(initializer.message)
{
}

MediaKeyMessageEvent::~MediaKeyMessageEvent()
{
}

EventInterface MediaKeyMessageEvent::eventInterface() const
{
    return MediaKeyMessageEventInterfaceType;
}

} // namespace WebCore

#endif // ENABLE(ENCRYPTED_MEDIA)
