/*
 * Copyright (C) 2016 Metrological Group B.V.
 * Copyright (C) 2016 Igalia S.L.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef MediaKeys_h
#define MediaKeys_h

#if ENABLE(ENCRYPTED_MEDIA)

#include "ExceptionCode.h"
#include "GenericTaskQueue.h"
#include "JSDOMPromise.h"
#include "MediaKeySessionType.h"
#include "SharedBuffer.h"
#include <wtf/Deque.h>
#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>

namespace WebCore {

class CDM;
class CDMInstance;
class BufferSource;
class MediaKeySession;

class MediaKeys : public RefCounted<MediaKeys> {
public:
    //using KeySessionType = MediaKeySessionType;

    static PassRefPtr<MediaKeys> create(bool useDistinctiveIdentifier, bool persistentStateAllowed, const Vector<MediaKeySessionType>& supportedSessionTypes, RefPtr<CDM> implementation, RefPtr<CDMInstance> instance);

    ~MediaKeys();

    RefPtr<MediaKeySession> createSession(ScriptExecutionContext*, const String&, ExceptionCode&);

    typedef DOMPromise<bool, ExceptionCode> PromiseBool;
    void setServerCertificate(const BufferSource&, PromiseBool);

    CDMInstance* cdmInstance() const { return m_instance.get(); }

protected:
    MediaKeys(bool useDistinctiveIdentifier, bool persistentStateAllowed, const Vector<MediaKeySessionType>&, RefPtr<CDM>, RefPtr<CDMInstance>);

    WeakPtr<MediaKeys> createWeakPtr() { return m_weakPtrFactory.createWeakPtr(this); }

    class Task : public GenericTask<MediaKeys>
	{
	public:
    	Task(MediaKeys*, RefPtr<SharedBuffer> certificate, PromiseBool promise);
    	virtual ~Task();
    	virtual void exec();

	private:
        RefPtr<SharedBuffer> certificate;
        PromiseBool promise;
	};

    bool m_useDistinctiveIdentifier;
    bool m_persistentStateAllowed;
    Vector<MediaKeySessionType> m_supportedSessionTypes;
    RefPtr<CDM> m_implementation;
    RefPtr<CDMInstance> m_instance;

    WeakPtrFactory<MediaKeys> m_weakPtrFactory;
    GenericTaskQueue<MediaKeys> m_taskQueue;
};

} // namespace WebCore

#endif // ENABLE(ENCRYPTED_MEDIA)

#endif // MediaKeys_h

