/*
 * Copyright (C) 2012 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef SpeechRecognitionAlternative_h
#define SpeechRecognitionAlternative_h

#if ENABLE(SCRIPTED_SPEECH)

#include "PlatformString.h"
#include <wtf/RefCounted.h>

namespace WebCore {

class ScriptExecutionContext;

class SpeechRecognitionAlternative : public RefCounted<SpeechRecognitionAlternative> {
public:
    static PassRefPtr<SpeechRecognitionAlternative> create(const String&, const String&, double);

    const String& transcript() const { return m_transcript; }
    const String& interpretation() const { return m_interpretation; }
    double confidence() const { return m_confidence; }

private:
    SpeechRecognitionAlternative(const String&, const String&, double);

    String m_transcript;
    String m_interpretation;
    double m_confidence;
};

inline PassRefPtr<SpeechRecognitionAlternative> SpeechRecognitionAlternative::create(const String& transcript, const String& interpretation, double confidence)
{
    return adoptRef(new SpeechRecognitionAlternative(transcript, interpretation, confidence));
}

inline SpeechRecognitionAlternative::SpeechRecognitionAlternative(const String& transcript, const String& interpretation, double confidence)
: m_transcript(transcript)
, m_interpretation(interpretation)
, m_confidence(confidence)
{
}

} // namespace WebCore

#endif // ENABLE(SCRIPTED_SPEECH)

#endif // SpeechRecognitionAlternative_h
