/*
 * Copyright (C) 2016 Canon Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted, provided that the following conditions
 * are required to be met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Canon Inc. nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CANON INC. AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CANON INC. AND ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "JSURLSearchParams.h"

#if ENABLE(FETCH_API)

#include "JSKeyValueIterator.h"
#include "CachedCall.h"

using namespace JSC;

namespace WebCore {

// FIXME: Move this code to JSURLSearchParams.
typedef JSKeyValueIterator<JSURLSearchParams, URLSearchParams> URLSearchParamsIterator;
typedef JSKeyValueIteratorPrototype<JSURLSearchParams, URLSearchParams> URLSearchParamsIteratorPrototype;

template<>
const JSC::ClassInfo URLSearchParamsIterator::s_info = { "URLSearchParams Iterator", &Base::s_info, 0, CREATE_METHOD_TABLE(URLSearchParamsIterator) };

template<>
const JSC::ClassInfo URLSearchParamsIteratorPrototype::s_info = { "URLSearchParams Iterator", &Base::s_info, 0, CREATE_METHOD_TABLE(URLSearchParamsIteratorPrototype) };

JSC::JSValue JSURLSearchParams::entries(JSC::ExecState*)
{
    return createIterator<JSURLSearchParams, URLSearchParams>(*globalObject(), *this, IterationKind::KeyValue);
}

JSC::JSValue JSURLSearchParams::keys(JSC::ExecState*)
{
    return createIterator<JSURLSearchParams, URLSearchParams>(*globalObject(), *this, IterationKind::Key);
}

JSC::JSValue JSURLSearchParams::values(JSC::ExecState*)
{
    return createIterator<JSURLSearchParams, URLSearchParams>(*globalObject(), *this, IterationKind::Value);
}

JSC::JSValue JSURLSearchParams::forEach(JSC::ExecState* state)
{
	return keyValueIteratorForEach<JSURLSearchParams, URLSearchParams>(*state, "forEach");
}

}

#endif
