/*
 *  Copyright (c) 2015, Canon Inc. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1.  Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *  2.  Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *  3.  Neither the name of Canon Inc. nor the names of
 *      its contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY CANON INC. AND ITS CONTRIBUTORS "AS IS" AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL CANON INC. AND ITS CONTRIBUTORS BE LIABLE FOR
 *  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WebCoreJSBuiltins_h
#define WebCoreJSBuiltins_h

#if ENABLE(FETCH_API)
#include "DOMWindowBuiltinsWrapper.h"
#include "FetchHeadersBuiltinsWrapper.h"
#include "FetchInternalsBuiltinsWrapper.h"
#include "FetchRequestBuiltinsWrapper.h"
#include "FetchResponseBuiltinsWrapper.h"
#include "WorkerGlobalScopeBuiltinsWrapper.h"
#endif

#if ENABLE(MEDIA_STREAM) && 0
#include "MediaDevicesBuiltinsWrapper.h"
#endif

#if ENABLE(STREAMS_API)
#include "ByteLengthQueuingStrategyBuiltinsWrapper.h"
#include "CountQueuingStrategyBuiltinsWrapper.h"
#include "ReadableStreamBuiltinsWrapper.h"
#include "ReadableStreamDefaultControllerBuiltinsWrapper.h"
#include "ReadableStreamDefaultReaderBuiltinsWrapper.h"
#include "ReadableStreamInternalsBuiltinsWrapper.h"
#include "StreamInternalsBuiltinsWrapper.h"
#include "WritableStreamBuiltinsWrapper.h"
#include "WritableStreamInternalsBuiltinsWrapper.h"
#endif

#include <runtime/VM.h>

namespace WebCore {

class JSBuiltinFunctions {
public:
    explicit JSBuiltinFunctions(JSC::VM& v)
        : vm(v)
#if ENABLE(FETCH_API)
    	, m_dOMWindowBuiltins(vm)
        , m_fetchHeadersBuiltins(vm)
    	, m_fetchInternalsBuiltins(vm)
    	, m_fetchRequestBuiltins(vm)
    	, m_fetchResponseBuiltins(vm)
    	, m_workerGlobalScopeBuiltins(vm)
#endif
#if ENABLE(STREAMS_API)
        , m_byteLengthQueuingStrategyBuiltins(vm)
        , m_countQueuingStrategyBuiltins(vm)
        , m_readableStreamBuiltins(vm)
        , m_readableStreamDefaultControllerBuiltins(vm)
        , m_readableStreamInternalsBuiltins(vm)
        , m_readableStreamDefaultReaderBuiltins(vm)
    	, m_streamInternalsBuiltins(vm)
    	, m_writableStreamBuiltins(vm)
    	, m_writableStreamInternalsBuiltins(vm)
#endif
#if ENABLE(MEDIA_STREAM) && 0
        , m_mediaDevicesBuiltins(vm)
#endif
    {
    }

#if ENABLE(FETCH_API)
    DOMWindowBuiltinsWrapper& dOMWindowBuiltins() { return m_dOMWindowBuiltins; }
    FetchHeadersBuiltinsWrapper& fetchHeadersBuiltins() { return m_fetchHeadersBuiltins; }
    FetchInternalsBuiltinsWrapper& fetchInternalsBuiltins() { return m_fetchInternalsBuiltins; }
    FetchRequestBuiltinsWrapper& fetchRequestBuiltins() { return m_fetchRequestBuiltins; }
    FetchResponseBuiltinsWrapper& fetchResponseBuiltins() { return m_fetchResponseBuiltins; }
    WorkerGlobalScopeBuiltinsWrapper& workerGlobalScopeBuiltins() { return m_workerGlobalScopeBuiltins; }
#endif
#if ENABLE(STREAMS_API)
    ByteLengthQueuingStrategyBuiltinsWrapper& byteLengthQueuingStrategyBuiltins() { return m_byteLengthQueuingStrategyBuiltins;}
    CountQueuingStrategyBuiltinsWrapper& countQueuingStrategyBuiltins() { return m_countQueuingStrategyBuiltins;}
    ReadableStreamBuiltinsWrapper& readableStreamBuiltins() { return m_readableStreamBuiltins;}
    ReadableStreamDefaultControllerBuiltinsWrapper& readableStreamDefaultControllerBuiltins() { return m_readableStreamDefaultControllerBuiltins;}
    ReadableStreamInternalsBuiltinsWrapper& readableStreamInternalsBuiltins() { return m_readableStreamInternalsBuiltins;}
    ReadableStreamDefaultReaderBuiltinsWrapper& readableStreamDefaultReaderBuiltins() { return m_readableStreamDefaultReaderBuiltins;}
    StreamInternalsBuiltinsWrapper& streamInternalsBuiltins() { return m_streamInternalsBuiltins; }
	WritableStreamBuiltinsWrapper& writableStreamBuiltins() { return m_writableStreamBuiltins; }
	WritableStreamInternalsBuiltinsWrapper& writableStreamInternalsBuiltins() { return m_writableStreamInternalsBuiltins;}
#endif
#if ENABLE(MEDIA_STREAM) && 0
    MediaDevicesBuiltinsWrapper& mediaDevicesBuiltins() { return m_mediaDevicesBuiltins; }
#endif

private:
    JSC::VM& vm;
#if ENABLE(FETCH_API)
    DOMWindowBuiltinsWrapper m_dOMWindowBuiltins;
    FetchHeadersBuiltinsWrapper m_fetchHeadersBuiltins;
    FetchInternalsBuiltinsWrapper m_fetchInternalsBuiltins;
    FetchRequestBuiltinsWrapper m_fetchRequestBuiltins;
    FetchResponseBuiltinsWrapper m_fetchResponseBuiltins;
    WorkerGlobalScopeBuiltinsWrapper m_workerGlobalScopeBuiltins;
#endif
#if ENABLE(STREAMS_API)
    ByteLengthQueuingStrategyBuiltinsWrapper m_byteLengthQueuingStrategyBuiltins;
    CountQueuingStrategyBuiltinsWrapper m_countQueuingStrategyBuiltins;
    ReadableStreamBuiltinsWrapper m_readableStreamBuiltins;
    ReadableStreamDefaultControllerBuiltinsWrapper m_readableStreamDefaultControllerBuiltins;
    ReadableStreamInternalsBuiltinsWrapper m_readableStreamInternalsBuiltins;
    ReadableStreamDefaultReaderBuiltinsWrapper m_readableStreamDefaultReaderBuiltins;
    StreamInternalsBuiltinsWrapper m_streamInternalsBuiltins;
    WritableStreamBuiltinsWrapper m_writableStreamBuiltins;
    WritableStreamInternalsBuiltinsWrapper m_writableStreamInternalsBuiltins;
#endif
#if ENABLE(MEDIA_STREAM) && 0
    MediaDevicesBuiltinsWrapper m_mediaDevicesBuiltins;
#endif
};

}

#endif
