
#if defined(ENABLE_CEHTML) && ENABLE_CEHTML

#include "config.h"
#include "KeyEvent.h"

#include "Document.h"
#include "DOMWindow.h"
#include "EventDispatcher.h"
#include "EventNames.h"
#include "EventHandler.h"
#include "Frame.h"
#include "PlatformKeyboardEvent.h"
#include "Settings.h"
#include "WindowsKeyboardCodes.h"

namespace WebCore {

static inline const AtomicString& eventTypeForKeyboardEventType(PlatformEvent::Type type)
{
    switch (type) {
        case PlatformEvent::KeyUp:
            return eventNames().keyupEvent;
        case PlatformEvent::RawKeyDown:
            return eventNames().keydownEvent;
        case PlatformEvent::Char:
            return eventNames().keypressEvent;
        case PlatformEvent::KeyDown:
            // The caller should disambiguate the combined event into RawKeyDown or Char events.
            break;
        default:
            break;
    }
    return eventNames().keydownEvent;
}

KeyEvent::KeyEvent()
{
}

DLL_PUBLIC KeyEvent::KeyEvent(const PlatformKeyboardEvent& key, AbstractView* view)
    : UIEventWithKeyState(eventTypeForKeyboardEventType(key.type()),
                          true, true, view, 0, key.ctrlKey(), key.altKey(), key.shiftKey(), key.metaKey())
    , m_keyEvent(adoptPtr(new PlatformKeyboardEvent(key)))
{
}

KeyEvent::KeyEvent(const AtomicString& typeArg, bool canBubbleArg, bool cancelableArg, bool ctrlKeyArg, bool altKeyArg, 
                        bool shiftKeyArg, bool metaKeyArg, unsigned keyCodeArg, unsigned charCodeArg, AbstractView* viewArg)
    : UIEventWithKeyState(typeArg, canBubbleArg, cancelableArg, viewArg, 0, ctrlKeyArg, altKeyArg, shiftKeyArg, metaKeyArg)
    , m_keyCode(keyCodeArg)
    , m_charCode(charCodeArg)
{
}

KeyEvent::~KeyEvent()
{
}

void KeyEvent::initKeyEvent(const AtomicString& typeArg, bool canBubbleArg, bool cancelableArg, bool ctrlKeyArg, bool altKeyArg, 
                        bool shiftKeyArg, bool metaKeyArg, unsigned keyCodeArg, unsigned charCodeArg, AbstractView* viewArg)
{
    if (dispatched())
        return;

    initUIEvent(typeArg, canBubbleArg, cancelableArg, viewArg, 0);

    m_keyCode = keyCodeArg;
    m_charCode = charCodeArg;

    m_ctrlKey = ctrlKeyArg;
    m_shiftKey = shiftKeyArg;
    m_altKey = altKeyArg;
    m_metaKey = metaKeyArg;
}

/*
    The value of keyCode holds the virtual key code value of the key which 
    was depressed if the event is a key event. Otherwise, the value is zero.
*/
int KeyEvent::keyCode() const
{
    if (!m_keyEvent)
        return 0;

    return m_keyCode;
}

/*
    charCode holds the value of the Unicode character associated with the 
    depressed key if the event is a key event. Otherwise, the value is zero.
*/
int KeyEvent::charCode() const
{
    if (!m_keyEvent)
        return 0;

    return m_charCode;
}

EventInterface KeyEvent::eventInterface() const
{
    return KeyEventInterfaceType;
}

bool KeyEvent::isKeyEvent() const
{
    return true;
}

int KeyEvent::which() const
{
    return keyCode();
}

} // namespace WebCore

#endif // defined(ENABLE_CEHTML) && ENABLE_CEHTML
