/*
 * Copyright (C) 2016 Apple, Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef LoadableClassicScript_h
#define LoadableClassicScript_h

#include "CachedResourceClient.h"
#include "CachedResourceHandle.h"
#include "CachedScript.h"
#include "LoadableScript.h"
#include "LoadableScriptClient.h"
#include "SecurityOrigin.h"
//#include <wtf/TypeCasts.h>

namespace WebCore {

// A CachedResourceHandle alone does not prevent the underlying CachedResource
// from purging its data buffer. This class holds a client until this class is
// destroyed in order to guarantee that the data buffer will not be purged.
class LoadableClassicScript FINAL : public LoadableScript, private CachedResourceClient {
	WTF_MAKE_FAST_ALLOCATED;
public:
    ~LoadableClassicScript();

    static RefPtr<LoadableClassicScript> create(CachedResourceHandle<CachedScript>&);
    virtual bool isLoaded() const OVERRIDE;
    virtual Optional<Error> wasErrored() const OVERRIDE;
    virtual bool wasCanceled() const OVERRIDE;

    CachedScript& cachedScript() { return *m_cachedScript; }
    virtual bool isClassicScript() const FINAL { return true; }

    void execute(ScriptElement&) OVERRIDE;

private:
    LoadableClassicScript(CachedResourceHandle<CachedScript>& cachedScript) : m_cachedScript(cachedScript) , m_error(Nullopt) { }

    virtual void notifyFinished(CachedResource*) OVERRIDE;

    CachedResourceHandle<CachedScript> m_cachedScript;
    Optional<Error> m_error;
};

}

//SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::LoadableClassicScript)
//    static bool isType(const WebCore::LoadableScript& script) { return script.isClassicScript(); }
//SPECIALIZE_TYPE_TRAITS_END()

#endif // LoadableClassicScript_h

