/*
 * Copyright (C) 2016 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef TextDecoder_h
#define TextDecoder_h

#include "BufferSource.h"
#include "ExceptionCode.h"
//#include "ExceptionOr.h"
#include "TextEncoding.h"
#include <wtf/Optional.h>
#include <wtf/PassRefPtr.h>
#include <wtf/RefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/WTFString.h>

namespace WebCore {

struct TextDecoderOptions {
	TextDecoderOptions() : fatal(false), ignoreBOM(false) { }
	TextDecoderOptions(bool fatal, bool ignoreBOM) : fatal(fatal), ignoreBOM(ignoreBOM) { }
    bool fatal;
    bool ignoreBOM;
};

struct TextDecodeOptions {
	TextDecodeOptions() : stream(false) { }
	TextDecodeOptions(bool stream) : stream(stream) { }
    bool stream;
};

class TextDecoder : public RefCounted<TextDecoder> {
public:
    static PassRefPtr<TextDecoder> create(const String& label, Optional<TextDecoderOptions>, ExceptionCode&);

    String encoding() const;
    bool fatal() const { return m_options.fatal; }
    bool ignoreBOM() const { return m_options.ignoreBOM; }
    String decode(Optional<BufferSource>, TextDecodeOptions, ExceptionCode&);

private:
    String prependBOMIfNecessary(const String&);
    void ignoreBOMIfNecessary(const uint8_t*& data, size_t& length);
    TextDecoder(const char*, Optional<TextDecoderOptions>);
    TextEncoding m_textEncoding;
    TextDecoderOptions m_options;
    bool m_hasDecoded;
    Vector<uint8_t> m_buffer;
};

}

#endif // TextDecoder_h
