
#include "config.h"
#include "EspialHTMLObjectElement.h"
#include "HTMLNames.h"

namespace WebCore {

ALWAYS_INLINE EspialHTMLObjectElement::EspialHTMLObjectElement(const QualifiedName& tagName, Document* document, HTMLFormElement* form, bool createdByParser)
: HTMLObjectElement(tagName, document, form, createdByParser)
, m_playerID(-1)
, m_playerState(0)
, m_isDownloadPlugin(false)
, m_isFullscreen(false)
, m_muted(MUTED_DEFAULT)
#if ENABLE(NETTV) || ENABLE(JP_HYBRIDCAST)
, m_lastPlayerTime(0.0f)
, m_playBackRate(1.0)
#endif
#if ENABLE(NETTV)
, m_releaseNativePlayerTimer(this, &EspialHTMLObjectElement::releaseNativePlayerTimerCallback)
#endif
{
	platformInit();
}

PassRefPtr<EspialHTMLObjectElement> EspialHTMLObjectElement::create(const QualifiedName& tagName, Document* document, HTMLFormElement* form, bool createdByParser)
{
    return adoptRef(new EspialHTMLObjectElement(tagName, document, form, createdByParser));
}

EspialHTMLObjectElement::~EspialHTMLObjectElement()
{
	document()->unregisterForDocumentActivationCallbacks(this);

	platformDestroy();
}

void EspialHTMLObjectElement::didMoveToNewDocument(Document* oldDocument)
{
    if (oldDocument)
    	oldDocument->unregisterForDocumentActivationCallbacks(this);

    document()->registerForDocumentActivationCallbacks(this);

	HTMLObjectElement::didMoveToNewDocument(oldDocument);
}

void EspialHTMLObjectElement::removedFrom(ContainerNode* insertionPoint)
{
	platformRemovedFrom(insertionPoint);
	HTMLObjectElement::removedFrom(insertionPoint);
}

#if PLATFORM(ESPIAL)
PassRefPtr<HTMLObjectElement> HTMLObjectElement::create(const QualifiedName& tagName, Document* document, HTMLFormElement* form, bool createdByParser)
{
    return EspialHTMLObjectElement::create(tagName, document, form, createdByParser);
}
#endif

}
