
#ifndef EspialHTMLObjectElement_h
#define EspialHTMLObjectElement_h

#include "HTMLObjectElement.h"
#include "IntRect.h"
#include "Timer.h"
#include <wtf/HashMap.h>

namespace JSC {
class JSObject;
}

namespace WebCore {

class EspialHTMLObjectElement : public HTMLObjectElement
{
public:
	enum MUTED_STATUS { MUTED_DEFAULT, MUTED_ON, MUTED_OFF };

    static PassRefPtr<EspialHTMLObjectElement> create(const QualifiedName&, Document*, HTMLFormElement*, bool createdByParser);

    virtual ~EspialHTMLObjectElement();

    int nativePlayerID() const { return m_playerID; }
    IntRect nativePlayerRect() const { return m_rect; }
    void setNativePlayerRect(const IntRect& rect) { m_rect = rect; }
    
    void createNativePlayer(bool isDownloadPlugin = false);
	void releaseNativePlayer();

	JSPropertyDelegate* jsPropertyDelegate() const { return m_jsPropertyDelegate.get(); }

    void setNativePlayerID(int playerID) { m_playerID = playerID; }

#if ENABLE(NETTV) || ENABLE(JP_HYBRIDCAST)
    double playBackRate() const { return m_playBackRate; }
    void setPlayBackRate(double playBackRate) { m_playBackRate = playBackRate; }
#endif

    // Use Document::registerForDocumentActivationCallbacks() to subscribe to these
    virtual void documentWillBecomeInactive();

    virtual void didMoveToNewDocument(Document* oldDocument) OVERRIDE;

#if ENABLE(NETTV) || ENABLE(JP_HYBRIDCAST)
    long request(const String& querystr);
#endif

    bool isFullscreen() const { return m_isFullscreen; }
    void setFullscreen(bool f) { m_isFullscreen = f; }

    MUTED_STATUS muted() const { return m_muted; }
    void setMuted(bool muted)
    {
    	m_muted = muted ? MUTED_ON : MUTED_OFF;
    }

#if ENABLE(NETTV) || ENABLE(JP_HYBRIDCAST)
	float lastPlayerTime() const { return m_lastPlayerTime; }
#endif

#if ENABLE(NETTV)
	void didDispatchOnEndedEvent();
#endif

	HashMap<JSC::JSObject*, String>& captionListenerURLMap() { return m_captionListenerURLMap; }

private:
	EspialHTMLObjectElement(const QualifiedName&, Document*, HTMLFormElement*, bool createdByParser);
    
    void platformInit();
    void platformDestroy();
    void platformRemovedFrom(ContainerNode*);

    virtual void createJSPropertyDelegate();
    
    virtual void removedFrom(ContainerNode*) OVERRIDE;

	int m_playerID;
	int m_playerState;
	IntRect m_rect;
	bool m_isDownloadPlugin : 1;
	bool m_isFullscreen : 1;
	MUTED_STATUS m_muted : 2;

#if ENABLE(NETTV) || ENABLE(JP_HYBRIDCAST)
	float m_lastPlayerTime;
	double m_playBackRate;
#endif

#if ENABLE(NETTV)
	void releaseNativePlayerTimerCallback(Timer<EspialHTMLObjectElement>*);
	Timer<EspialHTMLObjectElement> m_releaseNativePlayerTimer;
#endif

	HashMap<JSC::JSObject*, String> m_captionListenerURLMap;
};

}

#endif
