/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2010 Apple Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef HTMLFieldSetElement_h
#define HTMLFieldSetElement_h

#include "HTMLFormControlElement.h"
#include <wtf/HashSet.h>
#include <wtf/OwnPtr.h>

namespace WebCore {

class FormAssociatedElement;
class HTMLCollection;

class HTMLFieldSetElement : public HTMLFormControlElement {
public:
    static PassRefPtr<HTMLFieldSetElement> create(const QualifiedName&, Document*, HTMLFormElement*);
    HTMLLegendElement* legend() const;

    PassRefPtr<HTMLCollection> elements();

    const Vector<FormAssociatedElement*>& associatedElements() const;
    unsigned length() const;

    void addInvalidDescendant(const HTMLFormControlElement&);
    void removeInvalidDescendant(const HTMLFormControlElement&);

private:
    HTMLFieldSetElement(const QualifiedName&, Document*, HTMLFormElement*);
    ~HTMLFieldSetElement();

    virtual bool isEnumeratable() const { return true; }
    virtual bool supportsFocus() const;
    virtual RenderElement* createElementRenderer(PassRefPtr<RenderStyle>, const RenderTreePosition&);
    virtual const AtomicString& formControlType() const;
    virtual bool computeWillValidate() const { return false; }
    virtual void disabledAttributeChanged() OVERRIDE;
    virtual void disabledStateChanged() OVERRIDE;
    virtual void childrenChanged(const ChildChange&) OVERRIDE;
    virtual void didMoveToNewDocument(Document* oldDocument) OVERRIDE;

    virtual bool matchesValidPseudoClass() const OVERRIDE;
    virtual bool matchesInvalidPseudoClass() const OVERRIDE;

    void refreshElementsIfNeeded() const;

    mutable Vector<FormAssociatedElement*> m_associatedElements;
    // When dom tree is modified, we have to refresh the m_associatedElements array.
    mutable uint64_t m_documentVersion;
    HashSet<const HTMLFormControlElement*> m_invalidDescendants;
};

inline bool isHTMLFieldSetElement(const Node* node)
{
    return node->hasTagName(HTMLNames::fieldsetTag);
}

inline bool isHTMLFieldSetElement(const Element* element)
{
    return element->hasTagName(HTMLNames::fieldsetTag);
}

inline bool isHTMLFieldSetElement(const Node& node)
{
    return node.hasTagName(HTMLNames::fieldsetTag);
}

inline bool isHTMLFieldSetElement(const Element& element)
{
    return element.hasTagName(HTMLNames::fieldsetTag);
}

inline HTMLFieldSetElement* toHTMLFieldSetElement(Node* node)
{
    ASSERT(!node || isHTMLFieldSetElement(node));
    return static_cast<HTMLFieldSetElement*>(node);
}

inline HTMLFieldSetElement& toHTMLFieldSetElement(Node& node)
{
    ASSERT(isHTMLFieldSetElement(node));
    return static_cast<HTMLFieldSetElement&>(node);
}

} // namespace

#endif
