/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2010 Apple Inc. All rights reserved.
 * Copyright (C) 2010 Google Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"
#include "HTMLOpcodeElement.h"
#include "Attribute.h"
#include "ElementTraversal.h"
#include "ScriptElement.h"
#include <wtf/text/StringBuilder.h>

namespace WebCore {

using namespace HTMLNames;

HTMLOpcodeElement::HTMLOpcodeElement(const QualifiedName& tagName, Document* document)
: HTMLImageElement(tagName, document)
{
    ASSERT(hasTagName(opTag));
}

PassRefPtr<HTMLOpcodeElement> HTMLOpcodeElement::create(Document* document)
{
    return adoptRef(new HTMLOpcodeElement(opTag, document));
}

PassRefPtr<HTMLOpcodeElement> HTMLOpcodeElement::create(const QualifiedName& tagName, Document* document)
{
    return adoptRef(new HTMLOpcodeElement(tagName, document));
}

HTMLOpcodeElement::~HTMLOpcodeElement()
{
}

void HTMLOpcodeElement::parseAttribute(const Attribute& attribute)
{
	if (attribute.name() == codeAttr) {
		HTMLImageElement::setSrc(attribute.value().string());
	}
	else{
		HTMLImageElement::parseAttribute(attribute);
	}
}

void HTMLOpcodeElement::finishParsingChildren()
{
	HTMLImageElement::finishParsingChildren();

    StringBuilder text;

    for (Node* node = firstChild(); node; ) {
        if (node->isTextNode())
            text.append(node->nodeValue());

        // Text nodes inside script elements are not part of the option text.
        if (node->isElementNode() && toScriptElement(toElement(node)))
            node = NodeTraversal::nextSkippingChildren(node, this);
        else
            node = NodeTraversal::next(node, this);
    }

    HTMLImageElement::setAttribute(altAttr, text.toString());
}

}
