/*
 * Copyright (C) 2006, 2011, 2012 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#include "config.h"
#include "HTMLOptionsCollection.h"

#include "ExceptionCode.h"
#include "HTMLOptionElement.h"
#include "HTMLSelectElement.h"

namespace WebCore {

inline HTMLOptionsCollection::HTMLOptionsCollection(ContainerNode* select)
    : HTMLCollection(select, SelectOptions)
{
    ASSERT(select->hasTagName(HTMLNames::selectTag));
}

PassRefPtr<HTMLOptionsCollection> HTMLOptionsCollection::create(ContainerNode* select, CollectionType)
{
    return adoptRef(new HTMLOptionsCollection(select));
}

HTMLSelectElement* HTMLOptionsCollection::selectElement()
{
	return toHTMLSelectElement(&ownerNode());
}

const HTMLSelectElement* HTMLOptionsCollection::selectElement() const
{
	return toHTMLSelectElement(&ownerNode());
}

void HTMLOptionsCollection::add(HTMLElement* element, HTMLElement* beforeElement, ExceptionCode& ec)
{
    selectElement()->add(element, beforeElement, ec);
}

void HTMLOptionsCollection::add(HTMLElement* element, int beforeIndex, ExceptionCode& ec)
{
    add(element, toHTMLElement(item(beforeIndex)), ec);
}

void HTMLOptionsCollection::remove(int index)
{
    toHTMLSelectElement(ownerNode()).remove(index);
}

int HTMLOptionsCollection::selectedIndex() const
{
    return toHTMLSelectElement(ownerNode()).selectedIndex();
}

void HTMLOptionsCollection::setSelectedIndex(int index)
{
    toHTMLSelectElement(ownerNode()).setSelectedIndex(index);
}

void HTMLOptionsCollection::setLength(unsigned length, ExceptionCode& ec)
{
    toHTMLSelectElement(ownerNode()).setLength(length, ec);
}

} //namespace
