/*
 * Copyright (C) 2015 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef HTMLSlotElement_h
#define HTMLSlotElement_h

#if ENABLE(SHADOW_DOM) || ENABLE(DETAILS_ELEMENT)

#include "HTMLElement.h"
#include "Range.h"

namespace WebCore {

class HTMLSlotElement FINAL : public HTMLElement {
public:
    static PassRefPtr<HTMLSlotElement> create(const QualifiedName&, Document*);

    const Vector<Node*>* assignedNodes() const;

private:
    HTMLSlotElement(const QualifiedName&, Document*);

    virtual InsertionNotificationRequest insertedInto(ContainerNode*) OVERRIDE;
    virtual void removedFrom(ContainerNode*) OVERRIDE;
    virtual void attributeChanged(const QualifiedName&, const AtomicString& oldValue, const AtomicString& newValue/*, AttributeModificationReason*/) OVERRIDE;
};

inline bool isHTMLSlotElement(const Node* node)
{
    return node->hasTagName(HTMLNames::slotTag);
}

inline bool isHTMLSlotElement(const Element* element)
{
    return element->hasTagName(HTMLNames::slotTag);
}

inline bool isHTMLSlotElement(const Node& node)
{
    return node.hasTagName(HTMLNames::slotTag);
}

inline bool isHTMLSlotElement(const Element& element)
{
    return element.hasTagName(HTMLNames::slotTag);
}

inline HTMLSlotElement* toHTMLSlotElement(Node* node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!node || isHTMLSlotElement(node));
    return static_cast<HTMLSlotElement*>(node);
}

inline HTMLSlotElement& toHTMLSlotElement(Node& node)
{
    ASSERT_WITH_SECURITY_IMPLICATION(isHTMLSlotElement(node));
    return static_cast<HTMLSlotElement&>(node);
}

template <typename ArgType>
struct ElementTypeCastTraits<const HTMLSlotElement, ArgType> {
    static bool is(ArgType& node) { return isHTMLSlotElement(node); }
};

}

#endif
#endif
