/*
 * Copyright (C) 2008 Apple Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef TextMetrics_h
#define TextMetrics_h

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>

namespace WebCore {

class TextMetrics : public RefCounted<TextMetrics> {
public:
    static PassRefPtr<TextMetrics> create() { return adoptRef(new TextMetrics); }

    float width() const { return m_width; }
    void setWidth(float w) { m_width = w; }

    int unitsPerEm() const { return m_unitsPerEm; }
    void setUnitsPerEm(int value) { m_unitsPerEm = value; }

    float ascent() const { return m_ascent; }
    void setAscent(float value) { m_ascent = value; }

    float descent() const { return m_descent; }
    void setDescent(float value) { m_descent = value; }

    float lineGap() const { return m_lineGap; }
    void setLineGap(float value) { m_lineGap = value; }

    float lineSpacing() const { return m_lineSpacing; }
    void setLineSpacing(float value) { m_lineSpacing = value; }

    float xHeight() const { return m_xHeight; }
    void setXHeight(float value) { m_xHeight = value; }

private:
    TextMetrics()
        : m_width(0)
, m_unitsPerEm(0)
, m_ascent(0)
, m_descent(0)
, m_lineGap(0)
, m_lineSpacing(0)
, m_xHeight(0)
    { }

    float m_width;

    int m_unitsPerEm;
    float m_ascent;
    float m_descent;
    float m_lineGap;
    float m_lineSpacing;
    float m_xHeight;
};

} // namespace WebCore

#endif // TextMetrics_h
