/*
 * Copyright (C) 2016 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef URLSearchParams_h
#define URLSearchParams_h

#include "DOMURL.h"
#include <wtf/Optional.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {

class URLSearchParams : public RefCounted<URLSearchParams> {
public:
    static PassRefPtr<URLSearchParams> create(const String& string, DOMURL* associatedURL = 0) { return adoptRef(new URLSearchParams(string, associatedURL)); }
    static PassRefPtr<URLSearchParams> create(const Vector<std::pair<String, String>>& pairs) { return adoptRef(new URLSearchParams(pairs)); }
    static PassRefPtr<URLSearchParams> create(const URLSearchParams* init) { return adoptRef(new URLSearchParams(init)); }

    void associatedURLDestroyed() { m_associatedURL = 0; }
    void append(const String& name, const String& value);
    void remove(const String& name);
    String get(const String& name) const;
    Vector<String> getAll(const String& name) const;
    bool has(const String& name) const;
    void set(const String& name, const String& value);
    String toString() const;
    const Vector<std::pair<String, String>>& pairs() const { return m_pairs; }
    operator const Vector<std::pair<String, String>>&() { return m_pairs; }
    void updateFromAssociatedURL();

    class Iterator {
    public:
        explicit Iterator(URLSearchParams&);
        typedef WTF::KeyValuePair<String, String> NextPair;
        Optional<NextPair> next();

    private:
        RefPtr<URLSearchParams> m_target;
        size_t m_index;;
    };
    Iterator createIterator() { return Iterator(*this); }

private:
    URLSearchParams(const String&, DOMURL*);
    explicit URLSearchParams(const Vector<std::pair<String, String>>&);
    explicit URLSearchParams(const URLSearchParams*);
    void updateURL();

    DOMURL* m_associatedURL;
    Vector<std::pair<String, String>> m_pairs;
};

}

#endif // URLSearchParams_h
