/*
 * Copyright (C) 2016 Canon Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted, provided that the following conditions
 * are required to be met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Canon Inc. nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CANON INC. AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CANON INC. AND ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef FetchOptions_h
#define FetchOptions_h

#include "ReferrerPolicy.h"

namespace WebCore {

typedef ReferrerHeaderPolicy FetchReferrerPolicy;
enum class FetchRequestCache { Default, NoStore, Reload, NoCache, ForceCache, OnlyIfCached };
enum class FetchRequestCredentials { Omit, SameOrigin, Include };
enum class FetchRequestDestination { EmptyString, Document, Sharedworker, Subresource, Unknown, Worker };
enum class FetchRequestMode { Navigate, SameOrigin, NoCors, Cors };
enum class FetchRequestRedirect { Follow, Error, Manual };
enum class FetchRequestType { Default, EmptyString = Default, Audio, Font, Image, Script, Style, Track, Video };

class FetchOptions {
public:
    typedef FetchRequestType Type;
    typedef FetchRequestDestination Destination;
    typedef FetchRequestMode Mode;
    typedef FetchRequestCredentials Credentials;
    typedef FetchRequestCache Cache;
    typedef FetchRequestRedirect Redirect;

    FetchOptions();
    FetchOptions(Type, Destination, Mode, Credentials, Cache, Redirect, ReferrerHeaderPolicy);

    Type type() const { return m_type; }
    Destination destination() const { return m_destination; }
    Mode mode() const { return m_mode; }
    Credentials credentials() const { return m_credentials; }
    Cache cache() const { return m_cache; }
    Redirect redirect() const { return m_redirect; }
    ReferrerHeaderPolicy referrerPolicy() const { return m_referrerPolicy; }

    void setType(Type type) { m_type = type; }
    void setDestination(Destination destination) { m_destination = destination; }
    void setMode(Mode mode) { m_mode = mode; }
    void setCredentials(Credentials credentials) { m_credentials = credentials; }
    void setCache(Cache cache) { m_cache = cache; }
    void setRedirect(Redirect redirect) { m_redirect = redirect; }
    void setReferrerPolicy(ReferrerHeaderPolicy referrerPolicy) { m_referrerPolicy = referrerPolicy; }

    bool forceAllowed() const { return m_forceAllowed; }
    void setForceAllowed(bool allowed) { m_forceAllowed = true; }

private:
    Type m_type;
    Destination m_destination;
    Mode m_mode;
    Credentials m_credentials;
    Cache m_cache;
    Redirect m_redirect;
    ReferrerHeaderPolicy m_referrerPolicy;
    bool m_forceAllowed;
};

inline FetchOptions::FetchOptions()
: m_type(Type::Default)
, m_destination(Destination::EmptyString)
, m_mode(Mode::NoCors)
, m_credentials(Credentials::Omit)
, m_cache(Cache::Default)
, m_redirect(Redirect::Follow)
, m_referrerPolicy(ReferrerHeaderPolicy::EmptyString)
, m_forceAllowed(false)
{
}

inline FetchOptions::FetchOptions(Type type, Destination destination, Mode mode, Credentials credentials, Cache cache, Redirect redirect, ReferrerHeaderPolicy referrerPolicy)
: m_type(type)
, m_destination(destination)
, m_mode(mode)
, m_credentials(credentials)
, m_cache(cache)
, m_redirect(redirect)
, m_referrerPolicy(referrerPolicy)
, m_forceAllowed(false)
{
}

} // namespace WebCore

#endif // FetchOptions_h
