/*
 * Copyright (C) 2011 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ResourceLoaderOptions_h
#define ResourceLoaderOptions_h

#include "FetchOptions.h"
#include "ResourceHandleTypes.h"
#include <wtf/FastAllocBase.h>

namespace WebCore {
    
enum SendCallbackPolicy {
    SendCallbacks,
    DoNotSendCallbacks
};

enum ContentSniffingPolicy {
    SniffContent,
    DoNotSniffContent
};

enum DataBufferingPolicy {
    BufferData,
    DoNotBufferData
};

enum SecurityCheckPolicy {
    SkipSecurityCheck,
    DoSecurityCheck
};

enum CertificateInfoPolicy {
    IncludeCertificateInfo,
    DoNotIncludeCertificateInfo
};

enum class ContentSecurityPolicyImposition : uint8_t {
    SkipPolicyCheck,
    DoPolicyCheck
};

enum class CachingPolicy : uint8_t {
    AllowCaching,
    DisallowCaching
};

enum class ClientCredentialPolicy {
    CannotAskClientForCredentials,
    MayAskClientForCredentials
};

enum class SameOriginDataURLFlag {
    Set,
    Unset
};

struct ResourceLoaderOptions : public FetchOptions{
	WTF_MAKE_FAST_ALLOCATED;
public:
    ResourceLoaderOptions()
		: sendLoadCallbacks(DoNotSendCallbacks)
		, sniffContent(DoNotSniffContent)
		, dataBufferingPolicy(BufferData)
		, allowCredentials(DoNotAllowStoredCredentials)
		, securityCheck(DoSecurityCheck)
		, m_contentSecurityPolicyImposition(ContentSecurityPolicyImposition::DoPolicyCheck)
		, m_cachingPolicy(CachingPolicy::AllowCaching)
		, sameOriginDataURLFlag(SameOriginDataURLFlag::Unset)
		, persistent(false)
		, clientCredentialPolicy(ClientCredentialPolicy::CannotAskClientForCredentials)
	{
	}

    ResourceLoaderOptions(SendCallbackPolicy sendLoadCallbacks, ContentSniffingPolicy sniffContent, DataBufferingPolicy dataBufferingPolicy, StoredCredentials allowCredentials, ClientCredentialPolicy credentialPolicy, FetchOptions::Credentials credentials, SecurityCheckPolicy securityCheck, FetchOptions::Mode mode, ContentSecurityPolicyImposition contentSecurityPolicyImposition, CachingPolicy cachingPolicy)
        : sendLoadCallbacks(sendLoadCallbacks)
        , sniffContent(sniffContent)
        , dataBufferingPolicy(dataBufferingPolicy)
        , allowCredentials(allowCredentials)
        , securityCheck(securityCheck)
    	, m_contentSecurityPolicyImposition(contentSecurityPolicyImposition)
    	, m_cachingPolicy(cachingPolicy)
    	, sameOriginDataURLFlag(SameOriginDataURLFlag::Unset)
    	, persistent(false)
		, clientCredentialPolicy(credentialPolicy)
    {
    	setCredentials(credentials);
        setMode(mode);
    }
    ContentSecurityPolicyImposition contentSecurityPolicyImposition() const { return m_contentSecurityPolicyImposition; }
    void setContentSecurityPolicyImposition(ContentSecurityPolicyImposition imposition) { m_contentSecurityPolicyImposition = imposition; }
    CachingPolicy cachingPolicy() const { return m_cachingPolicy; }
	void setCachingPolicy(CachingPolicy cachingPolicy) { m_cachingPolicy = cachingPolicy; }

    SendCallbackPolicy sendLoadCallbacks;
    ContentSniffingPolicy sniffContent;
    DataBufferingPolicy dataBufferingPolicy;
    StoredCredentials allowCredentials; // Whether HTTP credentials and cookies are sent with the request.
	SecurityCheckPolicy securityCheck;
	ContentSecurityPolicyImposition m_contentSecurityPolicyImposition;
	CachingPolicy m_cachingPolicy;
    SameOriginDataURLFlag sameOriginDataURLFlag;
    bool persistent;

    ClientCredentialPolicy clientCredentialPolicy;
};

} // namespace WebCore    

#endif // ResourceLoaderOptions_h
