/*
 * Copyright (C) 2008 Apple Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "OfflineCache.h"

#include "Frame.h"
#include "Page.h"
#include "Settings.h"
#include "PlatformString.h"
#include <wtf/PassRefPtr.h>
#include "ApplicationCacheStorage.h"
#include "ApplicationCacheGroup.h"
#include "ExportedSymbols.h"

namespace WebCore
{

PassRefPtr<OfflineCache> OfflineCache::create(Frame* frame)
{
    return adoptRef(new OfflineCache(frame));
}

OfflineCache::OfflineCache(Frame* frame)
    : DOMWindowProperty(frame)
{
}

OfflineCache::~OfflineCache()
{
}

Vector<String> OfflineCache::getUsage() const
{
    if (!m_frame || !m_frame->document())
        return Vector<String>();

	Vector<String> res;
	const String host = frame()->document()->url().host();
	ApplicationCacheStorage & cacheStorage = cacheStorageTrustedOrUntrusted(host);
	
	res.append(host);
	
	RefPtr<SecurityOrigin> p = WebCoreAPIs::createSecurityOrigin(KURL(ParsedURLString, String("http://") + host));

	int64_t usage = -1;
	cacheStorage.calculateUsageForOrigin(p.get(), usage);
	res.append(String::number(usage));

	int64_t quota = -1;
	cacheStorage.calculateQuotaForOrigin(p.get(), quota);
	res.append(String::number(quota));
	
    return res;
}


}

