/*
 * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef CachedFont_h
#define CachedFont_h

#include "CachedResource.h"
#include "CachedResourceClient.h"
#include "FontOrientation.h"
#include "FontRenderingMode.h"
#include "FontWidthVariant.h"
#include "SimpleFontData.h"
#include "Timer.h"
#include <wtf/PassOwnPtr.h>

namespace WebCore {

class CachedResourceLoader;
class FontDescription;
class FontPlatformData;
class SVGDocument;
class SVGFontElement;
struct FontCustomPlatformData;

class CachedFont : public CachedResource {
public:
    CachedFont(CachedResourceRequest&, Type = FontResource);
    virtual ~CachedFont();
    
    virtual void load(CachedResourceLoader*);
    NO_RETURN_DUE_TO_ASSERT virtual void setBodyDataFrom(const CachedResource&) FINAL { ASSERT_NOT_REACHED(); }

    virtual void didAddClient(CachedResourceClient*);
    virtual void finishLoading(ResourceBuffer*) OVERRIDE;
    virtual void error(CachedResource::Status);

    virtual void allClientsRemoved();

    void checkNotify();

    void beginLoadIfNeeded(CachedResourceLoader* dl);
    bool stillNeedsLoad() const { return !m_loadInitiated; }

    virtual bool ensureCustomFontData(const AtomicString& remoteURI);

    virtual PassRefPtr<SimpleFontData> getFontData(const FontDescription&, const AtomicString& remoteURI, bool syntheticBold, bool syntheticItalic);

    virtual void purge();

protected:
    PassRefPtr<SimpleFontData> platformDataFromCustomData(const FontDescription&, bool syntheticBold, bool syntheticItalic);

    bool ensureCustomFontData(SharedBuffer* data);

private:
    virtual bool mayTryReplaceEncodedData() const OVERRIDE;

    OwnPtr<FontCustomPlatformData> m_fontData;
    bool m_loadInitiated;
    bool m_hasCreatedFontDataWrappingResource;

    void fontPlatformDataDeletionTimerCallback(Timer<CachedFont>*);

	Timer<CachedFont> m_fontPlatformDataDeletionTimer;

    friend class MemoryCache;
};

class CachedFontClient : public CachedResourceClient {
public:
    virtual ~CachedFontClient() { }
    static CachedResourceClientType expectedType() { return FontType; }
    virtual CachedResourceClientType resourceClientType() const { return expectedType(); }
    virtual void fontLoaded(CachedFont*) { }
};

}

#endif
