
#ifndef ExportedSymbols_h
#define ExportedSymbols_h

#include <wtf/RefPtr.h>
#include "Color.h"
#include "KURL.h"
#include "LayoutInlineMethods.h"
#include "RenderTreeAsText.h"
#include "ScrollView.h"
#include "SharedBuffer.h"
#include "Widget.h"
#include "ScriptValue.h"
#include "StyledElement.h"
#include "TextCodec.h"
#include <wtf/text/StringView.h>

namespace JSC
{
class MarkedArgumentBuffer;
}

namespace WebCore
{
class Document;
class Element;
class EventTarget;
class File;
class FloatRect;
class FloatQuad;
class LayoutRect;
class Frame;
class FrameView;
class JSElement;
class HitTestResult;
class Node;
class Page;
class QualifiedName;
class HTMLQualifiedName;
class RenderLayer;
class RenderWidget;
class ResourceLoader;
class SecurityOrigin;
class Widget;
#if ENABLE(RMAI)
class MOMPlayer;
class PlatformKeyboardEvent;
class PluginViewMOM;
#endif

}

namespace WebCoreAPIs
{
	const WebCore::HTMLQualifiedName& aTag();
	const WebCore::HTMLQualifiedName& appletTag();
	const WebCore::HTMLQualifiedName& areaTag();
	const WebCore::HTMLQualifiedName& bodyTag();
	const WebCore::HTMLQualifiedName& buttonTag();
	const WebCore::HTMLQualifiedName& canvasTag();
	const WebCore::HTMLQualifiedName& pTag();
	const WebCore::HTMLQualifiedName& brTag();
	const WebCore::HTMLQualifiedName& divTag();
	const WebCore::HTMLQualifiedName& embedTag();
	const WebCore::HTMLQualifiedName& formTag();
	const WebCore::HTMLQualifiedName& frameTag();
	const WebCore::HTMLQualifiedName& htmlTag();
	const WebCore::HTMLQualifiedName& iframeTag();
	const WebCore::HTMLQualifiedName& imgTag();
	const WebCore::HTMLQualifiedName& inputTag();
	const WebCore::HTMLQualifiedName& liTag();
	const WebCore::HTMLQualifiedName& objectTag();
	const WebCore::HTMLQualifiedName& paramTag();
	const WebCore::HTMLQualifiedName& preTag();
	const WebCore::HTMLQualifiedName& scriptTag();
	const WebCore::HTMLQualifiedName& spanTag();
	const WebCore::HTMLQualifiedName& styleTag();
	const WebCore::HTMLQualifiedName& textareaTag();
	const WebCore::HTMLQualifiedName& titleTag();
	const WebCore::HTMLQualifiedName& ulTag();
	const WebCore::HTMLQualifiedName& videoTag();

#if ENABLE(BML)
	const WebCore::HTMLQualifiedName& beitemTag();
#endif

	bool hasTagName(const WebCore::Node&, const WebCore::HTMLQualifiedName&);
	bool hasTagName(const WebCore::Node*, const WebCore::HTMLQualifiedName&);

	const WebCore::QualifiedName& dataAttr();
	const WebCore::QualifiedName& hrefAttr();
	const WebCore::QualifiedName& idAttr();
	const WebCore::QualifiedName& inputmodeAttr();
	const WebCore::QualifiedName& nameAttr();
	const WebCore::QualifiedName& readonlyAttr();
	const WebCore::QualifiedName& styleAttr();
	const WebCore::QualifiedName& typeAttr();
	const WebCore::QualifiedName& valueAttr();
	
	void jsGarbageCollectNow(bool noStackObjects, bool clearCache = false, bool shouldDeleteAllCodeBlocks = false, bool force = false);

	void clearAcceleratedSurfaceCache();

    WebCore::ScriptValue executeScript(WebCore::Frame*, const String& script, bool forceUserGesture = false);

	void scrollRectToVisible(WebCore::RenderLayer* layer, const WebCore::IntRect &rect);

	WebCore::Color colorFromPremultipliedARGB(unsigned);
	unsigned premultipliedARGBFromColor(const WebCore::Color&);

	WebCore::FrameView* frameView(const WebCore::Node*);

	void setShouldTrackVisitedLinks(bool);

	void addSubResourceLoad(WebCore::ResourceLoader*);

    WebCore::KURL iconURL(WebCore::Frame*);

    bool isLiveLink(const WebCore::HitTestResult&);
    WebCore::KURL absoluteLinkURL(const WebCore::HitTestResult&);

    WebCore::RenderWidget* find(const WebCore::Widget*);

    bool protocolIsJavaScript(const WebCore::KURL&);
    String decodeURLEscapeSequences(const String&);
    String decodeURLEscapeSequences(const String&, const String& encoding);

    WebCore::Color makeColor(int r, int g, int b, int a);

    String navProperty(WebCore::StyledElement*, WebCore::StyledElement::ENavProperty);

    WebCore::Element* getElementById(WebCore::Document*, const String&);

    String filenameFromHTTPContentDisposition(const String&);
    String extractMIMETypeFromMediaType(const String& mediaType);
    String extractCharsetFromMediaType(const String&);

    WebCore::IntRect enclosingIntRect(const WebCore::FloatRect&);
    WebCore::IntRect enclosingIntRect(const WebCore::LayoutRect&);
    WebCore::IntRect enclosingBoundingBox(const WebCore::FloatQuad&);

    PassRefPtr<WebCore::SharedBuffer> utf8Buffer(const String&);

    bool hasEventListeners(WebCore::EventTarget*, const AtomicString& eventType);

    PassRefPtr<WebCore::File> createFile(const String& filename);
    PassRefPtr<WebCore::Document> createDocument(WebCore::Frame* frame, const WebCore::KURL& url);
    PassRefPtr<WebCore::SecurityOrigin> createSecurityOrigin(const WebCore::KURL&);

    void addJSEventListener(WebCore::JSElement*, JSC::ExecState*, const String& eventName, JSC::JSValue);
    void removeJSEventListener(WebCore::JSElement*, JSC::ExecState*, const String& eventName, JSC::JSValue);
    void clearJSEventListeners(WebCore::JSElement*, JSC::ExecState*, const String& eventName);
    void dispatchJSEvent(WebCore::Element* target, const String& eventName, const JSC::MarkedArgumentBuffer& args);

    void dispatchSimulatedClick(WebCore::Element*, WebCore::Event* underlyingEvent, bool sendMouseEvents = false, bool showPressedLook = true);

    CString encode(const char* encodingName, StringView, WebCore::UnencodableHandling handling);

    String externalRepresentation(WebCore::Frame*, WebCore::RenderAsTextBehavior = WebCore::RenderAsTextBehaviorNormal);

    void setIndexedDBDatabasePath(WebCore::Page*, const String&);

    void webdatabaseSetQuota(const String& path, const char* domain, unsigned long long);

    void closeAllDatabases();

    void handleMemoryPressure();

    void invalidateDOMCookieCacheInAllDocuments();

    void scheduleRedirect(WebCore::Frame*, double delay, const String&);

#if ENABLE(RMAI)
    PassRefPtr<WebCore::MOMPlayer> createMOMPlayer(WebCore::ScriptExecutionContext* context);
    JSC::JSValue getJSValueFromMOMPlayer(JSC::ExecState* exec, WebCore::Element* element, WebCore::MOMPlayer* player);

    void initMOM();
    void loadApplicationTree(WebCore::Frame* frame);

    PassRefPtr<WebCore::Widget> createPluginViewMOM(WebCore::Frame* parentFrame, const WebCore::IntSize& pluginSize, WebCore::Element* element, const WebCore::KURL& url, const WTF::Vector<WTF::String>& paramNames, const WTF::Vector<WTF::String>& paramValues, const WTF::String& mimeType, bool loadManually);
    void setPluginViewMOMPlayer(WebCore::PluginViewMOM* pluginViewMOM, JSC::JSValue value);
    void setPluginViewActive(WebCore::PluginViewMOM* pluginViewMOM, JSC::JSValue value);

    bool handleKeyEvent(const WebCore::PlatformKeyboardEvent& kevt, WebCore::Frame* frame);
#endif
}

#endif /*ExportedSymbols_h*/
