/*
    Copyright (C) 1999 Lars Knoll (knoll@kde.org)
    Copyright (C) 2006, 2008 Apple Inc. All rights reserved.
    Copyright (c) 2012, Google Inc. All rights reserved.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef LengthBox_h
#define LengthBox_h

#include "Length.h"
#include "WritingMode.h"

namespace WebCore {

#if 0

template<typename T> class BoxExtent {
public:
    BoxExtent()
        : m_sides({{ T(0), T(0), T(0), T(0) }})
    {
    }

    BoxExtent(const T& top, const T& right, const T& bottom, const T& left)
        : m_sides({{ top, right, bottom, left }})
    {
    }

    T& at(PhysicalBoxSide side) { return m_sides[side]; }
    T& top() { return at(TopSide); }
    T& right() { return at(RightSide); }
    T& bottom() { return at(BottomSide); }
    T& left() { return at(LeftSide); }

    const T& at(PhysicalBoxSide side) const { return m_sides[side]; }
    const T& top() const { return at(TopSide); }
    const T& right() const { return at(RightSide); }
    const T& bottom() const { return at(BottomSide); }
    const T& left() const { return at(LeftSide); }

    void setAt(PhysicalBoxSide side, const T& v) { at(side) = v; }
    void setTop(const T& top) { setAt(TopSide, top); }
    void setRight(const T& right) { setAt(RightSide, right); }
    void setBottom(const T& bottom) { setAt(BottomSide, bottom); }
    void setLeft(const T& left) { setAt(LeftSide, left); }

    T& before(WritingMode writingMode)
    {
        return at(mapLogicalSideToPhysicalSide(writingMode, BeforeSide));
    }

    T& after(WritingMode writingMode)
    {
        return at(mapLogicalSideToPhysicalSide(writingMode, AfterSide));
    }

    T& start(WritingMode writingMode, TextDirection direction = LTR)
    {
        return at(mapLogicalSideToPhysicalSide(makeTextFlow(writingMode, direction), StartSide));
    }

    T& end(WritingMode writingMode, TextDirection direction = LTR)
    {
        return at(mapLogicalSideToPhysicalSide(makeTextFlow(writingMode, direction), EndSide));
    }

    const T& before(WritingMode writingMode) const
    {
        return at(mapLogicalSideToPhysicalSide(writingMode, BeforeSide));
    }

    const T& after(WritingMode writingMode) const
    {
        return at(mapLogicalSideToPhysicalSide(writingMode, AfterSide));
    }

    const T& start(WritingMode writingMode, TextDirection direction = LTR) const
    {
        return at(mapLogicalSideToPhysicalSide(makeTextFlow(writingMode, direction), StartSide));
    }

    const T& end(WritingMode writingMode, TextDirection direction = LTR) const
    {
        return at(mapLogicalSideToPhysicalSide(makeTextFlow(writingMode, direction), EndSide));
    }

    void setBefore(const T& before, WritingMode writingMode) { this->before(writingMode) = before; }
    void setAfter(const T& after, WritingMode writingMode) { this->after(writingMode) = after; }
    void setStart(const T& start, WritingMode writingMode, TextDirection direction = LTR) { this->start(writingMode, direction) = start; }
    void setEnd(const T& end, WritingMode writingMode, TextDirection direction = LTR) { this->end(writingMode, direction) = end; }

    bool operator==(const BoxExtent& other) const
    {
        return m_sides == other.m_sides;
    }

    bool operator!=(const BoxExtent& other) const
    {
        return m_sides != other.m_sides;
    }

protected:
    std::array<T, 4> m_sides;
};
#endif

struct LengthBox {
    LengthBox()
    {
    }

    explicit LengthBox(LengthType type)
        : m_left(type)
        , m_right(type)
        , m_top(type)
        , m_bottom(type)
    {
    }

    explicit LengthBox(int v)
        : m_left(Length(v, Fixed))
        , m_right(Length(v, Fixed))
        , m_top(Length(v, Fixed))
        , m_bottom(Length(v, Fixed))
    {
    }

    LengthBox(const Length& top, const Length& right, const Length& bottom, const Length& left)
        : m_left(left)
        , m_right(right)
        , m_top(top)
        , m_bottom(bottom)
    {
    }

    LengthBox(int top, int right, int bottom, int left)
        : m_left(Length(left, Fixed))
        , m_right(Length(right, Fixed))
        , m_top(Length(top, Fixed))
        , m_bottom(Length(bottom, Fixed))
    {
    }

    const Length& left() const { return m_left; }
    const Length& right() const { return m_right; }
    const Length& top() const { return m_top; }
    const Length& bottom() const { return m_bottom; }

    const Length& logicalLeft(const RenderStyle*) const;
    const Length& logicalRight(const RenderStyle*) const;

    const Length& before(const RenderStyle*) const;
    const Length& after(const RenderStyle*) const;
    const Length& start(const RenderStyle*) const;
    const Length& end(const RenderStyle*) const;

    bool operator==(const LengthBox& other) const
    {
        return m_left == other.m_left && m_right == other.m_right && m_top == other.m_top && m_bottom == other.m_bottom;
    }

    bool operator!=(const LengthBox& other) const
    {
        return !(*this == other);
    }

    bool nonZero() const
    {
        return !(m_left.isZero() && m_right.isZero() && m_top.isZero() && m_bottom.isZero());
    }

    Length m_left;
    Length m_right;
    Length m_top;
    Length m_bottom;
};

} // namespace WebCore

#endif // LengthBox_h
