/*
 * Copyright (C) 2016 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef URLParser_h
#define URLParser_h

#include "TextEncoding.h"
#include "KURL.h"
#include <wtf/Forward.h>
#include <wtf/text/StringBuilder.h>

namespace WebCore {

class URLParser {
public:
	URLParser()
	: m_urlIsSpecial(false)
	, m_hostHasPercentOrNonASCII(false)
	{
	}

    KURL parse(const String&, const KURL& = KURL(), const TextEncoding& = UTF8Encoding());
    static bool allValuesEqual(const KURL&, const KURL&);

    static bool enabled();
    static void setEnabled(bool);

    typedef Vector<std::pair<String, String>> URLEncodedForm;
    static URLEncodedForm parseURLEncodedForm(StringView);
    static String serialize(const URLEncodedForm&);

private:
    KURL m_url;
    StringBuilder m_buffer;
    bool m_urlIsSpecial;
    bool m_hostHasPercentOrNonASCII;
    void parseAuthority(StringView::CodePoints::Iterator&, const StringView::CodePoints::Iterator& end);
    bool parseHost(StringView::CodePoints::Iterator&, const StringView::CodePoints::Iterator& end);
    bool parsePort(StringView::CodePoints::Iterator&, const StringView::CodePoints::Iterator& end);
    KURL failure(const String& input);

    enum class URLPart {
        SchemeEnd,
        UserStart,
        UserEnd,
        PasswordEnd,
        HostEnd,
        PortEnd,
        PathAfterLastSlash,
        PathEnd,
        QueryEnd,
        FragmentEnd,
    };
    void copyURLPartsUntil(const KURL& base, URLPart);
    static size_t urlLengthUntilPart(const KURL&, URLPart);
    void popPath();
};

}

#endif // URLParser_h
