/*
 *  Copyright (C) 2007 Holger Hans Peter Freyther
 *  Copyright (C) 2010 Igalia S.L
 * Portions Copyright (c) 2010 Motorola Mobility, Inc.  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "config.h"

#if ENABLE(CONTEXT_MENUS)

#include "ContextMenuItem.h"

#include "ContextMenu.h"
#include "Element.h"
#include "NotImplemented.h"
#include <wtf/text/CString.h>

#define WEBKIT_CONTEXT_MENU_ACTION "webkit-context-menu"

namespace WebCore {

static PlatformMenuItemDescription createPlatformMenuItemDescription(ContextMenuItemType type, ContextMenuAction action, const String& title, bool enabled, bool checked)
{
    return 0;
}

// Extract the ActionType from the menu item
ContextMenuItem::ContextMenuItem(PlatformMenuItemDescription item)
    : m_platformDescription(item)
{
}

ContextMenuItem::ContextMenuItem(ContextMenu* subMenu)
{
    m_platformDescription = 0;
    if (subMenu)
        setSubMenu(subMenu);
}

ContextMenuItem::ContextMenuItem(ContextMenuItemType type, ContextMenuAction action, const String& title, ContextMenu* subMenu)
{
    m_platformDescription = createPlatformMenuItemDescription(type, action, title, true, false);
    if (subMenu)
        setSubMenu(subMenu);
}

ContextMenuItem::ContextMenuItem(ContextMenuItemType type, ContextMenuAction action, const String& title, bool enabled, bool checked)
{
    m_platformDescription = createPlatformMenuItemDescription(type, action, title, enabled, checked);
}

ContextMenuItem::ContextMenuItem(ContextMenuAction action, const String& title, bool enabled, bool checked, Vector<ContextMenuItem>& subMenuItems)
{
    m_platformDescription = createPlatformMenuItemDescription(SubmenuType, action, title, enabled, checked);
    setSubMenu(subMenuItems);
}

ContextMenuItem::~ContextMenuItem()
{
}

PlatformMenuItemDescription ContextMenuItem::releasePlatformDescription()
{
    PlatformMenuItemDescription platformDescription = m_platformDescription;
    m_platformDescription = 0;
    return platformDescription;
}

ContextMenuItemType ContextMenuItem::type() const
{
    return ActionType;
}

void ContextMenuItem::setType(ContextMenuItemType type)
{
}

ContextMenuAction ContextMenuItem::action() const
{
    return ContextMenuItemTagNoAction;
}

void ContextMenuItem::setAction(ContextMenuAction action)
{
}

String ContextMenuItem::title() const
{
    return String();
}

void ContextMenuItem::setTitle(const String& title)
{
}

PlatformMenuDescription ContextMenuItem::platformSubMenu() const
{
    return 0;
}

void ContextMenuItem::setSubMenu(ContextMenu* menu)
{
}

void ContextMenuItem::setSubMenu(Vector<ContextMenuItem>& subMenuItems)
{
}

void ContextMenuItem::setChecked(bool shouldCheck)
{
}

bool ContextMenuItem::checked() const
{
    return false;
}

bool ContextMenuItem::enabled() const
{
    return false;
}

void ContextMenuItem::setEnabled(bool shouldEnable)
{
}

}

#endif // ENABLE(CONTEXT_MENUS)
