/*
 * Copyright (C) 2008 Apple Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef ScrollbarThemeEspial_h
#define ScrollbarThemeEspial_h

#include "ScrollbarThemeComposite.h"

namespace WebCore {

class Scrollbar;

class ScrollbarThemeEspial : public ScrollbarThemeComposite {
public:
    ScrollbarThemeEspial();
    virtual ~ScrollbarThemeEspial();

    virtual bool hasButtons(ScrollbarThemeClient*) { return true; }
    virtual bool hasThumb(ScrollbarThemeClient*);

    virtual IntRect backButtonRect(ScrollbarThemeClient*, ScrollbarPart, bool);
    virtual IntRect forwardButtonRect(ScrollbarThemeClient*, ScrollbarPart, bool);
    virtual IntRect trackRect(ScrollbarThemeClient*, bool);

    bool paint(ScrollbarThemeClient*, GraphicsContext*, const IntRect& damageRect);

    virtual int scrollbarThickness(ScrollbarControlSize);

protected:
	void paintArrow(GraphicsContext* ctx, const IntRect& rect, Image* image, float rotation);
	
	void platformInit();

	int separatorWidth() const {
		return m_separatorWidth < 0 ? m_borderWidth : m_separatorWidth;
	}
	
	int m_borderWidth;
	int m_railWidth;
	int m_separatorWidth;
	Color m_bgColor;
	Color m_railColor;
	Color m_borderColor;
	Color m_arrowBgColor;
	Color m_arrowColor;
	Color m_separatorColor;
	Color m_thumbColor;

	static RefPtr<Image> s_scrollArrowUp;
	static RefPtr<Image> s_scrollArrowDown;
	static RefPtr<Image> s_scrollArrowLeft;
	static RefPtr<Image> s_scrollArrowRight;
	static RefPtr<Image> s_scrollBackground;
	static RefPtr<Image> s_scrollRail;
	static RefPtr<Image> s_scrollThumb;
};

}
#endif
