/*
 * Copyright (C) 2012 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef AudioTrackPrivate_h
#define AudioTrackPrivate_h

#include "TrackPrivateBase.h"

#if ENABLE(VIDEO_TRACK)

namespace WebCore {

class AudioTrackPrivate;

class AudioTrackPrivateClient : public TrackPrivateBaseClient {
public:
    virtual void enabledChanged(AudioTrackPrivate*, bool) = 0;
};

class AudioTrackPrivate : public TrackPrivateBase {
public:
    enum Kind { Alternative, Description, Main, MainDesc, Translation, Commentary, None };

    static PassRefPtr<AudioTrackPrivate> create(bool enabled = false, const String& language = String(), Kind kind = None)
    {
        return adoptRef(new AudioTrackPrivate(enabled, language, kind));
    }

    void setClient(AudioTrackPrivateClient* client) { m_client = client; }
    virtual AudioTrackPrivateClient* client() const OVERRIDE { return m_client; }

    virtual void setEnabled(bool enabled)
    {
        if (m_enabled == enabled)
            return;
        m_enabled = enabled;
        if (m_client)
            m_client->enabledChanged(this, enabled);
    };
    virtual bool enabled() const { return m_enabled; }

    virtual Kind kind() const { return m_kind; }
    void setKind(Kind kind) { m_kind = kind; }

    virtual AtomicString language() const { return m_language; }

protected:
    AudioTrackPrivate(bool enabled, const String& language, Kind kind)
    : m_client(0)
    , m_enabled(enabled)
	, m_language(language)
	, m_kind(kind)
    {
    }

private:
    AudioTrackPrivateClient* m_client;
    bool m_enabled;
    String m_language;
    Kind m_kind;
};

} // namespace WebCore

#endif
#endif
