/*
 * Copyright (C) 2012 Igalia S.L.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301 USA
 */

#ifndef GLContext_h
#define GLContext_h

#include "GraphicsContext3D.h"
#include "Widget.h"
#include <wtf/Noncopyable.h>
#include <wtf/PassOwnPtr.h>

struct EspialContext;

namespace WebCore {

class GLContext {
    WTF_MAKE_NONCOPYABLE(GLContext); WTF_MAKE_FAST_ALLOCATED;
public:
    static PassOwnPtr<GLContext> createOffscreenContext(bool isDirectGLMode);

    ~GLContext();
    bool makeContextCurrent();
    void swapBuffers() { }
    bool canRenderToDefaultFramebuffer() { return false; }
    const IntSize& defaultFrameBufferSize() const { return m_defaultFrameBufferSize; }
    void markContextChanged();
    bool isGLES2Compliant() const { return m_isGLES2Compliant; }

    unsigned bufferDataOptimizedSize() const { return m_bufferDataOptimizedSize; }
    unsigned cachedBufferNumber() const { return m_cachedBufferNumber; }

    void didCallUseProgram();

#if USE(3D_GRAPHICS)
    PlatformGraphicsContext3D platformContext();
#endif

private:
    GLContext(bool isDirectGLMode);

    EspialContext* m_platformContext;
    bool m_isDirectGLMode;
    bool m_isGLES2Compliant;
    IntSize m_defaultFrameBufferSize;
    unsigned m_bufferDataOptimizedSize;
    unsigned m_cachedBufferNumber;
};

} // namespace WebCore

#endif // GLContext_h
