/*
 * Copyright (C) 2008, 2012 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GeneratorGeneratedImage_h
#define GeneratorGeneratedImage_h

#include "FloatSize.h"
#include "GeneratedImage.h"
#include "Gradient.h"
#include "Image.h"
#include "ImageBuffer.h"
#include "Timer.h"
#include <wtf/RefPtr.h>

namespace WebCore {

class CachedGeneratorGeneratedImageBuffer : public RefCounted<CachedGeneratorGeneratedImageBuffer>
{
public:
	static PassRefPtr<CachedGeneratorGeneratedImageBuffer> create(PassRefPtr<Image> image, unsigned generatorHash, const FloatSize& size)
    {
        return adoptRef(new CachedGeneratorGeneratedImageBuffer(image, generatorHash, size));
    }

	~CachedGeneratorGeneratedImageBuffer()
	{
	}

	void scheduleEvict(double interval);

	Image* image() { return m_image.get(); }
	const FloatSize& adjustedSize() const { return m_adjustedSize; }
	unsigned generatorHash() const { return m_generatorHash; }

private:
	CachedGeneratorGeneratedImageBuffer(PassRefPtr<Image> image, unsigned generatorHash, const FloatSize& size)
    : m_image(image)
	, m_timer(this, &CachedGeneratorGeneratedImageBuffer::invalidateCacheTimerFired)
	, m_adjustedSize(size)
    , m_generatorHash(generatorHash)
    {
    }

    void invalidateCacheTimerFired(Timer<CachedGeneratorGeneratedImageBuffer>*);

private:
    RefPtr<Image> m_image;
    Timer<CachedGeneratorGeneratedImageBuffer> m_timer;
    FloatSize m_adjustedSize;
    unsigned m_generatorHash;
};

class GradientImage : public GeneratedImage {
public:
    static PassRefPtr<GradientImage> create(PassRefPtr<Gradient> generator, const FloatSize& size)
    {
        return adoptRef(new GradientImage(generator, size));
    }

    virtual ~GradientImage();

protected:
    virtual void draw(GraphicsContext*, const FloatRect& dstRect, const FloatRect& srcRect, CompositeOperator, BlendMode);
    virtual void drawPattern(GraphicsContext*, const FloatRect& srcRect, const AffineTransform& patternTransform,
    	const FloatPoint& phase, const FloatSize& spacing, CompositeOperator, const FloatRect& destRect, BlendMode);

    GradientImage(PassRefPtr<Gradient> generator, const FloatSize& size);

    RefPtr<Gradient> m_gradient;
};

void clearCachedGeneratorGeneratedImageBuffers();

}

#endif
