/*
 * Copyright (C) 2011 Igalia S.L.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301 USA
 */

#ifndef GraphicsContext3DPrivate_h
#define GraphicsContext3DPrivate_h

#include "GLContext.h"
#include "GraphicsContext3D.h"
#include <wtf/PassOwnPtr.h>

#if USE(ACCELERATED_COMPOSITING) && USE(TEXTURE_MAPPER)
#include "TextureMapper.h"
#endif

namespace WebCore {

class GraphicsContext3DPrivate :
#if USE(ACCELERATED_COMPOSITING) && USE(TEXTURE_MAPPER)
    public TextureMapperPlatformLayer
#elif USE(ACCELERATED_COMPOSITING)
    public PlatformLayer
#endif
{
	WTF_MAKE_FAST_ALLOCATED;
public:
    static PassOwnPtr<GraphicsContext3DPrivate> create(GraphicsContext3D*, bool isDirectGLMode);
    virtual ~GraphicsContext3DPrivate();

    bool makeContextCurrent();
    PlatformGraphicsContext3D platformContext();
    ALWAYS_INLINE GraphicsContext3D* context() {return m_context;}

    IntSize defaultFrameBufferSize();

    unsigned bufferDataOptimizedSize() const;
    unsigned cachedBufferNumber() const;

    void didCallUseProgram();

#if USE(ACCELERATED_COMPOSITING) && USE(TEXTURE_MAPPER)
    virtual void paintToTextureMapper(TextureMapper*, const FloatRect& target, const TransformationMatrix&, float opacity, BitmapTexture* mask);
#endif

#if USE(ACCELERATED_COMPOSITING)
    virtual void markComposited();
    virtual uint32_t glTexture();
    virtual bool flipped();
    virtual void markContextChanged();
#endif

    bool isGLES2Compliant();

private:
    GraphicsContext3DPrivate(GraphicsContext3D*, bool isDirectGLMode);

    GraphicsContext3D* m_context;
    OwnPtr<GLContext> m_glContext;
};

ALWAYS_INLINE bool GraphicsContext3DPrivate::makeContextCurrent()
{
    return m_glContext->makeContextCurrent();
}

ALWAYS_INLINE PlatformGraphicsContext3D GraphicsContext3DPrivate::platformContext()
{
    return m_glContext->platformContext();
}

#if USE(ACCELERATED_COMPOSITING)
ALWAYS_INLINE void GraphicsContext3DPrivate::markComposited()
{
#if USE(3D_GRAPHICS)
    context()->markLayerComposited();
#endif
}

ALWAYS_INLINE uint32_t GraphicsContext3DPrivate::glTexture()
{
    return context()->platformTexture();
}

ALWAYS_INLINE bool GraphicsContext3DPrivate::flipped()
{
    return true;
}

ALWAYS_INLINE void GraphicsContext3DPrivate::markContextChanged()
{
	m_glContext->markContextChanged();
}
#endif

ALWAYS_INLINE IntSize GraphicsContext3DPrivate::defaultFrameBufferSize()
{
    return m_glContext->defaultFrameBufferSize();
}

ALWAYS_INLINE unsigned GraphicsContext3DPrivate::bufferDataOptimizedSize() const
{
    return m_glContext->bufferDataOptimizedSize();
}

ALWAYS_INLINE unsigned GraphicsContext3DPrivate::cachedBufferNumber() const
{
    return m_glContext->cachedBufferNumber();
}

ALWAYS_INLINE bool GraphicsContext3DPrivate::isGLES2Compliant()
{
	return m_glContext->isGLES2Compliant();
}

ALWAYS_INLINE void GraphicsContext3DPrivate::didCallUseProgram()
{
	m_glContext->didCallUseProgram();
}

ALWAYS_INLINE bool GraphicsContext3D::makeContextCurrent()
{
    return m_private->makeContextCurrent();
}

}

#endif // GraphicsContext3DPrivate_h
