/*
 * Copyright (C) 2007, 2009 Apple Inc.  All rights reserved.
 * Copyright (C) 2007 Collabora Ltd. All rights reserved.
 * Copyright (C) 2007 Alp Toker <alp@atoker.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * aint with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef MediaPlayerPrivateMangareva_h
#define MediaPlayerPrivateMangareva_h

#if ENABLE(VIDEO)

#include "MediaPlayerPrivate.h"
#include "Timer.h"

namespace WebCore {

    class GraphicsContext;
    class IntSize;
    class IntRect;
    class String;
    class MediaPlayerPrivate;

    class MediaPlayerPrivateVariables;

    void mediaPlayerPrivateEOSCallback(void* context, int param);
    void mediaPlayerPrivateStateChangedCallback(void* context, int param);

    class __attribute__((visibility("default"))) MediaPlayerPrivateBC : public MediaPlayerPrivateInterface
    {
    public:
        friend void mediaPlayerPrivateEOSCallback(void* context, int param);
        friend void mediaPlayerPrivateStateChangedCallback(void* context, int param);
        friend void mediaPlayerPrivateUriChangedCallback(void* context, int param);
        static bool mediaPlayerPrivateResourceRequest(MediaPlayerPrivateBC* mediaPlayerPrivate);
	static MediaPlayerPrivateBC* getCurrentMediaPlayerPrivate();

        __attribute__((visibility("default"))) static void registerMediaEngine(MediaEngineRegistrar);
        ~MediaPlayerPrivateBC();

        virtual bool supportsFullscreen() const { return true; }
        IntSize naturalSize() const;
        bool hasVideo() const;
        __attribute__((visibility("default"))) bool hasAudio() const;

#if ENABLE(DAE)
        void load(PassRefPtr<Channel>);
#endif
        void load(const KURL &url);
        void cancelLoad();

        void play();
        void pause();

        bool paused() const;
        bool seeking() const;

        float duration() const;
        float currentTime() const;
        void seek(float);
        void setEndTime(float);

        void setRate(float);
        void setVolume(float);
        void setMuted(bool);

        int dataRate() const;
        float rate();

        MediaPlayer::NetworkState networkState() const;
        MediaPlayer::ReadyState readyState() const;

        PassRefPtr<TimeRanges> buffered() const;
        float maxTimeBuffered() const;
        float maxTimeSeekable() const;
        unsigned bytesLoaded() const;
        bool totalBytesKnown() const;
        unsigned totalBytes() const;

        void setVisible(bool);
        void setSize(const IntSize&);

        void setBounds(IntRect bounds);

        void loadStateChanged();
        void rateChanged();
        void sizeChanged();
        void timeChanged();
        void volumeChanged();
        void didEnd();
        void loadingFailed();

        void repaint();
        void paint(GraphicsContext*, const IntRect&);

        void stopInterrupt();

        static void notifyTimeChanged(void* context);
        static void notifyError(void* context);
        static void notifyInterrupt(void* context);
        MediaPlayer* player() const { return m_player; }

#if ENABLE(CEHTML_VIDEO)
        void stop();
        void setAspect(MediaPlayer::VideoScale);
        WTF::Vector<float> playSpeeds();
#endif

    private:
        MediaPlayerPrivateBC(MediaPlayer*);
        static MediaPlayerPrivateInterface* create(MediaPlayer* player);

        static void getSupportedTypes(HashSet<String>&);
        static MediaPlayer::SupportsType supportsType(const String& type, const String& codecs);
        static bool isAvailable() { return true; }

        void updateStates();
        void cancelSeek();
        void endPointTimerFired(Timer<MediaPlayerPrivateBC>*);
        float maxTimeLoaded() const;
        void startEndPointTimerIfNeeded();

        void stateTimerCallback(Timer<MediaPlayerPrivateBC>*);

    private:
        MediaPlayer* m_player;
        MediaPlayer::NetworkState m_networkState;
        MediaPlayer::ReadyState m_readyState;
        float m_rate;
	float m_endTime;
	double m_volume;
	bool m_startedPlaying;
	mutable bool m_isStreaming;
	IntSize m_size;
        IntRect m_bounds;
        IntRect m_playerBounds;
	bool m_visible;

	MediaPlayerPrivateVariables* m_privateVariables;
    };
}

#endif // ENABLE(VIDEO)
#endif // MediaPlayerPrivateMangareva_h
