/*
 * Copyright (C) 2008 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "PlatformContextCairo.h"

#if USE(ACCELERATED_COMPOSITING)
#include "PlatformLayer.h"
#endif

typedef struct _cairo_surface cairo_surface_t;

struct EspialTexture;
struct EspialSurface;

namespace WebCore {

class IntSize;

class ImageBufferData
#if USE(ACCELERATED_COMPOSITING)
: public PlatformLayer
#endif
{
	WTF_MAKE_FAST_ALLOCATED;
public:
    ImageBufferData(const IntSize&);
    ImageBufferData(const IntSize&, bool /*forceImageSurface*/);

#if USE(ACCELERATED_COMPOSITING)
    virtual
#endif
    ~ImageBufferData();

#if USE(ACCELERATED_COMPOSITING)
    bool isAccelerated() {return m_espialTexture != NULL;}
    virtual void markDirty();
    virtual uint32_t glTexture();
    virtual uint32_t webglTexture();
    virtual bool flipped() { return false; }
#endif

    cairo_surface_t* m_surface;
    PlatformContextCairo m_platformContext;

    bool m_needFlush;

private:
#if USE(ACCELERATED_COMPOSITING)
    EspialTexture* m_espialTexture;
#endif
};

} // namespace WebCore
