/*
 *  Copyright (C) 2010 Igalia S.L.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef RefPtrCairo_h
#define RefPtrCairo_h

#include <wtf/RefPtr.h>
#include <cairo.h>

#if USE(FREETYPE)
#include <cairo-ft.h>
#include <fontconfig/fcfreetype.h>
#endif

namespace WTF {

template<> ALWAYS_INLINE void refIfNotNull(cairo_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_reference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(cairo_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_destroy(ptr);
}

template<> ALWAYS_INLINE void refIfNotNull(cairo_surface_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_surface_reference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(cairo_surface_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_surface_destroy(ptr);
}

template<> ALWAYS_INLINE void refIfNotNull(cairo_font_face_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_font_face_reference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(cairo_font_face_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_font_face_destroy(ptr);
}

template<> ALWAYS_INLINE void refIfNotNull(cairo_scaled_font_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_scaled_font_reference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(cairo_scaled_font_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_scaled_font_destroy(ptr);
}

template<> ALWAYS_INLINE void refIfNotNull(cairo_pattern_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_pattern_reference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(cairo_pattern_t* ptr)
{
    if (LIKELY(ptr != 0))
        cairo_pattern_destroy(ptr);
}

template<> ALWAYS_INLINE void refIfNotNull(cairo_region_t* ptr)
{
    if (LIKELY(ptr))
        cairo_region_reference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(cairo_region_t* ptr)
{
    if (LIKELY(ptr))
        cairo_region_destroy(ptr);
}

#if USE(FREETYPE)
template<> ALWAYS_INLINE void refIfNotNull(FcPattern* ptr)
{
    if (LIKELY(ptr))
        FcPatternReference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(FcPattern* ptr)
{
    if (LIKELY(ptr))
        FcPatternDestroy(ptr);
}

template<> ALWAYS_INLINE void refIfNotNull(FcConfig* ptr)
{
    if (LIKELY(ptr))
        FcConfigReference(ptr);
}

template<> ALWAYS_INLINE void derefIfNotNull(FcConfig* ptr)
{
    if (LIKELY(ptr))
        FcConfigDestroy(ptr);
}
#endif

}

#endif // RefPtrCairo_h
