
#include "config.h"
#include "MediaPlayerPrivateEvo.h"
#include "ContentType.h"
#include "HTMLMediaElement.h"

namespace WebCore
{

DLL_PUBLIC void MediaPlayerPrivate::handlePlayKey()
{
	HTMLMediaElement* mediaElement = m_player->mediaPlayerClient()->mediaPlayerOwningElement();
	if(mediaElement){
		mediaElement->pause();
		mediaElement->setPlaybackRate(1.0f);
		mediaElement->play();
	}
}

DLL_PUBLIC void MediaPlayerPrivate::handlePauseKey()
{
	HTMLMediaElement* mediaElement = m_player->mediaPlayerClient()->mediaPlayerOwningElement();
	if(mediaElement){
		mediaElement->pause();
	}
}

DLL_PUBLIC void MediaPlayerPrivate::handleFastForwardKey(float factor)
{
	HTMLMediaElement* mediaElement = m_player->mediaPlayerClient()->mediaPlayerOwningElement();
	if(mediaElement){
		float playbackRate = mediaElement->playbackRate();

		if(playbackRate > 0){
			playbackRate *= factor;
		}
		else{
			playbackRate = factor;
		}

		mediaElement->setPlaybackRate(playbackRate);
	}
}

DLL_PUBLIC void MediaPlayerPrivate::handleFastBackKey(float factor)
{
	HTMLMediaElement* mediaElement = m_player->mediaPlayerClient()->mediaPlayerOwningElement();

	if(mediaElement){
		float playbackRate = mediaElement->playbackRate();

		if(playbackRate > 0){
			playbackRate = -factor;
		}
		else{
			playbackRate *= factor;
		}

		mediaElement->setPlaybackRate(playbackRate);
	}
}

DLL_PUBLIC bool MediaPlayerPrivate::supportsMIMEType(const String& mimeType)
{
	ContentType contentType(mimeType);

	switch(MediaPlayer::supportsType(contentType, String(), KURL(), true, 0))
	{
		case MediaPlayer::IsNotSupported:
			return false;

		case MediaPlayer::IsSupported:
		case MediaPlayer::MayBeSupported:
			return true;

		default:
			return false;
	}
}

}

