/*
 * Copyright (C) Research In Motion Limited 2009. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef FontPlatformData_h
#define FontPlatformData_h

#include "FontOrientation.h"
#include "FontWidthVariant.h"
#include "GlyphBuffer.h"

#include <wtf/HashFunctions.h>

#include <wtf/Forward.h>
#include <wtf/HashMap.h>
#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/RetainPtr.h>
#include <wtf/text/StringImpl.h>
//#include <wtf/text/AtomicString.h>

#include "FontPlatformDataPrivate.h"
#include "PlatformString.h"
#include "NotImplemented.h"

#include <fontconfig/fontconfig.h>
#if USE(FREETYPE)
#include <ft2build.h>
#include FT_FREETYPE_H
#include <fontconfig/fcfreetype.h>
#endif

#include <openvg.h>
#include <wtf/RefPtr.h>

namespace WebCore {

typedef HashMap<UChar32, UChar32> FontCharReplaceMap;

class PrivateFont;
class FontDescription;
class SharedBuffer;


class FontPlatformData {
	WTF_MAKE_FAST_ALLOCATED;
public:
    FontPlatformData(WTF::HashTableDeletedValueType);
    FontPlatformData(float size=0, bool bold=false, bool italic=false);
    FontPlatformData(const FontDescription&, PassRefPtr<PrivateFont>);
    FontPlatformData(const FontDescription& fontDescription, FcPattern* pattern, const String& familyName = String());
    FontPlatformData(SharedBuffer* buffer, float size, bool bold, bool italic);

    bool isValid() const;

    bool decodeGlyph(UChar32 character) const;
    void uploadGlyph(UChar32 character) const;
    float widthForGlyph(UChar32 character) const;
    float kerningDistance(UChar32 leftCharacter, UChar32 rightCharacter) const;

#ifdef OPENVG_VERSION_1_1
    VGFont vgFont() const;
#endif
#if USE(FREETYPE)
    FT_Face ftFace() const;
#endif

    FontPlatformData(const FontPlatformData&);
    FontPlatformData& operator=(const FontPlatformData&);

    void invalidate() const;

    // TODO: actually implement these
    FontOrientation orientation() const { return Horizontal; }
    FontWidthVariant widthVariant() const { return RegularWidth; }
    void setOrientation(FontOrientation orientation) { notImplemented(); }

    static bool init();

    bool bold() const {
    	return m_private->m_bold;
    }
    bool italic() const {
    	return m_private->m_italic;
    }

    float syntheticBoldOffset() const {
    	return bold() ? 1.0f : 0.0f;
    }

    bool operator==(const FontPlatformData& other) const;

#ifndef NDEBUG
    String description() const;
#endif

#if USE(FREETYPE)
    static FT_Library ftLibrary();
#endif

    int size() const;

    FontCharReplaceMap& charReplaceMap() { return m_charReplaceMap; }
    const FontCharReplaceMap& charReplaceMap() const { return m_charReplaceMap; }

private:
    RefPtr<FontPlatformDataPrivate> m_private;

    FontCharReplaceMap m_charReplaceMap;
};

} // namespace WebCore

#endif

