/*
 * Copyright (C) Research In Motion Limited 2009. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef ImageBufferData_h
#define ImageBufferData_h

#include <ColorSpace.h>
#include "SurfaceOpenVG.h"
#include <runtime/Uint8ClampedArray.h>

namespace WebCore {

class ImageBufferData {
	WTF_MAKE_FAST_ALLOCATED;
public:
    ImageBufferData(const IntSize& size, ColorSpace colorSpace, bool accelerated, bool& success);
    ~ImageBufferData();

    PassRefPtr<JSC::Uint8ClampedArray> getImageData(GraphicsContext* context, const IntRect& rect, const IntSize& size, VGImageFormat format) const;

    void putImageData(GraphicsContext* context, JSC::Uint8ClampedArray*& source, const IntSize& sourceSize,
		const IntRect& sourceRect, const IntPoint& destPoint,
		const IntSize& size, VGImageFormat format);

    void transformColorSpace(const Vector<int>& lookUpTable);

    VGImageFormat m_format;
    ColorSpace m_colorSpace;
    SurfaceOpenVG* m_surface;
    RefPtr<Image> m_cachedImage;
};

} // namespace WebCore

#endif  // ImageBufferData_h
