/*
 * Copyright (C) Research In Motion Limited 2009. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"
#include "SimpleFontData.h"
#include "FontPlatformData.h"

#include "NotImplemented.h"

#if USE(FREETYPE)
#include <ft2build.h>
#include FT_FREETYPE_H
#endif

namespace WebCore {

PassRefPtr<SimpleFontData> SimpleFontData::smallCapsFontData(const FontDescription& fontDescription) const
{
    notImplemented();
    return NULL;
}

PassRefPtr<SimpleFontData> SimpleFontData::emphasisMarkFontData(const FontDescription& fontDescription) const
{
    notImplemented();
    return NULL;
}

PassRefPtr<SimpleFontData> SimpleFontData::platformCreateScaledFontData(const FontDescription& fontDescription, float scaleFactor) const
{
    notImplemented();
	return 0;
}

void SimpleFontData::determinePitch()
{
    notImplemented();
}

void SimpleFontData::platformInit()
{
#if USE(FREETYPE)
    if (platformData().isValid()) {
        FT_Face ftFace = platformData().ftFace();
        ASSERT(ftFace);
		if (!ftFace)
			return;

        // FT_MulFix() converts from font units to 26.6 (1/64px) units.
        // The addition of 0.5 is done to get correct integer rounding.
        m_fontMetrics.m_ascent = FT_MulFix(ftFace->ascender, ftFace->size->metrics.y_scale) / 64.f;
        m_fontMetrics.m_descent = -(FT_MulFix(ftFace->descender, ftFace->size->metrics.y_scale) / 64.f); // usually negative
        m_fontMetrics.m_lineSpacing = FT_MulFix(ftFace->height, ftFace->size->metrics.y_scale) / 64.f;
        m_fontMetrics.m_lineGap = m_fontMetrics.m_lineSpacing - m_fontMetrics.m_ascent - m_fontMetrics.m_descent;
        //m_fontMetrics.m_maxCharWidth = FT_MulFix(ftFace->max_advance_width, ftFace->size->metrics.x_scale) / 64.f;
        m_fontMetrics.m_unitsPerEm = FT_MulFix(ftFace->units_per_EM, ftFace->size->metrics.x_scale) / 64.f;

        FT_Error error = FT_Load_Char(ftFace, 'x', FT_LOAD_DEFAULT);
        ASSERT_UNUSED(error, !error);
        m_fontMetrics.m_xHeight = ftFace->glyph->metrics.height / 64.f;
        //m_fontMetrics.m_avgCharWidth = ftFace->glyph->advance.x / 64.f;

        error = FT_Load_Char(ftFace, ' ', FT_LOAD_DEFAULT);
        ASSERT(!error);
    }
#endif
}

void SimpleFontData::platformCharWidthInit()
{
    m_avgCharWidth = 0.f;
    m_maxCharWidth = 0.f;
    initCharWidths();
}

FloatRect SimpleFontData::platformBoundsForGlyph(Glyph) const
{
	// unimplemented in SimpleFontDataCairo
    return FloatRect();
}

void SimpleFontData::platformDestroy()
{
}

float SimpleFontData::platformWidthForGlyph(Glyph glyph) const
{
    return platformData().widthForGlyph(glyph);
}

}
