/*
 * Copyright (C) Research In Motion Limited 2009-2010. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef TiledImageOpenVG_h
#define TiledImageOpenVG_h

#include "IntRect.h"
#include "IntSize.h"
#include "SharedResourceOpenVG.h"

#include <openvg.h>
#include <wtf/RefCounted.h>
#include <wtf/Vector.h>

namespace WebCore {

class FloatRect;

class TiledImageOpenVG : public SharedResourceOpenVG, public RefCounted<TiledImageOpenVG> {
public:
    TiledImageOpenVG(const IntSize& size, const IntSize& tileSize);
    TiledImageOpenVG(const TiledImageOpenVG&);
    ~TiledImageOpenVG();

    TiledImageOpenVG& operator=(const TiledImageOpenVG&);

    const IntSize& size() const { return m_size; }
    const IntSize& maxTileSize() const { return m_maxTileSize; }

    int numTiles() const { return m_tiles.size(); }
    int numColumns() const { return m_numColumns; }
    int numRows() const { return m_tiles.size() / m_numColumns; }

    IntRect tilesInRect(const FloatRect&) const;

    /**
     * Replace the existing image tile with a new one. If this tile has already
     * been set to a valid VGImage, that VGImage is destroyed.
     */
    void setTile(int xIndex, int yIndex, VGImage);

    VGImage tile(int xIndex, int yIndex) const;
    IntRect tileRect(int xIndex, int yIndex) const;

    /**
     * Whether this tiled image should be mirrored vertically when drawn
     */
    bool inverted() const {return m_inverted;}

    void setInverted(bool inverted) {
        m_inverted = inverted;
    }

private:
    void detachTiles();
    void destroyTiles();

    IntSize m_size;
    IntSize m_maxTileSize;

    int m_numColumns;
    
    bool m_inverted;

    Vector<VGImage, 1> m_tiles;
};

}

#endif
