#ifndef SocketStreamHandle_h
#define SocketStreamHandle_h

#include "SocketStreamHandleBase.h"

#include <sys/types.h>
#include <errno.h>
#include <net/if.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/select.h>
//#include "StringImpl.h"
#include "KURL.h"
#include "Logging.h"
#include <wtf/Vector.h>
#include <pthread.h>
#include <wtf/OwnPtr.h>
#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>

//secure websocket
#include <openssl/rand.h>
#include <openssl/ssl.h>
#include <openssl/err.h>


namespace WebCore {

    class AuthenticationChallenge;
    class Credential;
    class SocketStreamHandleClient;
    class KURL;
    class SocketStreamHandleBase;

    class SocketStreamHandle : public RefCounted<SocketStreamHandle>, public SocketStreamHandleBase {
    public:
        static PassRefPtr<SocketStreamHandle> create(const KURL& url, SocketStreamHandleClient* client) { return adoptRef(new SocketStreamHandle(url, client)); }

        static PassRefPtr<SocketStreamHandle> create(int socketfd, SocketStreamHandleClient* client) { return adoptRef(new SocketStreamHandle(socketfd, client)); }

        virtual ~SocketStreamHandle();

    protected:
        virtual int platformSend(const char* data, int length);
        virtual void platformClose();

    private:
        SocketStreamHandle(const KURL&, SocketStreamHandleClient*);
        SocketStreamHandle(int socketfd, SocketStreamHandleClient*);

        // No authentication for streams per se, but proxy may ask for credentials.
        void didReceiveAuthenticationChallenge(const AuthenticationChallenge&);
        void receivedCredential(const AuthenticationChallenge&, const Credential&);
        void receivedRequestToContinueWithoutCredential(const AuthenticationChallenge&);
        void receivedCancellation(const AuthenticationChallenge&);

        void ReadtimerEvent(Timer<SocketStreamHandle>*);
        void OpentimerEvent(Timer<SocketStreamHandle>*);

        int m_socket;
        //struct sockaddr_in m_srvaddr;
        SSL* m_sslhandle;
        SSL_CTX* m_sslctx;
        bool m_isSecure;
        Vector<char> m_sslPendingBuffer;
        Timer<SocketStreamHandle> m_rdtimer;
        Timer<SocketStreamHandle> m_contimer;
    };

}  // namespace WebCore

#endif  // SocketStreamHandle_h
