/*
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "config.h"
#include "CookieJar.h"
#include "NotImplemented.h"
#include "Cookie.h"
#include "Document.h"
#include "Frame.h"
#include "ResourceHandleManager.h"

namespace WebCore {

void setCookies(Document& document, const KURL& url, const String& value)
{
    if(Frame *frame = document.frame()) {
		ResourceHandleManager& jar = ResourceHandleManager::sharedInstance();
		if (jar.cookieEnabled())
			jar.setCookies(url, value);
    }
}

String cookies(const Document& document, const KURL& url)
{
    if(Frame *frame = document.frame()) {
		ResourceHandleManager& jar = ResourceHandleManager::sharedInstance();
		if (jar.cookieEnabled()) {
			// omit httpOnly cookies
			return jar.cookies(url, false);
		}
    }

    return String();
}

String cookieRequestHeaderFieldValue(const Document& document, const KURL& url)
{
    if(Frame *frame = document.frame()) {
    	ResourceHandleManager& jar = ResourceHandleManager::sharedInstance();
    	if (jar.cookieEnabled()) {
    		// include httpOnly cookies
    		return jar.cookies(url, true);
    	}
    }
    return String();
}

String cookieRequestHeaderFieldValue(NetworkingContext*, const KURL&, const KURL& url)
{
    ResourceHandleManager& jar = ResourceHandleManager::sharedInstance();
    if (jar.cookieEnabled()) {
		// include httpOnly cookies
		return jar.cookies(url, true);
    }

    return String();
}

bool cookiesEnabled(const Document& document)
{
    if(Frame *frame = document.frame())
    	return ResourceHandleManager::sharedInstance().cookieEnabled();
    return false;
}

bool getRawCookies(const Document&, const KURL& url, Vector<Cookie>& rawCookies)
{	
    rawCookies.clear();
    
    return false; // return true when implemented
}

void deleteCookie(const Document&, const KURL& url, const String& name)
{
}


#if !PLATFORM(EFL)
void setCookieStoragePrivateBrowsingEnabled(bool enabled)
{
    // FIXME: Not yet implemented
}
#endif

void getHostnamesWithCookies(HashSet<String>& hostnames)
{
    // FIXME: Not yet implemented
}

void deleteCookiesForHostname(const String& hostname)
{
    // FIXME: Not yet implemented
}

void deleteAllCookies()
{
    // FIXME: Not yet implemented
}

}
