/*
 * Copyright (C) 2006, 2007 Apple Inc.  All rights reserved.
 * Copyright (C) 2008 Collabora Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"

#if ENABLE(NETSCAPE_PLUGIN_API)

#include "PluginView.h"
#include "Document.h"
#include "DocumentLoader.h"
#include "Element.h"
#include "FrameLoader.h"
#include "FrameLoadRequest.h"
#include "FrameTree.h"
#include "Frame.h"
#include "FrameView.h"
#include "Image.h"
#include "HTMLPlugInElement.h"
#include "WindowsKeyboardCodes.h"
#include "KeyboardEvent.h"
#include "MouseEvent.h"
#include "NotImplemented.h"
#include "Page.h"
#include "PluginDebug.h"
#include "PluginMainThreadScheduler.h"
#include "PluginPackage.h"
#include "RenderElement.h"
#include "JSDOMBinding.h"
#include "ScriptController.h"
#include "npruntime_impl.h"
#include "runtime_root.h"
#include <runtime/JSLock.h>
#include <runtime/JSValue.h>
#include <wtf/PlatformRunState.h>

using JSC::ExecState;
using JSC::Interpreter;
using JSC::JSLock;
using JSC::JSObject;

namespace WebCore
{

bool PluginView::platformGetValue(NPNVariable variable, void* value, NPError* result)
{
    switch (variable)
	{
    	case NPNVxDisplay:
		{
    		*result = NPERR_NO_ERROR;
    		return true;
		}
    	case NPNVxtAppContext:
		{
        	if (!m_needsXEmbed)
			{
        		*result = NPERR_NO_ERROR;
        	}
			else
			{
        		*result = NPERR_GENERIC_ERROR;
			}

        	return true;
		}
    	case NPNVWindowNPObject:
		{
        	if (m_isJavaScriptPaused)
			{
        		*result = NPERR_GENERIC_ERROR;
        	}
        	else
			{
            	NPObject* windowScriptObject = m_parentFrame->script()->windowScriptNPObject();

            	// Return value is expected to be retained, as described here:
            	//<http://www.mozilla.org/projects/plugin/npruntime.html>
            	if (windowScriptObject)
				{
                	_NPN_RetainObject(windowScriptObject);
				}

            	void** v = (void**)value;
            	*v = windowScriptObject;

            	*result = NPERR_NO_ERROR;
        	}

        	return true;
		}
    	case NPNVPluginElementNPObject:
		{
        	if (m_isJavaScriptPaused)
			{
        		*result = NPERR_GENERIC_ERROR;
        	}
        	else
			{
	        	NPObject* pluginScriptObject = 0;

	        	if( m_element->hasTagName(HTMLNames::appletTag) ||
	        		m_element->hasTagName(HTMLNames::embedTag) ||
	        		m_element->hasTagName(HTMLNames::objectTag))
				{
	            	pluginScriptObject = static_cast<HTMLPlugInElement*>(m_element)->getNPObject();
				}

	        	// Return value is expected to be retained, as described here:
	        	//<http://www.mozilla.org/projects/plugin/npruntime.html>
	        	if (pluginScriptObject)
				{
	            	_NPN_RetainObject(pluginScriptObject);
				}

	        	void** v = (void**)value;
	        	*v = pluginScriptObject;

	        	*result = NPERR_NO_ERROR;
        	}

        	return true;
		}
        case NPNVnetscapeWindow:
		{
            *result = NPERR_NO_ERROR;
            return true;
		}
        default:
		{
            return false;
		}
    }
}

}

#endif // ENABLE(NETSCAPE_PLUGIN_API)
