/*
 * Copyright (C) 2014 Gurpreet Kaur (k.gurpreet@samsung.com). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RenderMathMLMenclose_h
#define RenderMathMLMenclose_h

#if ENABLE(MATHML)
#include "MathMLMencloseElement.h"
#include "RenderMathMLRow.h"

namespace WebCore {
    
class RenderMathMLMenclose FINAL: public RenderMathMLRow {
public:
    RenderMathMLMenclose(Element&, PassRefPtr<RenderStyle>);

private:
    virtual const char* renderName() const FINAL { return "RenderMathMLMenclose"; }
    virtual void computePreferredLogicalWidths() FINAL;
    virtual void layoutBlock(bool relayoutChildren, LayoutUnit pageLogicalHeight = 0) FINAL;
    virtual Optional<int> firstLineBaseline() const FINAL;
    virtual void paint(PaintInfo&, const LayoutPoint&) FINAL;

    LayoutUnit ruleThickness() const;
    bool hasNotation(MathMLMencloseElement::MencloseNotationFlag notationFlag) const { return toMathMLMencloseElement(element())->hasNotation(notationFlag); }

    void getSpaceAroundContent(LayoutUnit contentWidth, LayoutUnit contentHeight, LayoutUnit& leftSpace, LayoutUnit& rightSpace, LayoutUnit& topSpace, LayoutUnit& bottomSpace) const;

    LayoutUnit m_ascent;
    LayoutRect m_contentRect;
};

template<> inline bool isRendererOfType<const RenderMathMLMenclose>(const RenderObject& renderer) { return renderer.isRenderMathMLMenclose(); }

RENDER_OBJECT_TYPE_CASTS(RenderMathMLMenclose, isRenderMathMLMenclose())

}

//SPECIALIZE_TYPE_TRAITS_RENDER_OBJECT(RenderMathMLMenclose, isRenderMathMLMenclose())

#endif // ENABLE(MATHML)
#endif // RenderMathMLMenclose_h
