/*
 * Copyright (C) 2009 Alex Milowski (alex@milowski.com). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RenderMathMLRoot_h
#define RenderMathMLRoot_h

#if ENABLE(MATHML)

#include "MathOperator.h"
#include "RenderMathMLBlock.h"
#include "RenderMathMLRow.h"

namespace WebCore {

class RenderMathMLMenclose;

// Render base^(1/index), or sqrt(base) using radical notation.
class RenderMathMLRoot : public RenderMathMLRow {

public:
    RenderMathMLRoot(Element&, PassRefPtr<RenderStyle>);
    virtual void styleDidChange(StyleDifference, const RenderStyle* oldStyle) FINAL;
    virtual void updateFromElement() FINAL;

    virtual void computePreferredLogicalWidths() FINAL;
    virtual void layoutBlock(bool relayoutChildren, LayoutUnit pageLogicalHeight = 0) FINAL;

protected:
    virtual void paint(PaintInfo&, const LayoutPoint&) FINAL;

private:
    bool isValid() const;
    RenderBox& getBase() const;
    RenderBox& getIndex() const;
    virtual bool isRenderMathMLRoot() const FINAL { return true; }
    virtual const char* renderName() const FINAL { return "RenderMathMLRoot"; }
    void updateStyle();

    MathOperator m_radicalOperator;
    LayoutUnit m_verticalGap;
    LayoutUnit m_ruleThickness;
    LayoutUnit m_extraAscender;
    LayoutUnit m_kernBeforeDegree;
    LayoutUnit m_kernAfterDegree;
    float m_degreeBottomRaisePercent;
    LayoutUnit m_radicalOperatorTop;
    LayoutUnit m_baseWidth;

    enum RootType { SquareRoot, RootWithIndex };
    RootType m_kind;
    virtual bool isRenderMathMLSquareRoot() const FINAL { return m_kind == SquareRoot; }
};
    
}

#endif // ENABLE(MATHML)

#endif // RenderMathMLRoot_h
