/*
 * Copyright (C) 2010 Alex Milowski (alex@milowski.com). All rights reserved.
 * Copyright (C) 2013 The MathJax Consortium.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RenderMathMLScripts_h
#define RenderMathMLScripts_h

#if ENABLE(MATHML)

#include "RenderMathMLBlock.h"

namespace WebCore {
// Render a base with scripts.
class RenderMathMLScripts : public RenderMathMLBlock {
public:
    RenderMathMLScripts(Element&, PassRefPtr<RenderStyle>);
    virtual RenderMathMLOperator* unembellishedOperator() FINAL;
    virtual Optional<int> firstLineBaseline() const FINAL;
    virtual void layoutBlock(bool relayoutChildren, LayoutUnit pageLogicalHeight = 0) FINAL;

private:
    virtual bool isRenderMathMLScripts() const FINAL { return true; }
    virtual const char* renderName() const FINAL { return "RenderMathMLScripts"; }

    bool getBaseAndScripts(RenderBox*& base, RenderBox*& firstPostScript, RenderBox*& firstPreScript);
    LayoutUnit spaceAfterScript();
    LayoutUnit italicCorrection(RenderBox* base);
    virtual void computePreferredLogicalWidths() OVERRIDE;
    void getScriptMetricsAndLayoutIfNeeded(RenderBox* base, RenderBox* script, LayoutUnit& minSubScriptShift, LayoutUnit& minSupScriptShift, LayoutUnit& maxScriptDescent, LayoutUnit& maxScriptAscent);

    enum ScriptsType { Sub, Super, SubSup, Multiscripts };

    ScriptsType m_scriptType;
};

inline RenderMathMLScripts* toRenderMathMLScripts(RenderMathMLBlock* block)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!block || block->isRenderMathMLScripts());
    return static_cast<RenderMathMLScripts*>(block);
}

inline const RenderMathMLScripts* toRenderMathMLScripts(const RenderMathMLBlock* block)
{
    ASSERT_WITH_SECURITY_IMPLICATION(!block || block->isRenderMathMLScripts());
    return static_cast<const RenderMathMLScripts*>(block);
}

inline RenderMathMLScripts* toRenderMathMLScripts(RenderObject* object)
{
    return toRenderMathMLScripts(toRenderMathMLBlock(object));
}

inline const RenderMathMLScripts* toRenderMathMLScripts(const RenderObject* object)
{
    return toRenderMathMLScripts(toRenderMathMLBlock(object));
}

// This will catch anyone doing an unnecessary cast.
void toRenderMathMLScripts(const RenderMathMLScripts*);

}

#endif // ENABLE(MATHML)

#endif // RenderMathMLScripts_h
