/** @file GalioConfigurationComponent.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef GalioConfigurationComponent_h
#define GalioConfigurationComponent_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ExceptionCode.h"
#include "EventTarget.h"
#include "ActiveDOMObject.h"

extern "C" {
#include "layers.h"
#include "confman.h"
}

namespace WebCore {

class GalioConfigurationComponent : public RefCounted<GalioConfigurationComponent>, public ActiveDOMObject, public EventTarget {
public:
    static PassRefPtr<GalioConfigurationComponent> create(ScriptExecutionContext* context, confman_dynamic_component comp);

    virtual ~GalioConfigurationComponent();

    void addItem( const String & name, ExceptionCode& ec ) { return addItem(name, 0, String(), ec); }
    void addItem( const String & name, int type, ExceptionCode& ec ) { return addItem(name, type, String(), ec); }
    void addItem( const String & name, int type, const String & defaultValue, ExceptionCode& ec );
    void declare(ExceptionCode& ec);
    void restore();

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<GalioConfigurationComponent*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "GalioConfigurationComponent"; }

    // EventTarget interface
    GalioConfigurationComponent* toGalioConfigurationComponent() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<GalioConfigurationComponent>::ref;
    using RefCounted<GalioConfigurationComponent>::deref;

    virtual bool addEventListener(const AtomicString& eventType, PassRefPtr<EventListener>, bool useCapture);
    virtual bool removeEventListener(const AtomicString& eventType, EventListener*, bool useCapture);

protected:
    GalioConfigurationComponent(ScriptExecutionContext* context, confman_dynamic_component comp);

private:
    confman_dynamic_component m_component;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // GalioConfigurationComponent_h
