/** @file GalioDebug.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "Console.h"
#include "GalioDebug.h"

#include "UtilsMOM.h"

#include "Chrome.h"
#include "ChromeClient.h"
#include "Frame.h"
#include "InspectorConsoleInstrumentation.h"
#include "Page.h"
#include "ScriptArguments.h"
#include "ScriptCallStack.h"
#include "ScriptValue.h"

extern "C" {
#include "layers.h"
#include "layers_lid.h"
#include "layers_lid_types.h"
}

namespace WebCore {

PassRefPtr<GalioDebug> GalioDebug::create(PassRefPtr<Console> console)
{
    return adoptRef(new GalioDebug(console));
}

GalioDebug::GalioDebug(PassRefPtr<Console> console)
    : m_console(console)
{
}

int GalioDebug::flexAllocated() const
{
    return 0;
}

int GalioDebug::flexFree() const
{
    return 0;
}

int GalioDebug::heapFree() const
{
    return 0;
}

int GalioDebug::heapAllocated() const
{
    return 0;
}

/* Implement debug.log separately to console.log as it can take 1 or 2 arguments.
 */
void GalioDebug::log(PassRefPtr<ScriptArguments> prpArguments, PassRefPtr<ScriptCallStack> prpCallStack)
{
    RefPtr<ScriptArguments> arguments = prpArguments;
    RefPtr<ScriptCallStack> callStack = prpCallStack;

    Frame* frame = m_console->frame();
    if (!frame)
        return;

    Page* page = frame->page();
    if (!page)
        return;

    const ScriptCallFrame& callFrame = callStack->at(0);

    if (!arguments->argumentCount())
        return;

    String message;
    String message2;
    if (arguments->argumentAt(0).getString(arguments->globalState(), message) &&
        arguments->argumentCount() > 1 && 
        arguments->argumentAt(1).getString(arguments->globalState(), message2)) {
        message.append(' ');
        message.append(message2);
    }

    page->chrome()->client()->addMessageToConsole(JSMessageSource, LogMessageType, LogMessageLevel, message, callFrame.lineNumber(), callFrame.columnNumber(), callFrame.sourceURL());
    InspectorInstrumentation::addMessageToConsole(page, JSMessageSource, LogMessageType, LogMessageLevel, message, callStack);
}

void GalioDebug::quit( int rc )
{
}

void GalioDebug::dumpAllDocuments()
{
}

void GalioDebug::dumpAllViews()
{
}

void GalioDebug::dumpAllDOMObjects()
{
}

void GalioDebug::gc( bool debug )
{
}

String GalioDebug::getBacktrace( int level )
{
    return String();
}

String GalioDebug::getWaferActiveList()
{
    return String();
}

String GalioDebug::getWaferVisibleList()
{
    return String();
}

static const debug_area_descriptor* get_debug_area( const String & area )
{
    layers_string larea = UtilsMOM::LayersStringFromString(area);
    const lid_entry *lid = layers_lid_lookup_debug( larea );
    layers_string_dispose(larea);

    return layers_lid_get_debug_descriptor( lid );
}
    
void GalioDebug::enableDebug( const String & area )
{
    const debug_area_descriptor *dad = get_debug_area(area);
    if ( dad && dad->enable )
    {
        dad->enable();
    }
}

void GalioDebug::disableDebug( const String & area )
{
    const debug_area_descriptor *dad = get_debug_area(area);
    if ( dad && dad->disable )
    {
        dad->disable();
    }
}

void GalioDebug::resetDebug( const String & area )
{
    const debug_area_descriptor *dad = get_debug_area(area);
    if ( dad && dad->reset )
    {
        dad->reset();
    }
}

} // namespace WebCore

#endif // ENABLE(RMAI)
