/** @file GalioUtility.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "GalioUtility.h"
#include "Node.h"
#include "Document.h"
#include "FrameView.h"

#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "layers_util.h"
#include "layers_power.h"
#include "time_offset.h"
}

namespace WebCore {

PassRefPtr<GalioUtility> GalioUtility::create()
{
    return adoptRef(new GalioUtility());
}

GalioUtility::GalioUtility()
{
}

// not required
void GalioUtility::setScreenSize( int width, int height )
{
}

String GalioUtility::formatTime( const String & format, int time, bool formatAsUTC )
{
    char buffer[256];
    layers_string lformat = UtilsMOM::LayersStringFromString(format);
    BOOL ok = layers_strftime( buffer, sizeof( buffer ), layers_string_lock( lformat ), time, formatAsUTC ) != 0;
    layers_string_unlock(lformat);
    layers_string_dispose(lformat);
    return ok ? String(buffer) : String();
}

String GalioUtility::formatTime( const String & format, int time)
{
	return formatTime(format, time, false);
}

// not required but simple to include
void GalioUtility::setTimeOffset( int offset )
{
    layers_time_offset_set(offset);
}

// not required but simple to include
void GalioUtility::setStartTime( const String & startTime )
{
    layers_string ltime = UtilsMOM::LayersStringFromString(startTime);
    layers_time_offset_set_start_time( layers_string_lock(ltime) );
    layers_string_unlock( ltime );
    layers_string_dispose( ltime );
}

int GalioUtility::keyQueueFill()
{
    return 0;
}

int GalioUtility::keyQueuePeekCode( int index )
{
    return 0;
}

int GalioUtility::keyQueuePeekType( int index )
{
    return 0;
}

void GalioUtility::keyQueueConsume( int count )
{
}

void GalioUtility::setKeyRepeat( int initialInterval, int initialDelay, int finalInterval )
{
}

/* This flushes any layout changes but unlike Galio doesn't cayuse a
 * redraw. That must be triggered separately. */
void GalioUtility::flushRedraws( Node* node, bool updateImmediately )
{
    if (node->nodeType() != Node::DOCUMENT_NODE) return;

    Document* doc = static_cast<Document*>(node);
    if (FrameView* view = doc->view())
    {
        view->layoutIfNeededRecursive();
    }
}

void GalioUtility::systemReboot( const String & message )
{
    if (!message.isEmpty())
    {
        layers_string lmessage = UtilsMOM::LayersStringFromString(message);
        layers_power_reboot_with_message( layers_string_lock(lmessage) );
        layers_string_unlock( lmessage );
        layers_string_dispose( lmessage );
    }
    else
    {
        layers_power_reboot();
    }
}

void GalioUtility::systemReboot()
{
	systemReboot(String());
}

String GalioUtility::abbreviateText( Node* nodeContext, const String & content, int maxWidth, const String & ellipsis )
{
    // not available - just return original string
    return String(content);
}

String GalioUtility::abbreviateText( Node* nodeContext, const String & content, int maxWidth)
{
	return abbreviateText(nodeContext, content, maxWidth, String());
}

} // namespace WebCore

#endif // ENABLE(RMAI)
