/** @file GalioWafer.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef GalioWafer_h
#define GalioWafer_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>
#include "ContainerNode.h"

namespace WebCore {

class GalioWaferCollection;
class Script;
class EventTarget;
class DOMWindow;
class Event;
class Document;
class HTMLIFrameElement;

class GalioWafer : public ContainerNode {
public:

    static PassRefPtr<GalioWafer> create(Document* document);
    static PassRefPtr<GalioWafer> create(GalioWaferCollection* waferCollection, const String & url);
    ~GalioWafer();

    // From Node
    String nodeName() const;
    NodeType nodeType() const;
    virtual RefPtr<Node> cloneNodeInternal(Document&, CloningOperation) OVERRIDE;
    bool childTypeAllowed(NodeType) const;
    
    PassRefPtr<Node> appendChild(PassRefPtr<Node> prpNewChild, ExceptionCode& ec);

    DOMWindow* window() const;
    Document* contentDocument() const;
    //String id() const;
    //void setId(const String & id);

    bool visible() const;
    bool active() const;
    bool isPrimaryReceiver() const;
    bool isVisibleTopmost() const;
    //bool hasFocus() const;
    GalioWaferCollection* apis() const;
    //bool scrollable() const;
    //void setScrollable(bool scrollable);

    //Script* onLowMemory() const;
    //void setOnLowMemory(Script* onLowMemory);

    PassRefPtr<GalioWafer> createWafer( const String & url );
    void destroyWafer();
    void show();
    void hide();
    void activate();
    void deactivate();
    bool dispatchSystemEvent(PassRefPtr<Event> event, PassRefPtr<Node> target = nullptr);
    void blur();
    void focus();
    void restartApplicationTree();

    void setActive(bool active) { m_active = active; }
    void makeVisible(bool visible);

    void dispatchWaferEvent(const AtomicString& eventName );
    void start();
    bool isRootWafer();
    void documentLoaded(Document* doc);
    void documentUnloaded(Document* doc);
    String url() { return m_url; }

    void moveBy(float x, float y);
    void moveTo(float x, float y);
    void resizeBy(float x, float y);
    void resizeTo(float x, float y);

protected:
    GalioWafer(Document* document);

    PassRefPtr<Event> createUIEvent(const AtomicString& eventName);

private:
    bool m_destroyed;
    bool m_deleting;
    bool m_visible;
    bool m_active;
    GalioWaferCollection* m_waferCollection;
    String m_url;
    HTMLIFrameElement* m_iframe;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // GalioWafer_h
