/** @file JSMOMChannelConstructor.cpp
 *
 * Copyright (C) Espial Limited 2015 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "JSDOMBinding.h"
#include "JSDocument.h"

#include "MOMChannel.h"
#include "JSMOMChannel.h"

#include "ScriptExecutionContext.h"
#include <runtime/Error.h>
#include <wtf/RefPtr.h>

using namespace JSC;

namespace WebCore {

template<> EncodedJSValue JSC_HOST_CALL JSMOMChannelConstructor::construct(ExecState* exec)
{
	JSMOMChannelConstructor* jsConstructor = jsCast<JSMOMChannelConstructor*>(exec->callee());
    ScriptExecutionContext* context = jsConstructor->scriptExecutionContext();

    if (exec->argumentCount() < 2)
        return JSValue::encode(createNotEnoughArgumentsError(exec));

    if (!context)
        return JSValue::encode(createReferenceError(exec, "MOMChannel constructor associated document is unavailable"));

    String chan = WebCore::valueToStringWithNullCheck(exec, exec->argument(0));
    String name = WebCore::valueToStringWithNullCheck(exec, exec->argument(1));
    if (exec->hadException())
        return JSValue::encode(jsUndefined());

    ExceptionCode ec = 0;
    RefPtr<MOMChannel> list = MOMChannel::create(context, chan, name, ec);
    if (ec) {
        setDOMException(exec, ec);
        return JSValue::encode(jsUndefined());
    }
    return JSValue::encode(asObject(toJS(exec, jsConstructor->globalObject(), list.release())));
}

} // namespace WebCore

#endif // ENABLE(VIDEO)
