/** @file JSMOMContentItemCustom.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

//#include "MOMCable.h"
#include "MOMChannel.h"
#include "MOMChannelList.h"
#include "MOMChannelScanOptions.h"
#include "MOMContentCatalogue.h"
#include "MOMContentItem.h"
#include "MOMContentList.h"
#include "MOMContentPagedList.h"
#include "MOMContentSearch.h"
#include "MOMEPGProgramme.h"
#include "MOMParentalRating.h"
#include "MOMPackageDescriptor.h"
#include "MOMPackageList.h"
#include "MOMProgramme.h"
#include "MOMProgrammeList.h"
#include "MOMPurchasableAsset.h"
#include "MOMPurchasableContentCatalogue.h"
#include "MOMReminder.h"
#include "MOMResourceConflictItem.h"
#include "MOMResourceConflictList.h"
#include "MOMSatellite.h"
#include "MOMScheduledRecording.h"
#include "MOMTransportStream.h"
#include "MOMVODAsset.h"
#include "MOMVODCatalogue.h"
#include "MOMVODService.h"

#include "JSMOMContentItem.h"
#include "JSDOMBinding.h"

//#include "JSMOMCable.h"
#include "JSMOMChannel.h"
#include "JSMOMChannelList.h"
#include "JSMOMChannelScanOptions.h"
#include "JSMOMContentCatalogue.h"
#include "JSMOMContentItem.h"
#include "JSMOMContentList.h"
#include "JSMOMContentPagedList.h"
#include "JSMOMContentSearch.h"
#include "JSMOMEPGProgramme.h"
#include "JSMOMParentalRating.h"
#include "JSMOMPackageDescriptor.h"
#include "JSMOMPackageList.h"
#include "JSMOMProgramme.h"
#include "JSMOMProgrammeList.h"
#include "JSMOMPurchasableAsset.h"
#include "JSMOMPurchasableContentCatalogue.h"
#include "JSMOMReminder.h"
#include "JSMOMResourceConflictItem.h"
#include "JSMOMResourceConflictList.h"
#include "JSMOMSatellite.h"
#include "JSMOMScheduledRecording.h"
#include "JSMOMTransportStream.h"
#include "JSMOMVODAsset.h"
#include "JSMOMVODCatalogue.h"
#include "JSMOMVODService.h"

extern "C" {
#include "layers.h"
#include "layers_class.h"
#include "cdf/mom_content_item_defs.h"
#include "cdf/mom_content_item.h"
#include "cdf/mom_content_catalogue.h"
#include "cdf/mom_content_paged_list.h"
#include "cdf/mom_core.h"
#include "cdf/mom_content_search.h"
#include "cdf/mom_epgprogramme.h"
#include "cdf/mom_resource_conflict_list.h"
}

using namespace JSC;

namespace WebCore {

JSValue toJS(ExecState* exec, JSDOMGlobalObject* globalObject, MOMContentItem* item)
{
    if (!item)
        return jsNull();

    JSObject* wrapper = getCachedWrapper(globalObject->world(), item);
    if (wrapper)
        return wrapper;

    // keep this list in sync with the one in MOMContentItem.cpp
    switch (mom_content_item_get_item_type(item->itemAsItem()))
    {
    case mom_content_item_type_CONTENT_LIST:
        if (layers_class_is_instance(FLEX_USE(item->itemAsItem()), mom_content_paged_list)) {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMContentPagedList, item);
        }
        else if (layers_class_is_instance(FLEX_USE(item->itemAsItem()), mom_resource_conflict_list)) {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMResourceConflictList, item);
        }
        else {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMContentList, item);
        }
        break;
    case mom_content_item_type_VOD_ASSET:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMVODAsset, item);
        break;
    case mom_content_item_type_VOD_SERVICE:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMVODService, item);
        break;
    case mom_content_item_type_CHANNEL:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMChannel, item);
        break;
    case mom_content_item_type_PROGRAMME:
        if (layers_class_is_instance(FLEX_USE(item->itemAsItem()), mom_epgprogramme)) {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMEPGProgramme, item);
        }
        else {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMProgramme, item);
        }
        break;
    case mom_content_item_type_CHANNEL_LIST:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMChannelList, item);
        break;
    case mom_content_item_type_PROGRAMME_LIST:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMProgrammeList, item);
        break;
    case mom_content_item_type_RECORDING:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMScheduledRecording, item);
        break;
    case mom_content_item_type_REMINDER:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMReminder, item);
        break;
    case mom_content_item_type_PACKAGE:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMPackageDescriptor, item);
        break;
    case mom_content_item_type_PACKAGE_LIST:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMPackageList, item);
        break;
    case mom_content_item_type_RESOURCE_CONFLICT_ITEM:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMResourceConflictItem, item);
        break;
    case mom_content_item_type_DATA_DOWNLOAD:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMContentItem, item);
        //wrapper = CREATE_DOM_WRAPPER(globalObject,
        // MOMDataDownload, item);
        // TODO: this needs checking
        break;
    case mom_content_item_type_VOD_CATALOGUE:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMVODCatalogue, item);
        break;
    case mom_content_item_type_PURCHASABLE_ASSET:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMPurchasableAsset, item);
        break;
    case mom_content_item_type_PURCHASABLE_CONTENT_CATALOGUE:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMPurchasableContentCatalogue, item);
        break;
    case mom_content_item_type_SATELLITE:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMSatellite, item);
        break;
    case mom_content_item_type_TRANSPORTSTREAM:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMTransportStream, item);
        break;
    case mom_content_item_type_CABLE:
        //TODO: this shouldn't get here with any luck as
        //mom_content_item is used internally but not exposed
        //wrapper = CREATE_DOM_WRAPPER(globalObject, MOMCable, item);
        break;
    case mom_content_item_type_CHANNELSCANOPTIONS:
        wrapper = CREATE_DOM_WRAPPER(globalObject, MOMChannelScanOptions, item);
        break;
    default:
        /* mom_content_item subclasses that don't have a specific
         * content type enumeration */
        if (layers_class_is_instance(FLEX_USE(item->itemAsItem()), mom_content_catalogue)) {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMContentCatalogue, item);
        }
        else if (layers_class_is_instance(FLEX_USE(item->itemAsItem()), mom_content_search)) {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMContentSearch, item);
        }
        else {
            wrapper = CREATE_DOM_WRAPPER(globalObject, MOMContentItem, item);
        }
        break;
    }

    return wrapper;
}

} // namespace WebCore

#endif // ENABLE(RMAI)
