/** @file JSMOMDVRCustom.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMException.h"
#include "MOMDVR.h"
#include "MOMScheduledRecording.h"
#include "MOMContentList.h"

#include "UtilsMOM.h"

#include "JSMOMDVR.h"
#include "JSMOMScheduledRecording.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_scheduler.h"
#include "cdf/mom_recording_list.h"
}

using namespace JSC;

namespace WebCore {
    
JSValue JSMOMDVR::createFilteredList(ExecState* exec)
{
    MOMDVR& imp = static_cast<MOMDVR&>(wrapped());
    mom_sort_order sort = mom_sort_order_UID;
    BOOL sort_increasing = TRUE;
    uint32_t states = mom_recording_list_ALL_STATES;

    int argc = exec->argumentCount();
    if (argc > 0)
        sort = static_cast<mom_sort_order>(exec->argument(0).toInt32(exec));

    if (argc > 1)
        sort_increasing = exec->argument(1).toBoolean(exec);

    if (argc > 2)
    {
        unsigned int i;
        states = 0;
        /* the argument list is a list of states that we wish to
         * include in this list */
        for ( i = 2; i < argc; i++ )
        {
            int dom_state = exec->argument(i).toInt32(exec);
            mom_recording_state state = UtilsMOM::recordingStateFromCode(dom_state);
            if (state != mom_recording_UNKNOWN)
                states |= 1 << (unsigned int)state;
        }
    }

    mom_recording_list list = mom_scheduler_create_filtered_list( states, sort, sort_increasing );
    if (list == NULL)
        return jsNull();

    return toJS(exec, globalObject(), MOMContentItem::createAdopt(imp.scriptExecutionContext(), (mom_content_item)list));
}

JSValue JSMOMDVR::remove(ExecState* exec)
{
    MOMScheduledRecording* rec = toMOMScheduledRecording(exec->argument(0));

    MDARemoveReason reason = MDA_REMOVE_THIS;
    if (exec->argumentCount() > 1)
        reason = static_cast<MDARemoveReason>(exec->argument(1).toInt32(exec));

    mom_error err = mom_recording_remove(rec->itemAsRecording(), reason);
    ExceptionCode ec;
    MOMException::setFromMOMError(err, ec);
    setDOMException(exec, ec);

    return jsNull();
}
    
} // namespace WebCore

#endif // ENABLE(RMAI)
