/** @file JSMOMPlayerCustom.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMException.h"
#include "MOMPlayer.h"

#include "UtilsMOM.h"

#include "JSMOMPlayer.h"
#include "MOMPlayer.h"
#include "JSMOMContentItem.h"
#include "MOMContentItem.h"

using namespace JSC;

namespace WebCore {

/* First parameter may be DOMString or MOMContentItem (MOMChannel, MOMScheduledRecording, MOMVODAsset, etc)
   long connect( in DOMString source, in [Optional=CallWithDefaultValue] long mode, in [Optional=CallWithDefaultValue] long offset ) raises(MOMException);
   long connect( in MOMContentItem source, in [Optional=CallWithDefaultValue] long mode, in [Optional=CallWithDefaultValue] long offset ) raises(MOMException);
 */

JSValue JSMOMPlayer::connect(ExecState* exec)
{
    MOMPlayer& imp = static_cast<MOMPlayer&>(wrapped());
    int argc = exec->argumentCount();

    if (argc == 0) {
        setDOMException(exec, SYNTAX_ERR);
        return jsUndefined();
    }

    int mode = argc >= 2 && exec->argument(1).isInt32() ? exec->argument(1).toInt32(exec) : 0;
    int offset = argc >= 3 && exec->argument(2).isInt32() ? exec->argument(2).toInt32(exec) : 0;

    int rc;
    ExceptionCode ec = 0;
    if (exec->argument(0).isString())
        rc = imp.connect(WebCore::valueToStringWithNullCheck(exec, exec->argument(0)), mode, offset, ec);
    else
        rc = imp.connect(toMOMContentItem(exec->argument(0)), mode, offset, ec);
        
    if (ec) {
        setDOMException(exec, ec);
        return jsUndefined();
    }
    
    return jsNumber(rc);
}

} //namespace WebCore

#endif // ENABLE(RMAI)
