/** @file JSMOMProgrammeConstructor.cpp
 *
 * Copyright (C) Espial Limited 2015 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "JSMOMProgrammeConstructor.h"

#include "MOMProgramme.h"
#include "JSMOMProgramme.h"
#include "MOMChannel.h"
#include "JSMOMChannel.h"

#include "ScriptExecutionContext.h"
#include <runtime/Error.h>

using namespace JSC;

namespace WebCore {

template<> EncodedJSValue JSC_HOST_CALL JSMOMProgrammeConstructor::construct(ExecState* exec)
{
	JSMOMProgrammeConstructor* jsConstructor = jsCast<JSMOMProgrammeConstructor*>(exec->callee());
    ScriptExecutionContext* context = jsConstructor->scriptExecutionContext();

    if (exec->argumentCount() < 3)
        return JSValue::encode(createNotEnoughArgumentsError(exec));

    if (!context)
        return JSValue::encode(createReferenceError(exec, "MOMProgramme constructor associated document is unavailable"));

    MOMChannel* chan = toMOMChannel(exec->argument(0));
    int startTime = exec->argument(1).toInt32(exec);
    int duration = exec->argument(2).toInt32(exec);
    if (exec->hadException())
        return JSValue::encode(jsUndefined());

    ExceptionCode ec = 0;
    RefPtr<MOMProgramme> list = MOMProgramme::create(context, chan, startTime, duration, ec);
    if (ec) {
        setDOMException(exec, ec);
        return JSValue::encode(jsUndefined());
    }
    return JSValue::encode(asObject(toJS(exec, jsConstructor->globalObject(), list.release())));
}

} // namespace WebCore

#endif // ENABLE(VIDEO)
