/** @file JSMOMVODAssetCustom.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMException.h"
#include "MOMVODAsset.h"

#include "JSMOMVODAsset.h"
#include "JSMOMStringCollection.h"
#include "MOMStringCollection.h"
#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_vod_asset.h"
}

using namespace JSC;

namespace WebCore {
    
JSValue JSMOMVODAsset::price(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    layers_string price;
    if (mom_vod_asset_get_price(imp.itemAsVODAsset(), &price) == ERR_NONE)
        return jsString(exec->vm(), UtilsMOM::StringFromLayersString(price));
    return jsUndefined();
}

JSValue JSMOMVODAsset::previewUri(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    layers_string previewUri;
    if (mom_vod_asset_get_preview_uri(imp.itemAsVODAsset(), &previewUri) == ERR_NONE)
        return jsString(exec->vm(), UtilsMOM::StringFromLayersString(previewUri));
    return jsUndefined();
}

JSValue JSMOMVODAsset::rentalPeriod(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t rental_period;
    if (mom_vod_asset_get_rental_period(imp.itemAsVODAsset(), &rental_period) == ERR_NONE)
        return jsNumber(rental_period);
    return jsUndefined();
}

JSValue JSMOMVODAsset::expirationDate(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t date;
    if (mom_asset_item_get_expiry_date(imp.itemAsAssetItem(), &date) == ERR_NONE)
        return jsNumber(date);
    return jsUndefined();
}

JSValue JSMOMVODAsset::expiryDate(ExecState* exec) const
{
    return expirationDate(exec);
}

JSValue JSMOMVODAsset::purchaseDate(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t date;
    if (mom_vod_asset_get_purchase_date(imp.itemAsVODAsset(), &date) == ERR_NONE)
        return jsNumber(date);
    return jsUndefined();
}

JSValue JSMOMVODAsset::purchased(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    mom_asset_item_purchase_state purchased;
    if (mom_asset_item_get_purchased(imp.itemAsAssetItem(), &purchased))
        return jsBoolean(purchased == mom_asset_item_PURCHASE_COMPLETED);
    return jsUndefined();
}

JSValue JSMOMVODAsset::duration(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t duration;
    if (mom_vod_asset_get_duration(imp.itemAsVODAsset(), &duration) == ERR_NONE)
        return jsNumber(duration);
    return jsUndefined();
}

JSValue JSMOMVODAsset::bookmark(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t bookmark;
    if (mom_vod_asset_get_bookmark(imp.itemAsVODAsset(), &bookmark) == ERR_NONE)
        return jsNumber(bookmark);
    return jsUndefined();
}

JSValue JSMOMVODAsset::rentalViewings(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t viewings;
    if (mom_vod_asset_get_rental_viewings(imp.itemAsVODAsset(), &viewings) == ERR_NONE)
        return jsNumber(viewings);
    return jsUndefined();
}

JSValue JSMOMVODAsset::remainingViewings(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t viewings;
    if (mom_vod_asset_get_remaining_viewings(imp.itemAsVODAsset(), &viewings) == ERR_NONE)
        return jsNumber(viewings);
    return jsUndefined();
}

JSValue JSMOMVODAsset::subtitles(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    BOOL subtitles;
    if (mom_vod_asset_has_subtitles(imp.itemAsVODAsset(), &subtitles) == ERR_NONE)
        return jsBoolean(subtitles);
    return jsUndefined();
}

JSValue JSMOMVODAsset::isHD(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    BOOL hd;
    if (mom_vod_asset_is_hd(imp.itemAsVODAsset(), &hd) == ERR_NONE)
        return jsBoolean(hd);
    return jsUndefined();
}

JSValue JSMOMVODAsset::sapAvailable(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    BOOL sap;
    if (mom_vod_asset_has_sap(imp.itemAsVODAsset(), &sap) == ERR_NONE)
        return jsBoolean(sap);
    return jsUndefined();
}

JSValue JSMOMVODAsset::isMultilingual(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    BOOL multilingual;
    if (mom_vod_asset_is_multilingual(imp.itemAsVODAsset(), &multilingual) == ERR_NONE)
        return jsBoolean(multilingual);
    return jsUndefined();
}

JSValue JSMOMVODAsset::audioType(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t audio;
    if (mom_vod_asset_get_audio_type(imp.itemAsVODAsset(), &audio) == ERR_NONE)
        return jsNumber(audio);
    return jsUndefined();
}

JSValue JSMOMVODAsset::starRating(ExecState* exec) const
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    uint32_t rating;
    if (mom_vod_asset_get_star_rating(imp.itemAsVODAsset(), &rating) == ERR_NONE)
        return jsNumber(rating);
    return jsUndefined();
}

JSValue JSMOMVODAsset::isReady(ExecState* exec)
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());
    BOOL ready;
    mom_error err = mom_asset_item_is_ready(imp.itemAsAssetItem(), &ready);
    if (err == ERR_NONE)
        return jsBoolean(ready);
    if (err == mom_err_BUSY || err == mom_err_NOTAUTHORISED)
    {
        ExceptionCode ec;
        MOMException::setFromMOMError(err, ec);
        setDOMException(exec, ec);
    }
    return jsUndefined();
}

JSValue JSMOMVODAsset::lookupMetadata(ExecState* exec)
{
    MOMVODAsset& imp = static_cast<MOMVODAsset&>(wrapped());

    String key = WebCore::valueToStringWithNullCheck(exec, exec->argument(0));
    layers_string lkey = UtilsMOM::LayersStringFromString(key);

    mom_urilist list;
    mom_error err = mom_asset_item_lookup_metadata_list(imp.itemAsAssetItem(), lkey, &list);
    
    layers_string_dispose(lkey);

    if (err != ERR_NONE)
        return jsUndefined();

    if (list == NULL)
        return jsNull();

    return toJS(exec, globalObject(), MOMStringCollection::createAdopt(list));
}
    
} // namespace WebCore

#endif // ENABLE(RMAI)
