/** @file MOMAccessEvent.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMAccessEvent_h
#define MOMAccessEvent_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMEvent.h"

namespace WebCore {

class MOMContentItem;

class MOMAccessEvent : public MOMEvent {
public:
    enum {
        UNLOCK = 0,
        LOCK = 1,
        MANUAL_BLOCK = 1,
        PARENTAL_RATING = 2,
        RESTRICTED_TIME = 4,
        CONTENT_ADVISORY = 8,
    };

    static PassRefPtr<MOMAccessEvent> create()
    {
        return adoptRef(new MOMAccessEvent);
    }

    int reason() const { return getIntParam(mom_event_ACCESS_REASON); }
    int boxReason() const { return getIntParam(mom_event_ACCESS_BOXREASON); }
    int boxInfo() const { return getIntParam(mom_event_ACCESS_BOXINFO); }
    int channelReason() const { return getIntParam(mom_event_ACCESS_CHANNELREASON); }
    int channelInfo() const { return getIntParam(mom_event_ACCESS_CHANNELINFO); }
    int objectReason() const { return getIntParam(mom_event_ACCESS_OBJECTREASON); }
    int objectInfo() const { return getIntParam(mom_event_ACCESS_OBJECTINFO); }
    MOMContentItem* object() const { return m_object.get(); }

    MOMAccessEvent(const mom_event_params* params, ScriptExecutionContext* context);
private:
    MOMAccessEvent();

    mutable RefPtr<MOMContentItem> m_object;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMAccessEvent_h
