/** @file MOMCable.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMCable.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_content_item.h"
}

namespace WebCore {

PassRefPtr<MOMCable> MOMCable::create()
{
    return adoptRef(new MOMCable());
}

// TODO should be able to raise exception
MOMCable::MOMCable()
{
    mom_cable cable;
    mom_error err = mom_cable_object_create( &cable );
    m_cable = cable;
}

MOMCable::~MOMCable()
{
    mom_content_item_dispose((mom_content_item)m_cable);
}
    
int MOMCable::startFrequencyMHz() const
{
    return mom_cable_start_frequency(m_cable);
}

void MOMCable::setStartFrequencyMHz(int startFrequencyMHz)
{
    mom_cable_start_frequency_set(m_cable, startFrequencyMHz);
}

int MOMCable::endFrequencyMHz() const
{
    return mom_cable_end_frequency(m_cable);
}

void MOMCable::setEndFrequencyMHz(int endFrequencyMHz)
{
    mom_cable_end_frequency_set(m_cable, endFrequencyMHz);
}

int MOMCable::channelBandwidthMHz() const
{
    return mom_cable_channel_bandwidth(m_cable);
}

void MOMCable::setChannelBandwidthMHz(int channelBandwidthMHz)
{
    mom_cable_channel_bandwidth_set(m_cable, channelBandwidthMHz);
}

MOMCable* MOMCable::symbolRateMBd() const
{
    // TODO
    return NULL;
}

int MOMCable::length() const
{
    return 0;
}

} // namespace WebCore

#endif // ENABLE(RMAI)
