/** @file MOMChannelManager.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMChannelManager.h"
#include "MOMChannelList.h"
#include "MOMChannelScanOptions.h"
#include "MOMContentItem.h"
#include "MOMGuideSearch.h"
#include "MOMContentSearch.h"
#include "MOMStringCollection.h"

#include "UtilsMOM.h"
#include "MOMException.h"

extern "C" {
#include "layers.h"
#include "layers_flex.h"
#include "object_declare.h"
#include "cdf/mom_device_api.h"
#include "cdf/mda_guide.h"

#include "cdf/mom_core.h"
#include "cdf/mom_channel_list_all.h"
#include "cdf/mom_channel_list.h"
#include "cdf/mom_channel_filter.h"
#include "cdf/mom_mda_channel_list.h"
}

namespace WebCore {

PassRefPtr<MOMChannelManager> MOMChannelManager::create(ScriptExecutionContext* context)
{
    return adoptRef(new MOMChannelManager(context));
}

MOMChannelManager::MOMChannelManager(ScriptExecutionContext* context)
    : m_context(context)
{
}

MOMChannelList* MOMChannelManager::all() const
{
    if (!m_all)
        m_all = MOMChannelList::create(m_context, mom_channel_list_all());
    return m_all.get();
}

MOMChannelList* MOMChannelManager::favourites() const
{
    if (!m_favourites)
        m_favourites = MOMChannelList::create(m_context, mom_channel_filter_favourites());
    return m_favourites.get();
}

MOMChannelList* MOMChannelManager::unauthorised() const
{
    if (!m_unauthorised)
        m_unauthorised = MOMChannelList::create(m_context, mom_channel_list_unauthorised());
    return m_unauthorised.get();
}

int MOMChannelManager::guideDaysAvailable() const
{
    return mom_mda_channel_list_max_days_available();
}

MOMStringCollection* MOMChannelManager::genres() const
{
    if (!m_genres)
        m_genres = MOMStringCollection::create(mom_mda_channel_list_all_genres());
    return m_genres.get();
}

MOMStringCollection* MOMChannelManager::contentAdvisories() const
{
    if (!m_contentAdvisories)
        m_contentAdvisories = MOMStringCollection::create(mom_channel_list_all_content_advisories());
    return m_contentAdvisories.get();
}

PassRefPtr<MOMChannelList> MOMChannelManager::fetch( const String & name, ExceptionCode& ec )
{
    layers_string lname = UtilsMOM::LayersStringFromString(name);
    mom_channel_list list;
    RefPtr<MOMChannelList> wlist = PassRefPtr<MOMChannelList>();
    mom_error err = mom_channel_list_create(&list);
    if (err == ERR_NONE)
    {
        err = mom_channel_list_fetch(list, lname);
        if (err == mom_err_FAILURE)
        {
            err = mom_channel_list_add_named(list, lname);
        }
        if (err == ERR_NONE)
        {
            wlist = MOMChannelList::create(m_context, list);
        }
        mom_content_item_dispose( (mom_content_item)list );
    }
    layers_string_dispose(lname);
    MOMException::setFromMOMError(err, ec);
    return wlist.release();
}

void MOMChannelManager::store( MOMChannelList* list, const String & name, ExceptionCode& ec )
{
    layers_string lname = UtilsMOM::LayersStringFromString(name);
    mom_channel_list mlist = list->itemAsChannelList();
    mom_error err = mom_channel_list_store(mlist, lname);
    MOMException::setFromMOMError(err, ec);
    layers_string_dispose(lname);
}

void MOMChannelManager::remove( const String & name, ExceptionCode& ec )
{
    layers_string lname = UtilsMOM::LayersStringFromString(name);
    mom_error err = mom_channel_list_delete(lname);
    MOMException::setFromMOMError(err, ec);
    layers_string_dispose(lname);
}

void MOMChannelManager::startScan( bool options, MOMContentItem* target, ExceptionCode& ec )
{
    // TODO
}

void MOMChannelManager::startScan( ExceptionCode& ec )
{
    // TODO
}

void MOMChannelManager::startScan( bool options, ExceptionCode& ec )
{
    // TODO
}

void MOMChannelManager::stopScan( ExceptionCode& ec)
{
    mom_error err = mom_mda_channel_list_abort_scan_all();
    MOMException::setFromMOMError(err, ec);
}

void MOMChannelManager::resetFilters()
{
    mom_channel_filter_reset();
}

PassRefPtr<MOMContentItem> MOMChannelManager::createSearch()
{
    return MOMContentItem::createAdopt(m_context, (mom_content_item)mom_channel_module_create_search());
}

PassRefPtr<MOMGuideSearch> MOMChannelManager::createGuideSearch()
{
    return MOMGuideSearch::create(m_context);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
