/** @file MOMChannelManager.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMChannelManager_h
#define MOMChannelManager_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ExceptionCode.h"

#include "MOMChannelList.h"

namespace WebCore {

class MOMChannelScanOptions;
class MOMContentItem;
class MOMGuideSearch;
class MOMStringCollection;
class ScriptExecutionContext;

class MOMChannelManager : public RefCounted<MOMChannelManager> {
public:

    static PassRefPtr<MOMChannelManager> create(ScriptExecutionContext* m_context);

    MOMChannelList* all() const;
    MOMChannelList* favourites() const;
    MOMChannelList* unauthorised() const;
    int guideDaysAvailable() const;
    MOMStringCollection* genres() const;
    MOMStringCollection* contentAdvisories() const;

    PassRefPtr<MOMChannelList> fetch( const String & name, ExceptionCode& ec );
    void store( MOMChannelList* list, const String & name, ExceptionCode& ec );
    void remove( const String & name, ExceptionCode& ec );
    void startScan( ExceptionCode& ec );
    void startScan( bool options, ExceptionCode& ec );
    void startScan( bool options, MOMContentItem* target, ExceptionCode& ec );
    void stopScan(ExceptionCode& ec);
    void resetFilters();
    PassRefPtr<MOMContentItem> createSearch();
    PassRefPtr<MOMGuideSearch> createGuideSearch();

    ScriptExecutionContext* scriptExecutionContext() const { return m_context; };

protected:
    MOMChannelManager(ScriptExecutionContext* context);

private:
    mutable RefPtr<MOMChannelList> m_all;
    mutable RefPtr<MOMChannelList> m_favourites;
    mutable RefPtr<MOMChannelList> m_unauthorised;
    mutable RefPtr<MOMStringCollection> m_genres;
    mutable RefPtr<MOMStringCollection> m_contentAdvisories;
    ScriptExecutionContext* m_context;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMChannelManager_h
