/** @file MOMComponent.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMComponent.h"
#include "MOMVideoComponent.h"
#include "MOMAudioComponent.h"
#include "MOMSubtitleComponent.h"

#include "UtilsMOM.h"

namespace WebCore {

PassRefPtr<MOMComponent> MOMComponent::create(mom_component comp)
{
    RefPtr<MOMComponent> wcomp = static_cast<MOMComponent*>(mom_component_get_dom_obj(comp));
    if (wcomp)
        return wcomp.release();

    unsigned int type;
    mom_component_get_type(comp, &type);
    switch (type)
    {
    case MOM_AUDIO_COMPONENT:
        wcomp = adoptRef(new MOMAudioComponent(comp));
        break;
    case MOM_VIDEO_COMPONENT:
        wcomp = adoptRef(new MOMVideoComponent(comp));
        break;
    case MOM_SUBTITLE_COMPONENT:
        wcomp = adoptRef(new MOMSubtitleComponent(comp));
        break;
    default:
        wcomp = adoptRef(new MOMComponent(comp));
        break;
    }
    return wcomp.release();
}

MOMComponent::MOMComponent(mom_component comp)
    : m_component(comp)
{
    mom_component_set_dom_obj(comp, this);
}

MOMComponent::~MOMComponent()
{
    mom_component_set_dom_obj(m_component, NULL);
}

int MOMComponent::type() const
{
    unsigned int type;
    // TODO should retuen undefined
    mom_component_get_type(component(), &type);
    return type;
}

int MOMComponent::pid() const
{
    unsigned int pid;
    // TODO should retuen undefined
    mom_component_get_pid(component(), &pid);
    return pid;
}

String MOMComponent::encoding() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_component_get_encoding(component()));
}

bool MOMComponent::encrypted() const
{
    return mom_component_get_encrypted(component());
}

String MOMComponent::language() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_component_get_language(component()));
}

String MOMComponent::getData()
{
    // TODO, this should return undefined if no string
    layers_string str;
    mom_component_get_application_data(component(), &str);
    return UtilsMOM::StringFromLayersStringAndDispose(str);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
