/** @file MOMConfiguration.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMConfiguration.h"
#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_channel_module.h"
#include "cdf/mom_configuration.h"
}

namespace WebCore {

PassRefPtr<MOMConfiguration> MOMConfiguration::create()
{
    return adoptRef(new MOMConfiguration());
}

MOMConfiguration::MOMConfiguration()
{
}

String MOMConfiguration::preferredAudioLanguage() const
{
    return UtilsMOM::StringFromLayersString(mom_configuration_get_audio_language());
}

void MOMConfiguration::setPreferredAudioLanguage(const String & preferredAudioLanguage)
{
    layers_string llang = UtilsMOM::LayersStringFromString(preferredAudioLanguage);
    mom_configuration_set_audio_language(llang);
    layers_string_dispose(llang);
}

String MOMConfiguration::preferredSubtitleLanguage() const
{
    return UtilsMOM::StringFromLayersString(mom_configuration_get_subtitle_language());
}

void MOMConfiguration::setPreferredSubtitleLanguage(const String & preferredSubtitleLanguage)
{
    layers_string llang = UtilsMOM::LayersStringFromString(preferredSubtitleLanguage);
    mom_configuration_set_subtitle_language(llang);
    layers_string_dispose(llang);
}

String MOMConfiguration::screenLanguage() const
{
    return UtilsMOM::StringFromLayersString(mom_configuration_get_screen_language());
}

void MOMConfiguration::setScreenLanguage(const String & screenLanguage)
{
    layers_string llang = UtilsMOM::LayersStringFromString(screenLanguage);
    mom_configuration_set_screen_language(llang);
    layers_string_dispose(llang);
}

String MOMConfiguration::countryId() const
{
    return UtilsMOM::StringFromLayersString(mom_configuration_get_country_id());
}

void MOMConfiguration::setCountryId(const String & countryId)
{
    layers_string lid = UtilsMOM::LayersStringFromString(countryId);
    mom_configuration_set_country_id(lid);
    layers_string_dispose(lid);
}

int MOMConfiguration::regionId() const
{
    return mom_configuration_get_region_id();
}

void MOMConfiguration::setRegionId(int regionId)
{
    mom_configuration_set_region_id(regionId);
}

int MOMConfiguration::dvrPolicy() const
{
    return mom_configuration_get_dvr_policy();
}

void MOMConfiguration::setDvrPolicy(int dvrPolicy)
{
    mom_configuration_set_dvr_policy(dvrPolicy);
}

int MOMConfiguration::dvrSaveEpisodes() const
{
    return mom_configuration_get_dvr_save_episodes();
}

void MOMConfiguration::setDvrSaveEpisodes(int dvrSaveEpisodes)
{
    mom_configuration_set_dvr_save_episodes(dvrSaveEpisodes);
}

int MOMConfiguration::dvrSaveDays() const
{
    return mom_configuration_get_dvr_save_time();
}

void MOMConfiguration::setDvrSaveDays(int dvrSaveDays)
{
    mom_configuration_set_dvr_save_time(dvrSaveDays);
}

int MOMConfiguration::dvrStartPadding() const
{
    return mom_configuration_get_dvr_start_padding();
}

void MOMConfiguration::setDvrStartPadding(int dvrStartPadding)
{
    mom_configuration_set_dvr_start_padding(dvrStartPadding);
}

int MOMConfiguration::dvrEndPadding() const
{
    return mom_configuration_get_dvr_end_padding();
}

void MOMConfiguration::setDvrEndPadding(int dvrEndPadding)
{
    mom_configuration_set_dvr_end_padding(dvrEndPadding);
}

int MOMConfiguration::channelScanTime() const
{
    return mom_channel_module_get_scan_time();
}

void MOMConfiguration::setChannelScanTime(int channelScanTime)
{
    mom_channel_module_set_scan_time(channelScanTime);
}

void MOMConfiguration::restoreDefaults()
{
    mom_configuration_factory_restore();
}

} // namespace WebCore

#endif // ENABLE(RMAI)
