/** @file MOMContentItem.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMContentItem_h
#define MOMContentItem_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

extern "C" {
#include "layers.h"
#include "layers_flex.h"
#include "cdf/mom_content_item.h"
}

namespace WebCore {

class MOMParentalRating;
class ScriptExecutionContext;

class MOMContentItem : public RefCounted<MOMContentItem>, public ActiveDOMObject, public EventTarget {
public:
    enum {
        CONTENT_LIST = 0,
        VOD_ASSET = 1,
        VOD_SERVICE = 2,
        CHANNEL = 3,
        PROGRAMME = 4,
        CHANNEL_LIST = 5,
        PROGRAMME_LIST = 6,
        RECORDING = 7,
        REMINDER = 8,
        PACKAGE_ = 9,
        PACKAGE_LIST = 10,
        RESOURCE_CONFLICT_ITEM = 11,
        DATA_DOWNLOAD = 12,
        VOD_CATALOGUE = 13,
        PURCHASABLE_ASSET = 14,
        PURCHASABLE_CONTENT_CATALOGUE = 15,
    };

    static PassRefPtr<MOMContentItem> create(ScriptExecutionContext* context, mom_content_item item);
    static PassRefPtr<MOMContentItem> createAdopt(ScriptExecutionContext* context, mom_content_item item);
    virtual ~MOMContentItem();

    String name() const;
    void setName(const String & name);

    String longName() const;
    void setLongName(const String & longName);

    String description() const;
    void setDescription(const String & description);

    String uri() const;
    String uid() const;
    String thumbnailUri() const;
    int type() const;
    MOMParentalRating* parentalRating() const;
    bool locked() const;
    bool blocked() const;
    int requiredBandwidth() const;

    String getProperty( const String & property );
    int setProperty( const String & property, ExceptionCode& ec );
    int setProperty( const String & property, const String & value, ExceptionCode& ec );

    ALWAYS_INLINE mom_content_item itemAsItem() const { return m_item; }

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMContentItem*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMContentItem"; }

    // EventTarget interface
    MOMContentItem* toMOMContentItem() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMContentItem>::ref;
    using RefCounted<MOMContentItem>::deref;

protected:
    MOMContentItem(ScriptExecutionContext* context, mom_content_item item);
    mom_content_item m_item;

private:
    mutable RefPtr<MOMParentalRating> m_parentalRating;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMContentItem_h
