/** @file MOMContentList.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMContentList.h"
#include "MOMContentItem.h"

#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_content_list.h"
}

namespace WebCore {

PassRefPtr<MOMContentList> MOMContentList::createAdopt(ScriptExecutionContext *context, mom_content_list list)
{
    return static_pointer_cast<MOMContentList>(MOMContentItem::createAdopt(context, (mom_content_item)list));
}

PassRefPtr<MOMContentList> MOMContentList::create(ScriptExecutionContext *context, mom_content_list list)
{
    return static_pointer_cast<MOMContentList>(MOMContentItem::create(context, (mom_content_item)list));
}

MOMContentList::MOMContentList(ScriptExecutionContext *context, mom_content_item item)
    : MOMContentItem(context, item)
{
}

int MOMContentList::length() const
{
    return mom_content_list_length(itemAsContentList());
}

PassRefPtr<MOMContentItem> MOMContentList::item( int index )
{
    mom_content_item mitem = mom_content_list_item(itemAsContentList(), index);
    return mitem ? MOMContentItem::create(scriptExecutionContext(), mitem) : PassRefPtr<MOMContentItem>();
}
    
RefPtr<MOMContentItem> MOMContentList::namedItem( const String & name )
{
    layers_string lprop = UtilsMOM::LayersStringFromString(name);
    mom_content_item item = mom_content_list_find_uid(itemAsContentList(), lprop);
    layers_string_dispose(lprop);
    return MOMContentItem::create(scriptExecutionContext(), item);
}

PassRefPtr<MOMContentItem> MOMContentList::nextItem( MOMContentItem* item, bool wrap )
{
    mom_content_item next = mom_content_list_next(itemAsContentList(), item->itemAsItem());
    if (next == NULL && wrap)
        next = mom_content_list_item(itemAsContentList(), 0);
    return next ? MOMContentItem::create(scriptExecutionContext(), next) : PassRefPtr<MOMContentItem>();
}

PassRefPtr<MOMContentItem> MOMContentList::prevItem( MOMContentItem* item, bool wrap )
{
    mom_content_item prev = mom_content_list_prev(itemAsContentList(), item->itemAsItem());
    if (prev == NULL && wrap)
        prev = mom_content_list_item(itemAsContentList(), mom_content_list_length(itemAsContentList())-1);
    return prev ? MOMContentItem::create(scriptExecutionContext(), prev) : PassRefPtr<MOMContentItem>();
}

} // namespace WebCore

#endif // ENABLE(RMAI)
