/** @file MOMDRMAgent.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMDRMAgent.h"
#include "MOMStringCollection.h"
#include "EventNames.h"

#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_drm.h"
}

namespace WebCore {

PassRefPtr<MOMDRMAgent> MOMDRMAgent::create(ScriptExecutionContext* context)
{
    return adoptRef(new MOMDRMAgent(context));
}

MOMDRMAgent::MOMDRMAgent(ScriptExecutionContext* context)
    : ActiveDOMObject(context)
{
}

MOMDRMAgent::~MOMDRMAgent()
{
}

EventTargetInterface MOMDRMAgent::eventTargetInterface() const
{
    return MOMDRMAgentEventTargetInterfaceType;
}

MOMStringCollection* MOMDRMAgent::drmSystems() const
{
    if (!m_drmSystems)
        m_drmSystems = MOMStringCollection::create(mom_drm_systems_get());
    return m_drmSystems.get();
}

String MOMDRMAgent::sendDRMMessage( const String & msgType, const String & msg, const String & DRMSystemID )
{
    layers_string ltype = UtilsMOM::LayersStringFromString(msgType);
    layers_string lmsg = UtilsMOM::LayersStringFromString(msg);
    layers_string lid = UtilsMOM::LayersStringFromString(DRMSystemID);
    layers_string lresult = mom_drm_message_send(ltype, lmsg, lid);
    layers_string_dispose(ltype);
    layers_string_dispose(lmsg);
    layers_string_dispose(lid);
    return UtilsMOM::StringFromLayersStringAndDispose(lresult);
}

String MOMDRMAgent::getProperty( const String & propName, const String & DRMSystemID )
{
    layers_string lprop = UtilsMOM::LayersStringFromString(propName);
    layers_string lid = UtilsMOM::LayersStringFromString(DRMSystemID);
    layers_string lresult = mom_drm_get_property(lprop, lid);
    layers_string_dispose(lprop);
    layers_string_dispose(lid);
    return UtilsMOM::StringFromLayersStringAndDispose(lresult);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
