/** @file MOMDVR.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMDVR.h"
#include "MOMRecorder.h"
#include "MOMProgramme.h"
#include "MOMContentItem.h"
#include "MOMScheduledRecording.h"
#include "MOMReminder.h"
#include "MOMContentList.h"
#include "MOMChannel.h"

#include "MOMException.h"
#include "EventNames.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_scheduler.h"
}

namespace WebCore {

PassRefPtr<MOMDVR> MOMDVR::create(ScriptExecutionContext* context)
{
    return adoptRef(new MOMDVR(context));
}

MOMDVR::MOMDVR(ScriptExecutionContext* context)
    : ActiveDOMObject(context)
{
}

MOMDVR::~MOMDVR()
{
}

EventTargetInterface MOMDVR::eventTargetInterface() const
{
    return MOMDVREventTargetInterfaceType;
}

MOMContentItem* MOMDVR::schedule() const
{
    if (!m_schedule)
        m_schedule = MOMContentItem::create(scriptExecutionContext(), (mom_content_item)mom_scheduler_recordings());
    return m_schedule.get();
}

MOMContentItem* MOMDVR::recordingSeries() const
{
    if (!m_recordingSeries)
        m_recordingSeries = MOMContentItem::create(scriptExecutionContext(), (mom_content_item)mom_scheduler_recordings_series_list());
    return m_recordingSeries.get();
}

PassRefPtr<MOMScheduledRecording> MOMDVR::record( MOMProgramme* programme, bool seriesLink, MOMRecorder* recorder, ExceptionCode& ec )
{
    mom_driver rec_driver = recorder ? mom_recorder_get_driver(recorder->recorder()) : NULL;
    mom_scheduled_recording rec = NULL;
    if (rec_driver)
    {
        rec = mom_scheduler_record_programme_on_recorder(programme->itemAsProgramme(), seriesLink, rec_driver);
    }
    else
    {
        rec = mom_scheduler_record_programme(programme->itemAsProgramme(), seriesLink);
    }

    if (rec == NULL)
    {
        MOMException::setFromMOMError(mom_err_BUSY, ec);
        return nullptr;
    }

    return MOMScheduledRecording::createAdopt(scriptExecutionContext(), rec);
}

PassRefPtr<MOMScheduledRecording> MOMDVR::record( MOMProgramme* programme, bool seriesLink, ExceptionCode& ec )
{
	return record(programme, seriesLink, 0, ec);
}

PassRefPtr<MOMScheduledRecording> MOMDVR::record( MOMProgramme* programme, ExceptionCode& ec )
{
	return record(programme, false, 0, ec);
}

void MOMDVR::removeAll()
{
    mom_scheduler_remove_all();
}

void MOMDVR::stop( MOMScheduledRecording* recording )
{
    mom_scheduler_stop(recording->itemAsRecording());
}

void MOMDVR::refresh()
{
    mom_scheduler_refresh();
}

PassRefPtr<MOMReminder> MOMDVR::setReminder( MOMProgramme* programme, ExceptionCode& ec )
{
    mom_reminder rem = mom_reminder_set(programme->itemAsProgramme());
    if (rem == NULL)
    {
        MOMException::setFromMOMError(mom_err_FAILURE, ec);
        return nullptr;
    }
    return MOMReminder::createAdopt(scriptExecutionContext(), rem);
}

PassRefPtr<MOMScheduledRecording> MOMDVR::recordNow( MOMChannel* channel, int duration, MOMRecorder* recorder, ExceptionCode& ec )
{
    return recordAt(channel, 0, duration, recorder, ec);
}

PassRefPtr<MOMScheduledRecording> MOMDVR::recordNow( MOMChannel* channel, int duration, ExceptionCode& ec )
{
	return recordNow( channel, duration, 0, ec );
}

PassRefPtr<MOMScheduledRecording> MOMDVR::recordAt( MOMChannel* channel, int start, int duration, MOMRecorder* recorder, ExceptionCode& ec )
{
    mom_driver rec_driver = recorder ? mom_recorder_get_driver(recorder->recorder()) : NULL;
    mom_programme prog = mom_programme_create_manual(mom_content_item_get_uri(channel->itemAsItem()), start, duration);
    mom_scheduled_recording rec = NULL;
    if (prog)
    {
        if (rec_driver)
        {
            rec = mom_scheduler_record_programme_on_recorder(prog, FALSE, rec_driver);
        }
        else
        {
            rec = mom_scheduler_record_programme(prog, FALSE);
        }
        mom_content_item_dispose((mom_content_item)prog);
    }

    if (rec == NULL)
    {
        MOMException::setFromMOMError(mom_err_BUSY, ec);
        return nullptr;
    }

    return MOMScheduledRecording::createAdopt(scriptExecutionContext(), rec);
}

PassRefPtr<MOMScheduledRecording> MOMDVR::recordAt( MOMChannel* channel, int start, int duration, ExceptionCode& ec )
{
	return recordAt(channel, start, duration, 0, ec);
}
} // namespace WebCore

#endif // ENABLE(RMAI)
