/** @file MOMDVR.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMDVR_h
#define MOMDVR_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

namespace WebCore {

class MOMRecorder;
class MOMProgramme;
class MOMContentItem;
class MOMScheduledRecording;
class MOMReminder;
class MOMContentList;
class MOMChannel;
class ScriptExecutionContext;

class MOMDVR : public RefCounted<MOMDVR>, public ActiveDOMObject, public EventTarget {
public:
    enum {
        ALL = 0,
        NONE = 0,
        WATCHED = 1,
        OLD = 2,
        REMOVE_THIS = 1,
        REMOVE_ALL = 2,
        REMOVE_CONFLICT = 3,
    };

    static PassRefPtr<MOMDVR> create(ScriptExecutionContext* context);

    virtual ~MOMDVR();

    MOMContentItem* schedule() const;
    MOMContentItem* recordingSeries() const;

    PassRefPtr<MOMContentItem> createFilteredList( int sortOrder, bool sortIncreasing );
    PassRefPtr<MOMScheduledRecording> record( MOMProgramme* programme, ExceptionCode& ec );
    PassRefPtr<MOMScheduledRecording> record( MOMProgramme* programme, bool seriesLink, ExceptionCode& ec );
    PassRefPtr<MOMScheduledRecording> record( MOMProgramme* programme, bool seriesLink, MOMRecorder* recorder, ExceptionCode& ec );
    void remove( MOMScheduledRecording* recording, int reason, ExceptionCode& ec );
    void removeAll();
    void stop( MOMScheduledRecording* recording );
    void refresh();
    PassRefPtr<MOMReminder> setReminder( MOMProgramme* programme, ExceptionCode& ec );
    PassRefPtr<MOMScheduledRecording> recordNow( MOMChannel* channel, int duration, ExceptionCode& ec );
    PassRefPtr<MOMScheduledRecording> recordNow( MOMChannel* channel, int duration, MOMRecorder* recorder, ExceptionCode& ec );
    PassRefPtr<MOMScheduledRecording> recordAt( MOMChannel* channel, int start, int duration, ExceptionCode& ec );
    PassRefPtr<MOMScheduledRecording> recordAt( MOMChannel* channel, int start, int duration, MOMRecorder* recorder, ExceptionCode& ec );

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMDVR*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMDVR"; }

    // EventTarget interface
    MOMDVR* toMOMDVR() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMDVR>::ref;
    using RefCounted<MOMDVR>::deref;

protected:
    MOMDVR(ScriptExecutionContext* context);

private:
    mutable RefPtr<MOMContentItem> m_schedule;
    mutable RefPtr<MOMContentItem> m_recordingSeries;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMDVR_h
