/** @file MOMDVREvent.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMDVREvent.h"
#include "MOMScheduledRecording.h"
#include "UtilsMOM.h"

namespace WebCore {

MOMDVREvent::MOMDVREvent(const mom_event_params* params, ScriptExecutionContext* context)
    : MOMEvent(AtomicString("MOM_DVR"), params)
{
    ASSERT(params->arg_count == mom_event_DVR_COUNT);
    int action = UtilsMOM::codeFromRecordingAction(params->args[mom_event_DVR_ACTION]);
    int state = UtilsMOM::codeFromRecordingState(params->args[mom_event_DVR_STATE]);

    /* for backwards compatibility the state is actually set to
     * the action, unless action was ADDED or UPDATED */
    setIntParam(mom_event_DVR_ACTION, action);
    setIntParam(mom_event_DVR_STATE, UtilsMOM::recordingActionIsAddedOrUpdated(params->args[mom_event_DVR_ACTION]) ? state : action);

    m_object = params->object ? MOMScheduledRecording::create(context, static_cast<mom_scheduled_recording>(params->object)) : nullptr;
}

MOMDVREvent::MOMDVREvent()
    : MOMEvent(AtomicString("MOM_DVR"))
{
}

} // namespace WebCore

#endif // ENABLE(RMAI)
