/** @file MOMDevice.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMDevice_h
#define MOMDevice_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_driver.h"
}

namespace WebCore {

class MOMStream;
class ScriptExecutionContext;

class MOMDevice : public RefCounted<MOMDevice>, public ActiveDOMObject, public EventTarget {
public:

    static PassRefPtr<MOMDevice> create(ScriptExecutionContext* context, mom_driver driver);
    virtual ~MOMDevice();

    String stringAttr(MDAAttr attr) const;
    int intAttr(MDAAttr attr) const;

    virtual int deviceType() const = 0;
    String uri() const;
    String id() const;
    String name() const;
    bool enabled() const;
    void setEnabled(bool enabled);

    bool isLocal();
    PassRefPtr<MOMStream> open( const String & uri, ExceptionCode& ec );
    virtual void close( MOMStream* stream );

    ALWAYS_INLINE mom_driver driver() const { return m_driver; }

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMDevice*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMDevice"; }

    // EventTarget interface
    MOMDevice* toMOMDevice() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMDevice>::ref;
    using RefCounted<MOMDevice>::deref;

protected:
    MOMDevice(ScriptExecutionContext* context, mom_driver mdh);

private:
    mom_driver m_driver;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMDevice_h
