/** @file MOMEvent.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMEvent_h
#define MOMEvent_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMContentItem.h"

#include "Event.h"
#include "EventTarget.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_core.h"
}

namespace WebCore {

class ScriptExecutionContext;

class MOMEvent : public Event {
public:

    static void init();
    static PassRefPtr<MOMEvent> create(mom_event_t which, const mom_event_params* params, ScriptExecutionContext* context);
    static mom_error dispatchMOMEventToUntypedTarget(mom_event_t which, void* target, mom_event_params* params, unsigned int queue);
    static mom_error dispatchMOMEvent(mom_event_t which, EventTarget* target, mom_event_params* params, unsigned int queue);
    static EventTarget* getTarget(mom_event_t which, void* target);
    virtual ~MOMEvent();

    String toString();

    int getIntParam(int param) const { ASSERT(param < sizeof(m_intParam)/sizeof(m_intParam[0])); return m_intParam[param]; }
    String getStringParam(int param) const { ASSERT(param < sizeof(m_stringParam)/sizeof(m_stringParam[0])); return m_stringParam[param]; }

    void setIntParam(int param, int val) { ASSERT(param < sizeof(m_intParam)/sizeof(m_intParam[0])); m_intParam[param] = val; }
    void setStringParam(int param, String val) { ASSERT(param < sizeof(m_stringParam)/sizeof(m_stringParam[0])); m_stringParam[param] = val; }
    
    enum TargetType {
        TARGET_MOM,
        TARGET_PACKAGE_MANAGER,
        TARGET_NETWORK_INTERFACE,
        TARGET_DVR,
        TARGET_DRM_AGENT
        };

    mom_event_t momEventType() const { return m_momEventType; }

    bool isMOMEvent() const { return true; }

protected:
    MOMEvent(const AtomicString& type, const mom_event_params* params = 0);

private:
    mom_event_t m_momEventType;
    int m_intParam[10];
    String m_stringParam[3];

    static MOMEvent::TargetType getTargetTypeFromMOMEventT(mom_event_t which);
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMEvent_h
