/** @file MOMException.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMException.h"

#include "ExceptionCodeDescription.h"

namespace WebCore {

static const char* const momExceptionNames[] = {
    "INVALID",
    "BUSY",
    "NOTPRESENT",
    "DISCFULL",
    "FAILURE",
    "NOTAUTHORISED",
    "RATING",
    "DIALOG",
    "START_OF_STREAM",
    "END_OF_STREAM",
    "TSB_WRAP",
    "PIN_WRONG",
    "PIN_LOCKED",
    "EXPIRED",
    "UNKNOWN",
    "NOMEM"
};

static const char* const momExceptionDescriptions[] = {
    "The operation was invalid",
    "The resource is busy",
    "The resource is not present",
    "The disc is full",
    "The operation has failed",
    "The operation cannot be authorised",
    "Not possible due to rating",
    "Not possible due to user dialog",
    "Start of stream reached",
    "End of stream reached",
    "Live pause buffer is full",
    "The wrong PIN has been entered",
    "The PIN is locked due to too many failures",
    "The asset has expired",
    "Unknown error",
    "Out of memory"
};

PassRefPtr<MOMException> MOMException::create(const ExceptionCodeDescription& description)
{
    return adoptRef(new MOMException(description));
}

MOMException::MOMException(const ExceptionCodeDescription& description)
    : ExceptionBase(description)
{
}

void MOMException::setFromMOMError( mom_error err, ExceptionCode& ec )
{
    if ( err == ERR_NONE )
        ec = 0;
    else if ( err == ERR_NO_MEMORY )
        ec = MOMException::NOMEM;
    else if ( err == mom_err_UNKNOWN )
        ec = MOMException::UNKNOWN;
    else if ( err > ERR_BASE_MOM && err < ERR_BASE_MOM + 512 )
        ec = MOMException::MOMExceptionOffset + (err - ERR_BASE_MOM);
    else
        ec = MOMException::UNKNOWN;
}

int MOMException::codeFromMOMError( mom_error err )
{
    int code;
    if ( err == ERR_NONE )
        code = 0;
    else if ( err == ERR_NO_MEMORY )
        code = 257;
    else if ( err == mom_err_UNKNOWN )
        code = 256;
    else if ( err > ERR_BASE_MOM && err < ERR_BASE_MOM + 512 )
        code = (err - ERR_BASE_MOM);
    else
        code = 256;
    return code;
}

int MOMException::codeFromMOMError( int err )
{
    return MOMException::codeFromMOMError(static_cast<mom_error>(err));
}

bool MOMException::initializeDescription(ExceptionCode ec, ExceptionCodeDescription* description)
{
    if (ec < MOMExceptionOffset || ec > MOMExceptionMax)
        return false;

    description->typeName = "MOMException";
    description->code = ec - MOMExceptionOffset;
    description->type = MOMExceptionType;

    size_t tableSize = WTF_ARRAY_LENGTH(momExceptionNames);
    size_t tableIndex = ec - MOMException::INVALID;

    description->name = tableIndex < tableSize ? momExceptionNames[tableIndex] : 0;
    description->description = tableIndex < tableSize ? momExceptionDescriptions[tableIndex] : 0;

    return true;
}

} // namespace WebCore

#endif // ENABLE(RMAI)
