/** @file MOMGuideSearch.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMGuideSearch_h
#define MOMGuideSearch_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

extern "C" {
#include "layers.h"
#include "layers_flex.h"
#include "cdf/mom_guide_search.h"
}

namespace WebCore {

class MOMContentItem;
class MOMChannel;
class MOMChannelList;
class ScriptExecutionContext;

class MOMGuideSearch : public RefCounted<MOMGuideSearch>, public ActiveDOMObject, public EventTarget {
public:

    static PassRefPtr<MOMGuideSearch> create(ScriptExecutionContext* context);
    virtual ~MOMGuideSearch();

    MOMContentItem* channels() const;
    int from() const;
    int to() const;
    int startChannel() const;
    int endChannel() const;
    MOMContentItem* programmes() const;

    bool find( MOMChannelList* channels, int startChannel, int endChannel, int from, int to, ExceptionCode& ec );
    bool findProgrammes( MOMChannel* channel, int startTime, int count, ExceptionCode& ec );
    bool findNowNext( MOMChannel* channel, ExceptionCode& ec );
    void stop(ExceptionCode& ec);
    void update();
    void clear();

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMGuideSearch*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMGuideSearch"; }

    // EventTarget interface
    MOMGuideSearch* toMOMGuideSearch() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMGuideSearch>::ref;
    using RefCounted<MOMGuideSearch>::deref;

protected:
    MOMGuideSearch(ScriptExecutionContext* context);

private:
    static mom_error eventDispatcher(mom_event_t which, void* target, mom_event_params* params, unsigned int queue);

    mom_guide_search m_search;
    mutable RefPtr<MOMContentItem> m_channels;
    mutable RefPtr<MOMContentItem> m_programmes;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMGuideSearch_h
