/** @file MOMLocalSystem.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMLocalSystem_h
#define MOMLocalSystem_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMOutputCollection.h"
#include "MOMNetworkInterfaceCollection.h"
#include "MOMSatelliteManager.h"
#include "MOMCable.h"

#include "ExceptionCode.h"

namespace WebCore {

class MOMSatelliteManager;
class MOMCable;
class ScriptExecutionContext;

class MOMLocalSystem : public RefCounted<MOMLocalSystem> {
public:
    enum {
        DVB_SUBTITLES = 1,
        CLOSED_CAPTION = 2,
        NTSC = 1,
        PAL = 2,
        SECAM = 4,
        NOTIFICATION_KEY_DOWN = 1,
        NOTIFICATION_KEY_PRESS = 2,
        NOTIFICATION_KEY_UP = 3,
        NOTIFICATION_PORTAL_OPEN = 4,
        NOTIFICATION_PORTAL_CLOSE = 5,
    };

    static PassRefPtr<MOMLocalSystem> create(ScriptExecutionContext* context);

    String softwareVersion() const;
    String hardwareVersion() const;
    bool systemReady() const;
    void setSystemReady(bool systemReady);

    bool standbyState() const;
    void setStandbyState(bool standbyState);

    bool mute() const;
    void setMute(bool mute);

    int volume() const;
    void setVolume(int volume, ExceptionCode& ec);

    String macrovisionModeByte() const;
    String macrovisionAPS() const;
    String serialNumber() const;
    int subtitleType() const;
    int tvStandard() const;
    String boxName() const;
    void setBoxName(const String & boxName);

    String boxId() const;
    MOMOutputCollection* outputs() const;
    MOMNetworkInterfaceCollection* networkInterfaces() const;
    bool dvrEnabled() const;
    bool earlyVideo() const;
    MOMSatelliteManager* satelliteManager() const;
    MOMCable* smatvManager() const;

    void sendEASEvent();
    int setMacrovisionModeByte( const String & mode );
    int setMacrovisionAPS( const String & aps );
    String getProperty( const String & property );
    int setProperty( const String & property, const String & value );
    void notifyEvent( int notificationType, int data );

protected:
    MOMLocalSystem(ScriptExecutionContext* context);

private:
    mutable RefPtr<MOMOutputCollection> m_outputs;
    mutable RefPtr<MOMNetworkInterfaceCollection> m_networkInterfaces;
    mutable RefPtr<MOMSatelliteManager> m_satelliteManager;
    mutable RefPtr<MOMCable> m_smatvManager;
    ScriptExecutionContext* m_context;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMLocalSystem_h
