/** @file MOMNetworkInterface.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMNetworkInterface.h"
#include "UtilsMOM.h"
#include "EventNames.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_networkinterface.h"
}

namespace WebCore {

PassRefPtr<MOMNetworkInterface> MOMNetworkInterface::create(ScriptExecutionContext* context, int index)
{
    return adoptRef(new MOMNetworkInterface(context, index));
}

MOMNetworkInterface::MOMNetworkInterface(ScriptExecutionContext* context, int index)
    : ActiveDOMObject(context),
      m_index(index)
{
}

MOMNetworkInterface::~MOMNetworkInterface()
{
}

EventTargetInterface MOMNetworkInterface::eventTargetInterface() const
{
    return MOMNetworkInterfaceEventTargetInterfaceType;
}

String MOMNetworkInterface::type() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_type(m_index));
}

String MOMNetworkInterface::ipAddress() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_ip_address(m_index));
}

void MOMNetworkInterface::setIpAddress(const String & ipAddress)
{
    layers_string lstr = UtilsMOM::LayersStringFromString(ipAddress);
    mom_networkinterface_set_ip_address(m_index, lstr);
    layers_string_dispose(lstr);
}

String MOMNetworkInterface::macAddress() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_mac_address(m_index));
}

bool MOMNetworkInterface::connected() const
{
    return mom_networkinterface_get_connected(m_index);
}

bool MOMNetworkInterface::enabled() const
{
    return mom_networkinterface_get_enabled(m_index);
}

void MOMNetworkInterface::setEnabled(bool enabled)
{
    mom_networkinterface_set_enabled(m_index, enabled);
}

bool MOMNetworkInterface::bandwidthManagementAvailable() const
{
    return mom_networkinterface_get_bandwidth_management_available(m_index);
}

int MOMNetworkInterface::provisionedBandwidth() const
{
    return mom_networkinterface_get_provisioned_video_bandwidth(m_index);
}

int MOMNetworkInterface::currentBandwidth() const
{
    return mom_networkinterface_get_current_video_bandwidth(m_index);
}

bool MOMNetworkInterface::dhcpEnable() const
{
    return mom_networkinterface_get_dhcp_enable(m_index);
}

void MOMNetworkInterface::setDhcpEnable(bool dhcpEnable)
{
    mom_networkinterface_set_dhcp_enable(m_index, dhcpEnable);
}

bool MOMNetworkInterface::pppoeEnable() const
{
    return mom_networkinterface_get_pppoe_enable(m_index);
}

void MOMNetworkInterface::setPppoeEnable(bool pppoeEnable)
{
    mom_networkinterface_set_pppoe_enable(m_index, pppoeEnable);
}

String MOMNetworkInterface::gatewayAddress() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_gatewayaddress(m_index));
}

void MOMNetworkInterface::setGatewayAddress(const String & gatewayAddress)
{
    layers_string lstr = UtilsMOM::LayersStringFromString(gatewayAddress);
    mom_networkinterface_set_gatewayaddress(m_index, lstr);
    layers_string_dispose(lstr);
}

String MOMNetworkInterface::subnetMask() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_subnetmask(m_index));
}

void MOMNetworkInterface::setSubnetMask(const String & subnetMask)
{
    layers_string lstr = UtilsMOM::LayersStringFromString(subnetMask);
    mom_networkinterface_set_subnetmask(m_index, lstr);
    layers_string_dispose(lstr);
}

String MOMNetworkInterface::dnsAddresses() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_dnsaddress(m_index));
}

void MOMNetworkInterface::setDnsAddresses(const String & dnsAddresses)
{
    layers_string lstr = UtilsMOM::LayersStringFromString(dnsAddresses);
    mom_networkinterface_set_dnsaddress(m_index, lstr);
    layers_string_dispose(lstr);
}

String MOMNetworkInterface::pppoeUserName() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_pppoeusername(m_index));
}

void MOMNetworkInterface::setPppoeUserName(const String & pppoeUserName)
{
    layers_string lstr = UtilsMOM::LayersStringFromString(pppoeUserName);
    mom_networkinterface_set_pppoeusername(m_index, lstr);
    layers_string_dispose(lstr);
}

String MOMNetworkInterface::pppoePassword() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_networkinterface_get_pppoepassword(m_index));
}

void MOMNetworkInterface::setPppoePassword(const String & pppoePassword)
{
    layers_string lstr = UtilsMOM::LayersStringFromString(pppoePassword);
    mom_networkinterface_set_pppoepassword(m_index, lstr);
    layers_string_dispose(lstr);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
