/** @file MOMNetworkInterface.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMNetworkInterface_h
#define MOMNetworkInterface_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "EventTarget.h"

namespace WebCore {

class ScriptExecutionContext;

class MOMNetworkInterface : public RefCounted<MOMNetworkInterface>, public ActiveDOMObject, public EventTarget {
public:
    static PassRefPtr<MOMNetworkInterface> create(ScriptExecutionContext* context, int index);

    virtual ~MOMNetworkInterface();

    String type() const;
    String ipAddress() const;
    void setIpAddress(const String & ipAddress);

    String macAddress() const;
    bool connected() const;
    bool enabled() const;
    void setEnabled(bool enabled);

    bool bandwidthManagementAvailable() const;
    int provisionedBandwidth() const;
    int currentBandwidth() const;
    bool dhcpEnable() const;
    void setDhcpEnable(bool dhcpEnable);

    bool pppoeEnable() const;
    void setPppoeEnable(bool pppoeEnable);

    String gatewayAddress() const;
    void setGatewayAddress(const String & gatewayAddress);

    String subnetMask() const;
    void setSubnetMask(const String & subnetMask);

    String dnsAddresses() const;
    void setDnsAddresses(const String & dnsAddresses);

    String pppoeUserName() const;
    void setPppoeUserName(const String & pppoeUserName);

    String pppoePassword() const;
    void setPppoePassword(const String & pppoePassword);

    ALWAYS_INLINE int index() const { return m_index; }

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMNetworkInterface*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMNetworkInterface"; }

    // EventTarget interface
    MOMNetworkInterface* toMOMNetworkInterface() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMNetworkInterface>::ref;
    using RefCounted<MOMNetworkInterface>::deref;

protected:
    MOMNetworkInterface(ScriptExecutionContext* context, int index);

private:
    int m_index;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMNetworkInterface_h
