/** @file MOMOutput.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMOutput.h"
#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_output.h"
}

namespace WebCore {

PassRefPtr<MOMOutput> MOMOutput::create(int index)
{
    return adoptRef(new MOMOutput(index));
}

MOMOutput::MOMOutput(int index)
    : m_index(index)
{
}

String MOMOutput::name() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_name(m_index));
}

String MOMOutput::type() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_type(m_index));
}

bool MOMOutput::enable() const
{
    return mom_output_get_enable(m_index);
}

void MOMOutput::setEnable(bool enable)
{
    mom_output_set_enable(m_index, enable);
}

int MOMOutput::connected() const
{
    return mom_output_get_connected(m_index);
}

bool MOMOutput::subtitleEnable() const
{
    return mom_output_get_subtitle_enable(m_index);;
}

void MOMOutput::setSubtitleEnable(bool subtitleEnable)
{
    mom_output_set_subtitle_enable(m_index, subtitleEnable);
}

String MOMOutput::videoMode() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_video_mode(m_index));
}

void MOMOutput::setVideoMode(const String & videoMode)
{
    layers_string str = UtilsMOM::LayersStringFromString(videoMode);
    mom_output_set_video_mode(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::digitalAudioMode() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_digital_audio_mode(m_index));
}

void MOMOutput::setDigitalAudioMode(const String & digitalAudioMode)
{
    layers_string str = UtilsMOM::LayersStringFromString(digitalAudioMode);
    mom_output_set_digital_audio_mode(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::audioRange() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_audio_range(m_index));
}

void MOMOutput::setAudioRange(const String & audioRange)
{
    layers_string str = UtilsMOM::LayersStringFromString(audioRange);
    mom_output_set_audio_range(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::hdVideoFormat() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_hd_video_format(m_index));
}

void MOMOutput::setHdVideoFormat(const String & hdVideoFormat)
{
    layers_string str = UtilsMOM::LayersStringFromString(hdVideoFormat);
    mom_output_set_hd_video_format(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::hdVideoFormatsEnabled() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_hd_video_formats_enabled(m_index));
}

void MOMOutput::setHdVideoFormatsEnabled(const String & hdVideoFormatsEnabled)
{
    layers_string str = UtilsMOM::LayersStringFromString(hdVideoFormatsEnabled);
    mom_output_set_hd_video_formats_enabled(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::tvAspectRatio() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_tv_aspect_ratio(m_index));
}

void MOMOutput::setTvAspectRatio(const String & tvAspectRatio)
{
    layers_string str = UtilsMOM::LayersStringFromString(tvAspectRatio);
    mom_output_set_tv_aspect_ratio(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccAnalogueService() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_analogue_service(m_index));
}

void MOMOutput::setCcAnalogueService(const String & ccAnalogueService)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccAnalogueService);
    mom_output_set_cc_analogue_service(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccDigitalService() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_digital_service(m_index));
}

void MOMOutput::setCcDigitalService(const String & ccDigitalService)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccDigitalService);
    mom_output_set_cc_digital_service(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccBackgroundStyle() const
{ 
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_background_style(m_index));
}

void MOMOutput::setCcBackgroundStyle(const String & ccBackgroundStyle)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccBackgroundStyle);
    mom_output_set_cc_background_style(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccBackgroundColour() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_background_colour(m_index));
}

void MOMOutput::setCcBackgroundColour(const String & ccBackgroundColour)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccBackgroundColour);
    mom_output_set_cc_background_colour(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccCharacterColour() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_character_colour(m_index));
}

void MOMOutput::setCcCharacterColour(const String & ccCharacterColour)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccCharacterColour);
    mom_output_set_cc_character_colour(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccCharacterSize() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_character_size(m_index));
}

void MOMOutput::setCcCharacterSize(const String & ccCharacterSize)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccCharacterSize);
    mom_output_set_cc_character_size(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccCharacterEdge() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_character_edge(m_index));
}

void MOMOutput::setCcCharacterEdge(const String & ccCharacterEdge)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccCharacterEdge);
    mom_output_set_cc_character_edge(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccCharacterFont() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_character_font(m_index));
}

void MOMOutput::setCcCharacterFont(const String & ccCharacterFont)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccCharacterFont);
    mom_output_set_cc_character_font(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccWindowColor() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_window_color(m_index));
}

void MOMOutput::setCcWindowColor(const String & ccWindowColor)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccWindowColor);
    mom_output_set_cc_window_color(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccWindowStyle() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_window_style(m_index));
}

void MOMOutput::setCcWindowStyle(const String & ccWindowStyle)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccWindowStyle);
    mom_output_set_cc_window_style(m_index, str);
    layers_string_dispose(str);
}

String MOMOutput::ccCharacterStyle() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_output_get_cc_character_style(m_index));
}

void MOMOutput::setCcCharacterStyle(const String & ccCharacterStyle)
{
    layers_string str = UtilsMOM::LayersStringFromString(ccCharacterStyle);
    mom_output_set_cc_character_style(m_index, str);
    layers_string_dispose(str);
}

bool MOMOutput::ccStyleSetByProgramme() const
{
    return mom_output_get_cc_style_set_by_programme(m_index);;
}

void MOMOutput::setCcStyleSetByProgramme(bool ccStyleSetByProgramme)
{
    mom_output_set_cc_style_set_by_programme(m_index, ccStyleSetByProgramme);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
