/** @file MOMPackageDescriptor.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMPackageDescriptor.h"
#include "MOMStringCollection.h"

#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_package_manager.h"
#include "cdf/mom_package_descriptor.h"
}

namespace WebCore {

MOMPackageDescriptor::MOMPackageDescriptor(ScriptExecutionContext *context, mom_content_item item)
    : MOMContentItem(context, item)
{
}

String MOMPackageDescriptor::name() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_package_descriptor_get_name(itemAsPackage()));
}

String MOMPackageDescriptor::longName() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_package_descriptor_get_long_name(itemAsPackage()));
}

String MOMPackageDescriptor::description() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_package_descriptor_get_description(itemAsPackage()));
}

int MOMPackageDescriptor::type() const
{
    return mom_package_descriptor_get_type(itemAsPackage());
}

String MOMPackageDescriptor::packageType() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_package_type_str(itemAsPackage()));
}

int MOMPackageDescriptor::availability() const
{
    return mom_package_descriptor_get_availability(itemAsPackage());
}

bool MOMPackageDescriptor::upgradeToBuiltIn() const
{
    return mom_package_descriptor_get_upgrade_to_builtin(itemAsPackage());
}

int MOMPackageDescriptor::usageType() const
{
    return mom_package_descriptor_get_usage_type(itemAsPackage());
}

String MOMPackageDescriptor::version() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_version(itemAsPackage()));
}

String MOMPackageDescriptor::uri() const
{
    return UtilsMOM::StringFromLayersString(mom_content_item_get_uri(itemAsItem()));
}

String MOMPackageDescriptor::baseUri() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_base_uri(itemAsPackage()));
}

String MOMPackageDescriptor::launchUri() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_launch_uri(itemAsPackage()));
}

String MOMPackageDescriptor::fallbackUri() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_fallback_uri(itemAsPackage()));
}

String MOMPackageDescriptor::organisationID() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_organisation_id(itemAsPackage()));
}

String MOMPackageDescriptor::packageID() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_package_id(itemAsPackage()));
}

int MOMPackageDescriptor::isLaunchable() const
{
    return mom_package_descriptor_is_launchable(itemAsPackage());
}

bool MOMPackageDescriptor::autostart() const
{
    return mom_package_descriptor_is_auto_start(itemAsPackage());
}

int MOMPackageDescriptor::stored() const
{
    return mom_package_descriptor_get_stored(itemAsPackage());
}

int MOMPackageDescriptor::priority() const
{
    return mom_package_descriptor_get_priority(itemAsPackage());
}

int MOMPackageDescriptor::terminationBehaviour() const
{
    return mom_package_descriptor_get_termination_behaviour(itemAsPackage());
}

int MOMPackageDescriptor::group() const
{
    return mom_package_descriptor_get_group(itemAsPackage());
}

int MOMPackageDescriptor::flexMemoryRequired() const
{
    return mom_package_descriptor_get_required_memory(itemAsPackage(), mom_package_descriptor_required_memory_FLEX);
}

int MOMPackageDescriptor::mallocMemoryRequired() const
{
    return mom_package_descriptor_get_required_memory(itemAsPackage(), mom_package_descriptor_required_memory_MALLOC);
}

String MOMPackageDescriptor::author() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_author(itemAsPackage()));
}

bool MOMPackageDescriptor::displayInMenu() const
{
    return mom_package_descriptor_get_display_in_menu(itemAsPackage());
}

bool MOMPackageDescriptor::serviceBound() const
{
    return mom_package_descriptor_get_service_bound(itemAsPackage());
}

int MOMPackageDescriptor::menuPriority() const
{
    return mom_package_descriptor_get_menu_priority(itemAsPackage());
}

PassRefPtr<MOMStringCollection> MOMPackageDescriptor::hotkeys() const
{
    return MOMStringCollection::create(mom_package_descriptor_get_hotkeys(itemAsPackage()));
}

String MOMPackageDescriptor::license() const
{
    return UtilsMOM::StringFromLayersString(mom_package_descriptor_get_license(itemAsPackage()));
}

bool MOMPackageDescriptor::running() const
{
    return mom_package_descriptor_get_running(itemAsPackage());
}

void MOMPackageDescriptor::setRunning(bool running)
{
    mom_package_descriptor_set_running(itemAsPackage(), running);
}

bool MOMPackageDescriptor::active() const
{
    return mom_package_descriptor_get_active(itemAsPackage());
}

void MOMPackageDescriptor::setActive(bool active)
{
    mom_package_descriptor_set_active(itemAsPackage(), active);
}

bool MOMPackageDescriptor::authorised() const
{
    return mom_package_descriptor_get_authorised(itemAsPackage());
}

String MOMPackageDescriptor::getIcon( int width, int height, const String & variant )
{
    layers_string lvariant = UtilsMOM::LayersStringFromString(variant);
    String str = UtilsMOM::StringFromLayersStringAndDispose(mom_package_manager_get_package_icon(itemAsPackage(), width, height, lvariant));
    layers_string_dispose(lvariant);
    return str;
}

String MOMPackageDescriptor::getIcon( int width, int height)
{
	return getIcon(width, height, String());
}

String MOMPackageDescriptor::getIcon( int width)
{
	return getIcon(width, 0);
}

String MOMPackageDescriptor::getIcon()
{
	return getIcon(0);
}

String MOMPackageDescriptor::getProperty( const String & name )
{
    layers_string lname = UtilsMOM::LayersStringFromString(name);
    mom_urilist result_strcol = NULL;
    mom_package_manager_get_package_property(itemAsPackage(), lname, &result_strcol );
    layers_string_dispose(lname);
    
    String str = UtilsMOM::StringFromLayersStringAndDispose(mom_urilist_create_csv(result_strcol));
    mom_urilist_dispose(result_strcol);
    return str;
}

PassRefPtr<MOMStringCollection> MOMPackageDescriptor::getPropertyCollection( const String & name )
{
    layers_string lname = UtilsMOM::LayersStringFromString(name);
    mom_urilist result_strcol = NULL;
    mom_package_manager_get_package_property(itemAsPackage(), lname, &result_strcol );
    layers_string_dispose(lname);
    return result_strcol ? MOMStringCollection::createAdopt(result_strcol) : nullptr;
}

String MOMPackageDescriptor::resolveUri( const String & fragment )
{
    layers_string lfragment = UtilsMOM::LayersStringFromString(fragment);
    String str = UtilsMOM::StringFromLayersStringAndDispose(mom_package_descriptor_resolve_uri(itemAsPackage(), lfragment));
    layers_string_dispose(lfragment);
    return str;
}

String MOMPackageDescriptor::getName( const String & language )
{
    if (language.isEmpty())
        return MOMContentItem::name();

    layers_string llang = UtilsMOM::LayersStringFromString(language);
    String str = UtilsMOM::StringFromLayersStringAndDispose(mom_package_descriptor_get_name_for_lang(itemAsPackage(), llang));
    layers_string_dispose(llang);
    return str;
}

String MOMPackageDescriptor::getName()
{
	return getName(String());
}
} // namespace WebCore

#endif // ENABLE(RMAI)
