/** @file MOMPackageDescriptor.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMPackageDescriptor_h
#define MOMPackageDescriptor_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>
#include "MOMContentItem.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_package_descriptor.h"
}

namespace WebCore {

class MOMStringCollection;

class MOMPackageDescriptor : public MOMContentItem {
public:
    enum {
        SIGNALLED = 1,
        STORED = 2,
        BUILT_IN = 4,
        DOWNLOADED = 8,
        PUSHED = 16,
        TYPE_APPLICATION = 1,
        TYPE_LANGUAGE_PACK = 2,
        TYPE_THEME = 4,
        TYPE_LOGO = 8,
        NOT_LAUNCHABLE = 1,
        LAUNCHABLE_BY_USER = 2,
        LAUNCHABLE = 3,
        NOT_STORED = 0,
        USAGE_TYPE_NONE = 0,
        USAGE_TYPE_TEXT_APPLICATION = 1,
    };

    MOMPackageDescriptor(ScriptExecutionContext *context, mom_content_item item);

    String name() const;
    String longName() const;
    String description() const;
    int type() const;
    String packageType() const;
    int availability() const;
    bool upgradeToBuiltIn() const;
    int usageType() const;
    String version() const;
    String uri() const;
    String baseUri() const;
    String launchUri() const;
    String fallbackUri() const;
    String organisationID() const;
    String packageID() const;
    int isLaunchable() const;
    bool autostart() const;
    int stored() const;
    int priority() const;
    int terminationBehaviour() const;
    int group() const;
    int flexMemoryRequired() const;
    int mallocMemoryRequired() const;
    String author() const;
    bool displayInMenu() const;
    bool serviceBound() const;
    int menuPriority() const;
    PassRefPtr<MOMStringCollection> hotkeys() const;
    String license() const;
    bool running() const;
    void setRunning(bool running);

    bool active() const;
    void setActive(bool active);

    bool authorised() const;

    String getIcon();
    String getIcon( int width);
    String getIcon( int width, int height);
    String getIcon( int width, int height, const String & variant );
    String getProperty( const String & name );
    PassRefPtr<MOMStringCollection> getPropertyCollection( const String & name );
    String resolveUri( const String & fragment );
    String getName();
    String getName( const String & language );

    ALWAYS_INLINE mom_package_descriptor itemAsPackage() const { return (mom_package_descriptor)m_item; }
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMPackageDescriptor_h
