/** @file MOMPackageEvent.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMPackageEvent_h
#define MOMPackageEvent_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMEvent.h"

namespace WebCore {

class MOMContentItem;

class MOMPackageEvent : public MOMEvent {
public:
    enum {
        ADDED = 0,
        REMOVED = 1,
        CHANGED = 2,
        TERMINATE = 3,
        REQUEST_STORE = 4,
        REQUEST_DELETE = 5,
        STORED = 6,
        DELETED = 7,
        UPDATED = 8,
        STORAGE_FAILED = 9,
        UPDATE_FAILED = 10,
        DEFERRED = 11,
        LIST_UPDATED = 12,
        SERVICE_SELECTED = 13,
        STORING = 14,
        PROGRESS = 15,
        REASON_UNAVAILABLE = 0,
        REASON_INVALID = 1,
        REASON_NO_SPACE = 2,
        REASON_NOT_STORABLE = 3,
        REASON_NOT_AUTHORISED = 4,
        REASON_NATIVE_APP = 5,
    };

    static PassRefPtr<MOMPackageEvent> create()
    {
        return adoptRef(new MOMPackageEvent);
    }
    
    int code() const { return getIntParam(mom_event_PACKAGE_CODE); }
    int reason() const { return getIntParam(mom_event_PACKAGE_REASON); }
    int progress() const { return getIntParam(mom_event_PACKAGE_PROGRESS); }
    int length() const { return getIntParam(mom_event_PACKAGE_LENGTH); }

    MOMContentItem* pkg() const { return m_object.get(); }

    MOMPackageEvent(const mom_event_params* params, ScriptExecutionContext* context);
private:
    MOMPackageEvent();
    mutable RefPtr<MOMContentItem> m_object;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMPackageEvent_h
