/** @file MOMPackageManager.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMPackageManager_h
#define MOMPackageManager_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMPackageList.h"

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

namespace WebCore {

class MOMPackageDescriptor;
class ScriptExecutionContext;

class MOMPackageManager : public RefCounted<MOMPackageManager>, public ActiveDOMObject, public EventTarget {
public:

    static PassRefPtr<MOMPackageManager> create(ScriptExecutionContext* context);
    static void init();

    virtual ~MOMPackageManager();

    PassRefPtr<MOMPackageList> all() const;

    PassRefPtr<MOMPackageList> createFilteredList( int type, int signalled, int stored, int builtin, int displayInMenu );
    void store( const String & pkg );
    void store( MOMPackageDescriptor* pkg );
    void upgrade( MOMPackageDescriptor* pkg );
    void remove( MOMPackageDescriptor* pkg );
    void setPackageRoot( const String & root, int period );
    void setManifestUri( const String & uri, const String & base_uri );
    void scanDirectory( const String & directory );

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMPackageManager*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMPackageManager"; }

    // EventTarget interface
    MOMPackageManager* toMOMPackageManager() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMPackageManager>::ref;
    using RefCounted<MOMPackageManager>::deref;

protected:
    MOMPackageManager(ScriptExecutionContext* context);

private:
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMPackageManager_h
