/** @file MOMParentalRatingSchemeCollection.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMParentalRatingSchemeCollection_h
#define MOMParentalRatingSchemeCollection_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

namespace WebCore {

class MOMContentItem;
class MOMStringCollection;
class MOMParentalRatingScheme;

class MOMParentalRatingSchemeCollection : public RefCounted<MOMParentalRatingSchemeCollection> {
public:

    static PassRefPtr<MOMParentalRatingSchemeCollection> create();

    int length() const;
    bool isPINEntryLocked() const;
    bool isPurchasePINEntryLocked() const;

    PassRefPtr<MOMParentalRatingScheme> item( int index );
    int setPIN( const String & oldPcPIN, const String & newPcPIN );
    int setPurchasePIN( const String & oldPurchasePIN, const String & newPurchasePIN );
    int setPINEnable( const String & pcPIN, bool enable );
    int setPurchasePINEnable( const String & pcPIN, bool enable );
    bool getPINEnable();
    bool getPurchasePINEnable();
    int unlockWithPIN( const String & pcPIN, MOMContentItem* target, int duration );
    int verifyPIN( const String & pcPIN );
    int verifyPurchasePIN( const String & purchasePIN );
    int setBlockUnrated( const String & pcPIN, bool block );
    bool getBlockUnrated();
    int setShowTitlesWhenLocked( const String & pcPIN, bool show, bool suppressEvents );
    bool getShowTitlesWhenLocked();
    int setParentalControlEnable( const String & pcPIN, bool enable );
    bool getParentalControlEnable();
    int setContentAdvisories( const String & pcPIN, const String & advisories );
    PassRefPtr<MOMStringCollection> getContentAdvisories();

protected:
    MOMParentalRatingSchemeCollection();
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMParentalRatingSchemeCollection_h
