/** @file MOMPurchasableAsset.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMPurchasableAsset_h
#define MOMPurchasableAsset_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>
#include "MOMContentItem.h"
#include "ExceptionCode.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_purchasable_asset.h"
}

namespace WebCore {

class MOMProgramme;

class MOMPurchasableAsset : public MOMContentItem {
public:
    enum {
        NOT_PURCHASABLE = 0,
        BEFORE_PURCHASE_WINDOW = 1,
        PURCHASABLE = 2,
        AFTER_PURCHASE_WINDOW = 3,
        UNPURCHASED = 0,
        PURCHASE_WINDOW_PENDING = 1,
        PURCHASE_PENDING = 2,
        PURCHASE_ENTITLEMENTS_PENDING = 3,
        PURCHASE_COMPLETED = 4,
        PURCHASE_FAILED = 5,
        PURCHASE_CANCELLING = 6,
        PURCHASE_CANCELLED = 7,
        PPV_TYPE_IPPV = 1,
        PPV_TYPE_RPPV = 2,
    };

    MOMPurchasableAsset(ScriptExecutionContext *context, mom_content_item item);

    String price() const;
    int purchasable() const;
    int purchaseWindowStart() const;
    int purchaseWindowEnd() const;
    bool advertisable() const;
    int advertisingWindowStart() const;
    int advertisingWindowEnd() const;
    bool previewable() const;
    int previewWindowStart() const;
    int previewWindowEnd() const;
    bool cancellable() const;
    int cancelWindowEnd() const;
    String previewUri() const;
    String contactTelephone() const;
    int contactTelephoneWindowStart() const;
    int purchased() const;
    int purchaseDate() const;
    String purchaseDiagnostic() const;
    String cancelDiagnostic() const;
    PassRefPtr<MOMProgramme> programme(ExceptionCode& ec) const;
    int purchaseTypes() const;

    int purchase( const String & pin, ExceptionCode& ec );
    int cancel(ExceptionCode& ec);
    bool isReady(ExceptionCode& ec);
    PassRefPtr<MOMProgramme> findNextShowing(ExceptionCode& ec);

    ALWAYS_INLINE mom_purchasable_asset itemAsPurchasableAsset() const { return (mom_purchasable_asset)m_item; }
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMPurchasableAsset_h
