/** @file MOMRecorder.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMRecorder_h
#define MOMRecorder_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>
#include "MOMDevice.h"
#include "MOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_recorder.h"
}

namespace WebCore {

class MOMDiscInfo;
class MOMStream;
class MOMStringCollection;

class MOMRecorder : public MOMDevice {
public:
    enum {
        DISC_OPERATION_FORMAT = 0,
        DISC_OPERATION_CHECK = 1,
    };

    static PassRefPtr<MOMRecorder> create(ScriptExecutionContext* context, mom_driver driver);
    ~MOMRecorder();

    PassRefPtr<MOMStringCollection> recordings() const;
    int deviceType() const { return MOM::RECORDER; }
    bool isRemovable() const;
    MOMDiscInfo* discInfo() const;
    int tsbMaxLength() const;
    void setTsbMaxLength(int tsbMaxLength, ExceptionCode& ec);

    String boxName() const;
    String boxId() const;

    String record( MOMStream* source, ExceptionCode& ec );
    void close( MOMStream* stream );
    void erase( const String & uri, ExceptionCode& ec );
    void beginDiscOperation( int operation, ExceptionCode& ec );
    void cancelDiscOperation(ExceptionCode& ec);

    ALWAYS_INLINE mom_recorder recorder() const { return m_recorder; }

protected:
    MOMRecorder(ScriptExecutionContext* context, mom_driver driver);

private:
    static mom_error eventDispatcher(mom_event_t which, void* target, mom_event_params* params, unsigned int queue);

    mom_recorder m_recorder;
    mutable RefPtr<MOMDiscInfo> m_discInfo;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMRecorder_h
