/** @file MOMRenderer.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMRenderer_h
#define MOMRenderer_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_renderer.h"
}

namespace WebCore {

class MOMComponentCollection;
class MOMComponent;
class MOMStream;
class ScriptExecutionContext;

class MOMRenderer : public RefCounted<MOMRenderer>, public ActiveDOMObject, public EventTarget {
public:
    enum {
        POSITION_START = 0,
        POSITION_CURRENT = 1,
        POSITION_END = 2,
    };

    static PassRefPtr<MOMRenderer> create(ScriptExecutionContext* context, mom_renderer renderer, bool isPlayer);
    virtual ~MOMRenderer();

    virtual int offset() const;
    virtual int duration() const;

    virtual void connect( MOMStream* stream, ExceptionCode& ec ) { connect(stream, 0, ec); }
    virtual void connect( MOMStream* stream, int mode, ExceptionCode& ec );
    virtual void disconnect();
    virtual void play( int speed, ExceptionCode& ec );
    virtual void rendererStop();
    virtual void seek( int offset, int reference, ExceptionCode& ec );

    PassRefPtr<MOMComponentCollection> getComponents( int componentType );
    void selectComponent( MOMComponent* component );
    void removeComponent( MOMComponent* component );

    ALWAYS_INLINE mom_renderer renderer() const { return m_renderer; }
    bool isPlayer() const { return m_isPlayer; }

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMRenderer*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMRenderer"; }

    // EventTarget interface
    MOMRenderer* toMOMRenderer() { return this; }
    using RefCounted<MOMRenderer>::ref;
    using RefCounted<MOMRenderer>::deref;

    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

protected:
    MOMRenderer(ScriptExecutionContext* context, mom_renderer renderer, bool withEvents);

private:
    static mom_error eventDispatcher(mom_event_t which, void* target, mom_event_params* params, unsigned int queue);

    mom_renderer m_renderer;
    EventTargetData m_eventTargetData;
    bool m_isPlayer;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMRenderer_h
