/** @file MOMResourceConflictEvent.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMResourceConflictEvent_h
#define MOMResourceConflictEvent_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMEvent.h"

namespace WebCore {

class MOMContentItem;

class MOMResourceConflictEvent : public MOMEvent {
public:
    enum {
        SCHEDULED_RECORDING = 1,
        UPCOMING_RECORDING = 2,
        CURRENT_RECORDING = 3,
        TUNE = 4,
        SERIES_RECORDING = 5,
        SERIES_CONVERSION = 6,
        DATA_DOWNLOAD = 7,
        SERIES_PRIORITY = 8,
        IMAGE_DOWNLOAD = 9,
        RESOURCE_TUNERS = 1,
        RESOURCE_RECORDING_STREAMS = 2,
        RESOURCE_BANDWIDTH = 3,
        RESOURCE_DISK_SPACE = 4,
        RESOURCE_IMAGE_DOWNLOAD = 5,
    };
    static PassRefPtr<MOMResourceConflictEvent> create()
    {
        return adoptRef(new MOMResourceConflictEvent);
    }
    
    int reason() const { return getIntParam(mom_event_RESOURCECONFLICT_REASON); }
    int resourceType() const { return getIntParam(mom_event_RESOURCECONFLICT_TYPE); }
    bool manualRestart() const { return getIntParam(mom_event_RESOURCECONFLICT_MANUAL); }
    bool otherBoxes() const { return getIntParam(mom_event_RESOURCECONFLICT_OTHER_BOXES); }
    MOMContentItem* items() const { return m_object.get(); }

    MOMResourceConflictEvent(const mom_event_params* params, ScriptExecutionContext* context);
private:
    MOMResourceConflictEvent();
    mutable RefPtr<MOMContentItem> m_object;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMResourceConflictEvent_h
