/** @file MOMResourceConflictItem.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMResourceConflictItem.h"
#include "MOMContentItem.h"
#include "MOMContentList.h"

#include "UtilsMOM.h"

namespace WebCore {

MOMResourceConflictItem::MOMResourceConflictItem(ScriptExecutionContext *context, mom_content_item item)
    : MOMContentItem(context, item)
{
}

String MOMResourceConflictItem::boxId() const
{
    return UtilsMOM::StringFromLayersString(mom_resource_conflict_item_get_box_id(itemAsResourceConflictItem()));
}

String MOMResourceConflictItem::boxName() const
{
    return UtilsMOM::StringFromLayersString(mom_resource_conflict_item_get_box_name(itemAsResourceConflictItem()));
}

bool MOMResourceConflictItem::isLocal() const
{
    return mom_resource_conflict_item_is_local(itemAsResourceConflictItem());
}

bool MOMResourceConflictItem::cancelled() const
{
    return mom_resource_conflict_item_get_cancelled(itemAsResourceConflictItem());
}

void MOMResourceConflictItem::setCancelled(bool cancelled)
{
    mom_resource_conflict_item_set_cancelled(itemAsResourceConflictItem(), cancelled);
}

int MOMResourceConflictItem::referredType() const
{
    return mom_resource_conflict_item_get_referred_type(itemAsResourceConflictItem());
}

MOMContentItem* MOMResourceConflictItem::referredItem() const
{
    if (!m_referredItem)
        m_referredItem = MOMContentItem::create(scriptExecutionContext(), mom_resource_conflict_item_get_referred_item(itemAsResourceConflictItem()));
    return m_referredItem.get();
}

MOMContentList* MOMResourceConflictItem::conflicts() const
{
    if (!m_conflicts)
        m_conflicts = MOMContentList::create(scriptExecutionContext(), (mom_content_list)mom_resource_conflict_item_get_conflicts(itemAsResourceConflictItem()));
    return m_conflicts.get();
}

int MOMResourceConflictItem::startTime() const
{
    return mom_resource_conflict_item_get_start_time(itemAsResourceConflictItem());
}

int MOMResourceConflictItem::duration() const
{
    return mom_resource_conflict_item_get_duration(itemAsResourceConflictItem());
}

bool MOMResourceConflictItem::suspended() const
{
    return mom_resource_conflict_item_get_suspended(itemAsResourceConflictItem());
}

void MOMResourceConflictItem::setSuspended(bool suspended)
{
    mom_resource_conflict_item_set_suspended(itemAsResourceConflictItem(), suspended);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
