/** @file MOMResourceConflictList.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMResourceConflictList.h"
#include "MOMContentList.h"

#include "MOMException.h"

namespace WebCore {

MOMResourceConflictList::MOMResourceConflictList(ScriptExecutionContext *context, mom_content_item item)
    : MOMContentList(context, item)
{
}

bool MOMResourceConflictList::hasConflicts() const
{
    return mom_resource_conflict_list_has_conflicts(itemAsResourceConflictList());
}

bool MOMResourceConflictList::ignorePadding() const
{
    return mom_resource_conflict_list_get_ignore_padding(itemAsResourceConflictList());
}

void MOMResourceConflictList::setIgnorePadding(bool ignorePadding)
{
    mom_resource_conflict_list_set_ignore_padding(itemAsResourceConflictList(), ignorePadding);
}

void MOMResourceConflictList::commit()
{
    mom_resource_conflict_list_commit(itemAsResourceConflictList());
}

void MOMResourceConflictList::dispose()
{
    mom_resource_conflict_list_dispose(itemAsResourceConflictList());
}

void MOMResourceConflictList::setPriority( bool highest )
{
    mom_resource_conflict_list_set_priority(itemAsResourceConflictList(), highest);
}

PassRefPtr<MOMContentList> MOMResourceConflictList::seriesResolutionPolicy( int policy, ExceptionCode& ec )
{
    mom_content_list list;
    mom_error err = mom_resource_conflict_list_series_resolution_policy(itemAsResourceConflictList(), static_cast<MDAConflictResolverSeriesPolicy>(policy), &list);
    if (err != ERR_NONE)
    {
        MOMException::setFromMOMError(err, ec);
        return nullptr;
    }
    return MOMContentList::createAdopt(scriptExecutionContext(), list);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
