/** @file MOMSatellite.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMSatellite_h
#define MOMSatellite_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMContentItem.h"
#include "MOMContentList.h"
#include "ExceptionCode.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_satellite.h"
}

namespace WebCore {

class MOMTransportStream;
class MOMChannelScanOptions;
class ScriptExecutionContext;

class MOMSatellite : public MOMContentItem {
public:
    enum {
        LNB_UNIVERSAL = 0,
        LNB_SINGLE = 1,
    };

    static PassRefPtr<MOMSatellite> createAdopt(ScriptExecutionContext *context, mom_satellite satellite);
    MOMSatellite(ScriptExecutionContext* context, mom_content_item item);

    bool enabled() const;
    void setEnabled(bool enabled);

    bool userCreated() const;
    bool useTone() const;
    void setUseTone(bool useTone);

    bool lNBPower() const;
    void setLNBPower(bool LNBPower);

    int lNBType() const;
    void setLNBType(int LNBType);

    int lNBFrequencyLowMHz() const;
    void setLNBFrequencyLowMHz(int LNBFrequencyLowMHz);

    int lNBFrequencyHighMHz() const;
    void setLNBFrequencyHighMHz(int LNBFrequencyHighMHz);

    int diSEqCPort() const;
    void setDiSEqCPort(int DiSEqCPort);

    MOMContentList* transportStreams() const;
    MOMTransportStream* homeTransponder() const;

    PassRefPtr<MOMTransportStream> createTransportStream(ExceptionCode& ec);
    void remove( MOMTransportStream* transportStream );
    void startScan();
    void startScan( MOMChannelScanOptions* options );

    ALWAYS_INLINE mom_satellite itemAsSatellite() const { return (mom_satellite)m_item; }

private:
    mutable RefPtr<MOMContentList> m_transportStreams;
    mutable RefPtr<MOMTransportStream> m_homeTransponder;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMSatellite_h
