/** @file MOMSatelliteManager.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMSatelliteManager.h"
#include "MOMSatellite.h"
#include "MOMContentItem.h"

#include "MOMException.h"
#include "UtilsMOM.h"

namespace WebCore {

PassRefPtr<MOMSatelliteManager> MOMSatelliteManager::create(ScriptExecutionContext *context)
{
    return adoptRef(new MOMSatelliteManager(context));
}

MOMSatelliteManager::MOMSatelliteManager(ScriptExecutionContext* context)
    : m_context(context)
{
}

MOMContentList* MOMSatelliteManager::satellites() const
{
    if (!m_satellites)
        m_satellites = MOMContentList::create(m_context, mom_satellite_collection_get());
    return m_satellites.get();
}

MOMContentList* MOMSatelliteManager::enabledSatellites() const
{
    if (!m_enabledSatellites)
        m_enabledSatellites = MOMContentList::create(m_context, mom_satellite_enabled_collection_get());
    return m_enabledSatellites.get();
}

PassRefPtr<MOMSatellite> MOMSatelliteManager::createSatellite( const String & name, ExceptionCode& ec )
{
    layers_string lname = UtilsMOM::LayersStringFromString(name);
    mom_satellite satellite;
    mom_error err = mom_satellite_object_create(lname, &satellite);
    layers_string_dispose(lname);
    if (err)
    {
        MOMException::setFromMOMError(err, ec);
        return nullptr;
    }
    return MOMSatellite::createAdopt(m_context, satellite);
}

void MOMSatelliteManager::remove( MOMSatellite* satellite )
{
    mom_satellite_object_destroy(satellite->itemAsSatellite());
}

} // namespace WebCore

#endif // ENABLE(RMAI)
