/** @file MOMScheduledRecording.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMPlayer.h"
#include "MOMContentItem.h"
#include "MOMScheduledRecording.h"

#include "MOMException.h"
#include "UtilsMOM.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_scheduler.h"
#include "cdf/mom_parentalcontrols.h"
}

namespace WebCore {

PassRefPtr<MOMScheduledRecording> MOMScheduledRecording::createAdopt(ScriptExecutionContext *context, mom_scheduled_recording rec)
{
    return static_pointer_cast<MOMScheduledRecording>(MOMContentItem::create(context, (mom_content_item)rec));
}

PassRefPtr<MOMScheduledRecording> MOMScheduledRecording::create(ScriptExecutionContext *context, mom_scheduled_recording rec)
{
    return static_pointer_cast<MOMScheduledRecording>(MOMContentItem::create(context, (mom_content_item)rec));
}

MOMScheduledRecording::MOMScheduledRecording(ScriptExecutionContext *context, mom_content_item item)
    : MOMProgramme(context, item)
{
}

String MOMScheduledRecording::id() const
{
    return UtilsMOM::StringFromLayersString(mom_content_item_get_uri(itemAsItem()));
}

int MOMScheduledRecording::recordingDuration() const
{
    return mom_recording_get_duration(itemAsRecording());
}

int MOMScheduledRecording::recordingStartTime() const
{
    return mom_recording_get_start_time(itemAsRecording());
}

int MOMScheduledRecording::state() const
{
    return UtilsMOM::codeFromRecordingState(mom_recording_get_recording_state(itemAsRecording()));
}

MOMContentItem* MOMScheduledRecording::programme() const
{
    return const_cast<MOMScheduledRecording*>(this);
}

bool MOMScheduledRecording::isManual() const
{
    return mom_recording_get_manual(itemAsRecording());
}

bool MOMScheduledRecording::isSeries() const
{
    return mom_recording_get_series_link(itemAsRecording());
}

void MOMScheduledRecording::setIsSeries(bool isSeries)
{
    mom_recording_set_series_link(itemAsRecording(), isSeries);
}

PassRefPtr<MOMScheduledRecording> MOMScheduledRecording::series() const
{
    mom_scheduled_recording series = mom_recording_get_series(itemAsRecording());
    return series ? MOMScheduledRecording::create(scriptExecutionContext(), series) : nullptr;
}

bool MOMScheduledRecording::doNotDelete() const
{
    return mom_recording_get_keep_flag(itemAsRecording());
}

void MOMScheduledRecording::setDoNotDelete(bool doNotDelete)
{
    mom_recording_set_keep_flag(itemAsRecording(), doNotDelete);
}

int MOMScheduledRecording::repeatDays() const
{
    return mom_recording_get_repeat_days(itemAsRecording());
}

void MOMScheduledRecording::setRepeatDays(int repeatDays)
{
    mom_recording_set_repeat_days(itemAsRecording(), repeatDays);
}

int MOMScheduledRecording::saveTime() const
{
    return mom_recording_get_save_days(itemAsRecording());
}

void MOMScheduledRecording::setSaveTime(int saveTime)
{
    mom_recording_set_save_days(itemAsRecording(), saveTime);
}

int MOMScheduledRecording::saveEpisodes() const
{
    return mom_recording_get_save_episodes(itemAsRecording());
}

void MOMScheduledRecording::setSaveEpisodes(int saveEpisodes)
{
    mom_recording_set_save_episodes(itemAsRecording(), saveEpisodes);
}

int MOMScheduledRecording::viewed() const
{
    return mom_recording_get_viewed_state(itemAsRecording());
}

String MOMScheduledRecording::boxName() const
{
    return UtilsMOM::StringFromLayersString(mom_recording_get_box_name(itemAsRecording()));
}

String MOMScheduledRecording::boxId() const
{
    return UtilsMOM::StringFromLayersString(mom_recording_get_box_id(itemAsRecording()));
}

bool MOMScheduledRecording::manualBlock() const
{
    return mom_content_item_get_manual_block(itemAsItem());
}

void MOMScheduledRecording::setManualBlock(bool manualBlock)
{
    mom_content_item_set_manual_block(itemAsItem(), manualBlock);
    mom_recording_set_blocked(itemAsRecording(), manualBlock );
    mom_parentalcontrols_notify_changes( mom_parentalcontrols_update_BLOCK, itemAsItem() );
}

int MOMScheduledRecording::recordingPriority() const
{
    return mom_recording_get_priority(itemAsRecording());
}

void MOMScheduledRecording::setRecordingPriority(int recordingPriority, ExceptionCode& ec)
{
    mom_error err = mom_recording_set_priority(itemAsRecording(), recordingPriority);
    if (err == ERR_NOT_IMPLEMENTED)
        MOMException::setFromMOMError(err, ec);
}

bool MOMScheduledRecording::atRiskOfNotRecording() const
{
    return mom_recording_get_atriskofnotrecording(itemAsRecording());
}

bool MOMScheduledRecording::atRiskOfDeletion() const
{
    return mom_recording_get_atriskofdeletion(itemAsRecording());
}

bool MOMScheduledRecording::alwaysDelete() const
{
    return mom_recording_get_always_delete(itemAsRecording());
}

void MOMScheduledRecording::setAlwaysDelete(bool alwaysDelete)
{
    mom_recording_set_always_delete(itemAsRecording(), alwaysDelete);
}

int MOMScheduledRecording::seriesRecordedCount() const
{
    return mom_recording_get_series_recorded_count(itemAsRecording());
}

int MOMScheduledRecording::seriesScheduledCount() const
{
    return mom_recording_get_series_scheduled_count(itemAsRecording());
}

int MOMScheduledRecording::seriesConflictedCount() const
{
    return mom_recording_get_series_conflicted_count(itemAsRecording());
}

void MOMScheduledRecording::stop()
{
    mom_scheduler_stop(itemAsRecording());
}

void MOMScheduledRecording::start()
{
    mom_scheduler_start(itemAsRecording());
}

void MOMScheduledRecording::connect( MOMPlayer* player, int mode, int offset, ExceptionCode& ec )
{
    if ( mode >= mom_stream_connect_mode__MAX )
    {
        MOMException::setFromMOMError(mom_err_INVALID, ec);
        return;
    }

    mom_error err = mom_player_connect_content_item(player->player(), itemAsItem(), static_cast<mom_stream_connect_mode>(mode), offset);
    MOMException::setFromMOMError(err, ec);
}

void MOMScheduledRecording::connect( MOMPlayer* player, int mode, ExceptionCode& ec )
{
	return connect(player, mode, 0, ec);
}

void MOMScheduledRecording::connect( MOMPlayer* player, ExceptionCode& ec )
{
	return connect(player, 0, ec);
}

void MOMScheduledRecording::resolveConflict()
{
    mom_recording_resolve_conflict(itemAsRecording());
}

} // namespace WebCore

#endif // ENABLE(RMAI)
