/** @file MOMScheduledRecording.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMScheduledRecording_h
#define MOMScheduledRecording_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ExceptionCode.h"
#include "MOMProgramme.h"

namespace WebCore {

class MOMPlayer;
class MOMContentItem;
class MOMScheduledRecording;


class MOMScheduledRecording : public MOMProgramme {
public:
    static PassRefPtr<MOMScheduledRecording> createAdopt(ScriptExecutionContext *context, mom_scheduled_recording rec);
    static PassRefPtr<MOMScheduledRecording> create(ScriptExecutionContext *context, mom_scheduled_recording rec);
    MOMScheduledRecording(ScriptExecutionContext *context, mom_content_item item);

    String id() const;
    int recordingDuration() const;
    int recordingStartTime() const;
    int state() const;
    MOMContentItem* programme() const;
    bool isManual() const;
    bool isSeries() const;
    void setIsSeries(bool isSeries);

    PassRefPtr<MOMScheduledRecording> series() const;
    bool doNotDelete() const;
    void setDoNotDelete(bool doNotDelete);

    void setStartPadding(int startPadding);
    void setEndPadding(int endPadding);
    void setRepeatDays(int repeatDays);
    int repeatDays() const;
    void setSaveTime(int saveTime);
    int saveTime() const;
    void setSaveEpisodes(int saveEpisodes);
    int saveEpisodes() const;

    int viewed() const;
    String boxName() const;
    String boxId() const;
    bool manualBlock() const;
    void setManualBlock(bool manualBlock);

    int recordingPriority() const;
    void setRecordingPriority(int recordingPriority, ExceptionCode& ec);

    bool atRiskOfNotRecording() const;
    bool atRiskOfDeletion() const;
    bool alwaysDelete() const;
    void setAlwaysDelete(bool alwaysDelete);

    int seriesRecordedCount() const;
    int seriesScheduledCount() const;
    int seriesConflictedCount() const;

    void stop();
    void start();
    void connect( MOMPlayer* player, ExceptionCode& ec );
    void connect( MOMPlayer* player, int model, ExceptionCode& ec );
    void connect( MOMPlayer* player, int mode, int offset, ExceptionCode& ec );
    void resolveConflict();

    ALWAYS_INLINE mom_scheduled_recording itemAsRecording() const { return (mom_scheduled_recording)m_item; }
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMScheduledRecording_h
