/** @file MOMStream.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMStream_h
#define MOMStream_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "ActiveDOMObject.h"
#include "ExceptionCode.h"
#include "EventTarget.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_stream.h"
}

namespace WebCore {

class MOMPlayer;
class ScriptExecutionContext;

class MOMStream : public RefCounted<MOMStream>, public ActiveDOMObject, public EventTarget {
public:

    static PassRefPtr<MOMStream> create(ScriptExecutionContext* context, mom_stream stream );
    virtual ~MOMStream();

    void addMetaData( const String & key, const String & value );
    void connect( MOMPlayer* player, ExceptionCode& ec );
    void connect( MOMPlayer* player, int mode, ExceptionCode& ec );
    void connect( MOMPlayer* player, int mode, int offset, ExceptionCode& ec );

    ALWAYS_INLINE mom_stream stream() const { return m_stream; }

    // ActiveDOMObject interface
    bool hasPendingActivity() const /*OVERRIDE*/ { return const_cast<MOMStream*>(this)->hasEventListeners(); }
    void stop() /*OVERRIDE*/ {};
    virtual const char* activeDOMObjectName() const OVERRIDE { return "MOMStream"; }

    // EventTarget interface
    MOMStream* toMOMStream() { return this; }
    ScriptExecutionContext* scriptExecutionContext() const /*OVERRIDE*/ { return ActiveDOMObject::scriptExecutionContext(); }
    EventTargetData* eventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    EventTargetData* ensureEventTargetData() /*OVERRIDE*/ { return &m_eventTargetData; }
    void refEventTarget() /*OVERRIDE*/ { ref(); }
    void derefEventTarget() /*OVERRIDE*/ { deref(); }

    virtual EventTargetInterface eventTargetInterface() const;

    using RefCounted<MOMStream>::ref;
    using RefCounted<MOMStream>::deref;

protected:
    MOMStream(ScriptExecutionContext* context, mom_stream stream);

private:
    mom_stream m_stream;
    EventTargetData m_eventTargetData;
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMStream_h
