/** @file MOMTransportStream.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMTransportStream.h"
#include "MOMChannelScanOptions.h"
#include "MOMContentList.h"
#include "MOMChannelList.h"

#include "UtilsMOM.h"

namespace WebCore {

PassRefPtr<MOMTransportStream> MOMTransportStream::createAdopt(ScriptExecutionContext *context, mom_transportstream stream)
{
    return static_pointer_cast<MOMTransportStream>(MOMContentItem::createAdopt(context, (mom_content_item)stream));
}

PassRefPtr<MOMTransportStream> MOMTransportStream::create(ScriptExecutionContext *context, mom_transportstream stream)
{
    return static_pointer_cast<MOMTransportStream>(MOMContentItem::create(context, (mom_content_item)stream));
}

MOMTransportStream::MOMTransportStream(ScriptExecutionContext *context, mom_content_item item)
    : MOMContentItem(context, item)
{
}

int MOMTransportStream::frequencyMHz() const
{
    return mom_transportstream_frequency(itemAsTransportStream());
}

void MOMTransportStream::setFrequencyMHz(int frequencyMHz)
{
    mom_transportstream_frequency_set(itemAsTransportStream(), frequencyMHz);
}

int MOMTransportStream::polarisation() const
{
    return mom_transportstream_polarisation(itemAsTransportStream());
}

void MOMTransportStream::setPolarisation(int polarisation)
{
    mom_transportstream_polarisation_set(itemAsTransportStream(), polarisation);
}

int MOMTransportStream::symbolRateMBd() const
{
    return mom_transportstream_symbolRate(itemAsTransportStream());
}

void MOMTransportStream::setSymbolRateMBd(int symbolRateMBd)
{
    mom_transportstream_symbolRate_set(itemAsTransportStream(), symbolRateMBd);
}

int MOMTransportStream::modulation() const
{
    return mom_transportstream_modulation(itemAsTransportStream());
}

void MOMTransportStream::setModulation(int modulation)
{
    mom_transportstream_modulation_set(itemAsTransportStream(), modulation);
}

String MOMTransportStream::puncturingRate() const
{
    return UtilsMOM::StringFromLayersStringAndDispose(mom_transportstream_puncturingRate(itemAsTransportStream()));
}

void MOMTransportStream::setPuncturingRate(const String & puncturingRate)
{
    layers_string lrate = UtilsMOM::LayersStringFromString(puncturingRate);
    mom_transportstream_puncturingRate_set(itemAsTransportStream(), lrate);
    layers_string_dispose(lrate);
}

bool MOMTransportStream::pilot() const
{
    return mom_transportstream_pilot(itemAsTransportStream());
}

void MOMTransportStream::setPilot(bool pilot)
{
    mom_transportstream_pilot_set(itemAsTransportStream(), pilot);
}

bool MOMTransportStream::userCreated() const
{
    return mom_transportstream_userCreated(itemAsTransportStream());
}

PassRefPtr<MOMContentList> MOMTransportStream::channels() const
{
    return MOMChannelList::create(scriptExecutionContext(), mom_transportstream_channels(itemAsTransportStream()));
}

void MOMTransportStream::startScan( MOMChannelScanOptions* options )
{
    // unused
}

void MOMTransportStream::startScan()
{
	return startScan(0);
}

} // namespace WebCore

#endif // ENABLE(RMAI)
