/** @file MOMTransportStream.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef MOMTransportStream_h
#define MOMTransportStream_h

#if ENABLE(RMAI)

#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>
#include <wtf/text/AtomicString.h>

#include "MOMContentItem.h"
#include "MOMContentList.h"

extern "C" {
#include "layers.h"
#include "cdf/mom_transportstream.h"
}

namespace WebCore {

class MOMChannelScanOptions;
class MOMContentList;
class ScriptExecutionContext;

class MOMTransportStream : public MOMContentItem {
public:
    enum {
        MODULATION_DVBS1 = 0,
        MODULATION_DVBS2_QPSK = 1,
        MODULATION_DVBS2_8PSK = 2,
        POLARISATION_AUTO = 0,
        POLARISATION_H = 1,
        POLARISATION_V = 2,
    };

    static PassRefPtr<MOMTransportStream> createAdopt(ScriptExecutionContext *context, mom_transportstream stream);
    static PassRefPtr<MOMTransportStream> create(ScriptExecutionContext *context, mom_transportstream stream);
    MOMTransportStream(ScriptExecutionContext *context, mom_content_item item);

    int frequencyMHz() const;
    void setFrequencyMHz(int frequencyMHz);

    int polarisation() const;
    void setPolarisation(int polarisation);

    int symbolRateMBd() const;
    void setSymbolRateMBd(int symbolRateMBd);

    int modulation() const;
    void setModulation(int modulation);

    String puncturingRate() const;
    void setPuncturingRate(const String & puncturingRate);

    bool pilot() const;
    void setPilot(bool pilot);

    bool userCreated() const;
    PassRefPtr<MOMContentList> channels() const;

    void startScan();
    void startScan( MOMChannelScanOptions* options );

    ALWAYS_INLINE mom_transportstream itemAsTransportStream() const { return (mom_transportstream)m_item; }
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOMTransportStream_h
