/** @file MOMVODService.cpp
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#include "config.h"

#if ENABLE(RMAI)

#include "MOMVODAsset.h"
#include "MOMVODService.h"
#include "MOMPlayer.h"
#include "MOMStringCollection.h"

#include "MOMException.h"
#include "UtilsMOM.h"

namespace WebCore {

MOMVODService::MOMVODService(ScriptExecutionContext *context, mom_content_item item)
    : MOMContentPagedList(context, item)
{
}

bool MOMVODService::isBrowsable() const
{
    return mom_vod_service_is_browsable(itemAsVODService());
}

void MOMVODService::connect( MOMPlayer* player, int mode, ExceptionCode& ec )
{
    mom_stream_connect_mode smode = static_cast<mom_stream_connect_mode>(mode);

    /* default mode for service is preview, view is unsupported */
    if (smode == mom_stream_connect_mode_VIEW)
        smode = mom_stream_connect_mode_PREVIEW;

    if (smode >= mom_stream_connect_mode__MAX)
    {
        MOMException::setFromMOMError(mom_err_INVALID, ec);
        return;
    }

    mom_error err = mom_player_connect_content_item(player->player(), itemAsItem(), smode, 0);
    MOMException::setFromMOMError(err, ec);
}

void MOMVODService::connect( MOMPlayer* player, ExceptionCode& ec )
{
	return connect(player, 0, ec);
}

int MOMVODService::purchase( const String & pin, ExceptionCode& ec )
{
    layers_string lpin = UtilsMOM::LayersStringFromString(pin);
    mom_error err = mom_vod_service_purchase(itemAsVODService(), lpin);
    layers_string_dispose(lpin);
    if (err == mom_err_BUSY || err == mom_err_NOTAUTHORISED)
        MOMException::setFromMOMError(err, ec);
    return MOMException::codeFromMOMError(err);
}

void MOMVODService::cancel(ExceptionCode& ec)
{
    mom_error err = mom_vod_service_cancel(itemAsVODService());
    if (err == mom_err_BUSY || err == mom_err_NOTAUTHORISED)
        MOMException::setFromMOMError(err, ec);
}

MOMStringCollection* MOMVODService::lookupMetadata( const String & )
{
    // this is meant to return NULL, this function is defined but not implemented
    return NULL;
}

} // namespace WebCore

#endif // ENABLE(RMAI)
