/** @file UtilsMOM.h
 *
 * Copyright (C) Espial Limited 2014 - Company confidential - All rights reserved.
 *
 */

#ifndef UtilsMOM_h
#define UtilsMOM_h

#if ENABLE(RMAI)

#include <wtf/text/AtomicString.h>
#include "wtf/text/CString.h"
#include "ExceptionCode.h"

extern "C" {
#include "layers.h"
#include "layers_string.h"
#include "cdf/mom_core.h"
#include "cdf/mom_recording.h"
#include "cdf/mom_stream.h"
}

namespace WebCore {

inline layers_string LayersStringFromStringInline( const String & str )
{
    if (str.isNull())
        return NULL;
    CString cstr = str.utf8();
    return layers_string_create_counted(cstr.data(), cstr.length());
}

inline String StringFromLayersStringInline( layers_string lstr )
{
    if ( lstr == NULL )
        return String();
    return String::fromUTF8(layers_string_use(lstr), layers_string_length(lstr));
}

class UtilsMOM {
public:
    static layers_string LayersStringFromString( const String & str );
    static String StringFromLayersString( layers_string lstr );
    static String StringFromLayersStringAndDispose( layers_string lstr );

    static int codeFromRecordingState(mom_recording_state enumerator);
    static int codeFromRecordingState(int enumerator);
    static mom_recording_state recordingStateFromCode(int dom_state);

    static int codeFromStreamState(mom_stream_state enumerator);
    static int codeFromStreamState(int enumerator);

    static int codeFromRecordingAction(mom_recording_action enumerator);
    static int codeFromRecordingAction(int enumerator);
    static bool recordingActionIsAddedOrUpdated(mom_recording_action action);
    static bool recordingActionIsAddedOrUpdated(int action);
};

} // namespace WebCore

#endif // ENABLE(RMAI)

#endif // MOM_h
