/*
 * Copyright (C) 2008 Apple Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef Storage_h
#define Storage_h

#include "DOMWindowProperty.h"
#include "ScriptWrappable.h"
#include "StorageArea.h"
#include "StorageNamespaceImpl.h"
#include <wtf/Vector.h>
#include <wtf/Forward.h>
#include <wtf/RefCounted.h>
#include <wtf/RefPtr.h>
#include <wtf/ListHashSet.h>

namespace WebCore {

    class Frame;
    typedef int ExceptionCode;

    class Storage : public ScriptWrappable, public RefCounted<Storage>, public DOMWindowProperty {
    public:
		typedef ListHashSet<StorageArea*> LRUList;
		typedef pair<LRUList, LRUList> LRUMap;

		static LRUMap& totalLRUSet();

		static PassRefPtr<Storage> create(Frame*, PassRefPtr<StorageArea>);
        ~Storage();

        unsigned used() const;
        unsigned total() const;
        bool trusted() const;
		StorageType type() const;
        String url() const;

        unsigned length(ExceptionCode& ec) const { return m_storageArea->length(ec, m_frame); }
        String key(unsigned index, ExceptionCode& ec) const;
		String getItem(const String& key, ExceptionCode& ec) const;
        void setItem(const String& key, const String& value, ExceptionCode& ec);
        void removeItem(const String& key, ExceptionCode& ec);
        void clear(ExceptionCode& ec) { m_storageArea->clear(ec, m_frame); }
        bool contains(const String& key, ExceptionCode& ec) const;

		static void clearAll();
		static void clearIfNeeded(const LRUList& itLRU, const StorageNamespaceImpl::LocalStorageNamespace& itNamespace, unsigned totalQuota);
		static void clearDomain(const String& url);

        StorageArea* area() const { return m_storageArea.get(); }

    private:
        void totalLRUSetDelete() const;
        void totalLRUSetTop() const;
        bool totalLRUSetTrim() const;
        unsigned totalUsed() const;
	    static void clearAll(const LRUList& itLRU, const StorageNamespaceImpl::LocalStorageNamespace& itNamespace);

        Storage(Frame*, PassRefPtr<StorageArea>);

        RefPtr<StorageArea> m_storageArea;
    };

} // namespace WebCore

#endif // Storage_h
