/*
 * Copyright (C) Research In Motion Limited 2011. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "config.h"

#if ENABLE(SVG)
#include "SVGAttributeToPropertyMap.h"

#include "SVGAnimatedProperty.h"
#include <wtf/PassOwnPtr.h>

namespace WebCore {

void SVGAttributeToPropertyMap::addProperties(const SVGAttributeToPropertyMap& map)
{
    for (auto it = map.m_map.begin(); it != map.m_map.end(); ++it) {
        const PropertyInfoVector& vector = it->second;
        ASSERT(!vector.isEmpty());
        auto& properties = m_map.add(vector[0]->attributeName, PropertyInfoVector()).iterator->second;
        properties.reserveCapacity(properties.size() + vector.size());
        for (auto vectorIt = vector.begin(); vectorIt != vector.end(); ++vectorIt)
            properties.uncheckedAppend(*vectorIt);
    }
}

void SVGAttributeToPropertyMap::addProperty(const SVGPropertyInfo& info)
{
    m_map.add(info.attributeName, PropertyInfoVector()).iterator->second.append(&info);
}

void SVGAttributeToPropertyMap::addProperty(const SVGPropertyInfo* info)
{
	return addProperty(*info);
}

Vector<RefPtr<SVGAnimatedProperty>> SVGAttributeToPropertyMap::properties(SVGElement& contextElement, const QualifiedName& attributeName) const
{
    Vector<RefPtr<SVGAnimatedProperty>> properties;
    auto it = m_map.find(attributeName);
    if (it == m_map.end())
        return properties;
    const PropertyInfoVector& vector = it->second;
    properties.reserveInitialCapacity(vector.size());
    for (auto vectorIt = vector.begin(); vectorIt != vector.end(); ++vectorIt)
        properties.uncheckedAppend((*vectorIt)->lookupOrCreateWrapperForAnimatedProperty(&contextElement));
    return properties;
}

Vector<AnimatedPropertyType> SVGAttributeToPropertyMap::types(const QualifiedName& attributeName) const
{
    Vector<AnimatedPropertyType> types;
    auto it = m_map.find(attributeName);
    if (it == m_map.end())
        return types;
    const PropertyInfoVector& vector = it->second;
    types.reserveInitialCapacity(vector.size());
    for (auto vectorIt = vector.begin(); vectorIt != vector.end(); ++vectorIt)
        types.uncheckedAppend((*vectorIt)->animatedPropertyType);
    return types;
}

void SVGAttributeToPropertyMap::synchronizeProperties(SVGElement& contextElement) const
{
    for (auto it = m_map.begin(); it != m_map.end(); ++it) {
        const PropertyInfoVector& vector = it->second;
        for (auto vectorIt = vector.begin(); vectorIt != vector.end(); ++vectorIt)
        	(*vectorIt)->synchronizeProperty(&contextElement);
    } 
}

bool SVGAttributeToPropertyMap::synchronizeProperty(SVGElement& contextElement, const QualifiedName& attributeName) const
{
    auto it = m_map.find(attributeName);
    if (it == m_map.end())
        return false;
    const PropertyInfoVector& vector = it->second;
    for (auto vectorIt = vector.begin(); vectorIt != vector.end(); ++vectorIt)
    	(*vectorIt)->synchronizeProperty(&contextElement);
    return true;
}

}

#endif
