/*
 * Copyright (C) Research In Motion Limited 2010. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef SVGMatrixTearOff_h
#define SVGMatrixTearOff_h

#if ENABLE(SVG)

#include "SVGPropertyTearOff.h"
#include "SVGTransform.h"

namespace WebCore {

class SVGMatrixTearOff : public SVGPropertyTearOff<SVGMatrix> {
public:
    // Used for non-animated POD types that are not associated with a SVGAnimatedProperty object, nor with a XML DOM attribute
    // and that contain a parent type that's exposed to the bindings via a SVGStaticPropertyTearOff object
    // (for example: SVGTransform::matrix).
    static PassRefPtr<SVGMatrixTearOff> create(SVGPropertyTearOff<SVGTransform>& parent, SVGMatrix& value)
	{
		ASSERT(&parent.propertyReference().svgMatrix() == &value);
		RefPtr<SVGMatrixTearOff> result = adoptRef(new SVGMatrixTearOff(&parent));
		parent.addChild(result->m_weakPtrFactory.createWeakPtr(result.get()));
		return result.release();
	}

	virtual SVGMatrix& propertyReference() OVERRIDE { return m_parent->propertyReference().svgMatrix(); }

	virtual void setValue(SVGMatrix& value) OVERRIDE { m_parent->propertyReference().setMatrix(value); }

    virtual void commitChange()
    {
        m_parent->propertyReference().updateSVGMatrix();
        m_parent->commitChange();
    }

private:
    SVGMatrixTearOff(SVGPropertyTearOff<SVGTransform>* parent)
        : SVGPropertyTearOff<SVGMatrix>(0)
        , m_parent(parent)
    {
    }

    RefPtr<SVGPropertyTearOff<SVGTransform> > m_parent;
    WeakPtrFactory<SVGPropertyTearOffBase> m_weakPtrFactory;
};

}

#endif // ENABLE(SVG)
#endif // SVGMatrixTearOff_h
